/*
 *
 * File: IDeviceSubject.java
 * Date: Jan 26, 2007; 6:33:03 PM
 */
package de.desy.mst.libs.accelerator;

import java.io.Serializable;
import java.util.Collection;

import de.desy.mst.libs.accelerator.view.IViewComponent;

/**
 * <p>This is the subject element of the observer design pattern.</p>
 * <p>It will register the enumeration element with a list of components. Each 
 * registered component has all methods needed to set all relevant information.</p>
 * <p>Each observer element has two elements it stores in a private
 * class. The first element is the component itself and the second is a boolean
 * value, which provides the caller if this component should react on the
 * {@link #notify(IEData, EViewElement, Object)} method or not. By
 * registering an element with the methods {@link #register(IEData)}
 * or {@link #register(IEData, IViewComponent)} the corresponding component is 
 * enabled by the notify method (i.e. set to <tt>true</tt>). So each stored 
 * component reacts on the {@link #notify(IEData, EViewElement, Object)} method.</p>
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: IComponentSubject.java 36151 2010-05-25 13:51:55Z walla $
 * @see #notify(IEData, EViewElement, Object)
 * @see #register(IEData)
 * @see #register(IEData, IViewComponent)
 */
public interface IComponentSubject extends Serializable
{

	/**
	 * Provides the caller if the current data enumeration and the component
	 * itself is enabled or not.
	 *
	 * @param data_ Enumeration interface element.
	 * @param component_ Is the viewable component.
	 * @return Returns if the component is enabled (reacts on this component
	 * 		with the <tt>notify()</tt> method or not).
	 */
	public abstract boolean isRegisterEnabled(final IEData data_,
			final IViewComponent component_);

	/**
	 * Fetches the ignore flag of the specified data enumeration element. If the
	 * data element doesn't exist in the internal ignore map, then a
	 * {@code false} is returned.
	 * 
	 * @param data_ Data enumeration element.
	 * @return The ignoring flag.
	 */
	public abstract boolean isIgnored(final IEData data_);
	
	/**
	 * Sets the data enumeration and the component itself to be enabled or
	 * not depending on the <tt>boolean</tt> value. So, the boolean value provides
	 * the {@link #notify(IEData, EViewElement, Object)} method if the
	 * component should act on the method notify or not.
	 *
	 * @param data_ Is the enumeration interface of the data component.
	 * @param component_ Is the component to which to set the boolean value.
	 * @param enabled_ Is a boolean value, which tells the
	 * 		{@link #notify(IEData, EViewElement, Object)} method
	 * 		to get notice of the notification or not.
	 */
	public abstract void setRegisterEnabled(final IEData data_,
			final IViewComponent component_,	final boolean enabled_);

	/**
	 * Notifies all instance of the corresponding data element and executes
	 * the view element. For more information on this method you should have
	 * a look at {@link #notify(IEData, EViewElement, Object)}.
	 *
	 * @param data_ Is the enumeration object.
	 * @param view_ This is the view element on the screen.
	 * @see #notify(IEData, EViewElement, Object)
	 */
	public abstract void notify(final IEData data_, final EViewElement view_);

	/**
	 * <p>Will notify all component instances that the current object has changed.
	 * 
	 * <p>This method will look for a current enumeration element of
 	 * {@link EViewElement} which represents the
 	 * corresponding method name. If the enumeration has no such object an
 	 * {@code IllegalArgumentException} will be thrown at runtime. You are also
 	 * allowed to set primitive types, which are autoboxed to their wrapper 
 	 * classes.</p>
 	 * 
 	 * <p>The following example gives you a first impression on using this 
 	 * method. For example, you wanted to set the visibility of the enumeration 
 	 * of {@code ERF104.NEAR} to <tt>false</tt>. Here you can call this method 
 	 * with a boolean value which will be wrapped to an instance of <tt>Boolean</tt>:</p>
	 * 
	 * <pre>
	 * IDeviceSubject subject = new ...
	 * ...
	 * RFLabel label = new ...
	 * subject.register(ERF104.NEAR, label);
	 * ...
	 * subject.notify(ERF104.NEAR, EViewElement.VISIBEL, <b>false</b>);
	 * </pre>
	 *
	 * @param data_ Is the enumeration object.
	 * @param view_ This is the view element on the screen.
	 * @param object_ Is the object you wanted to set.
	 */
	public abstract void notify(final IEData data_,
			final EViewElement view_, final Object object_);

	/**
	 * Fetches all components of the specified enumeration. The returned 
	 * collection is not thread-safe and should be copied if you use this 
	 * collection in the furthermore eradication of your code, for example 
	 * {@link java.util.Collections#synchronizedCollection(Collection)} or by
	 * the unmodifiable collection of 
	 * {@link java.util.Collections#unmodifiableCollection(Collection)}.
	 *
	 * @param data_ Is the enumeration from which all components are returned.
	 * @return A collection of all components is returned. This collection can
	 * 		also have null elements inside of it. But there must be returned a
	 * 		collection instance.
	 */
	public abstract Collection<IViewComponent> fetchComponents(
			final IEData data_);

	/**
	 * Returns all observer enumeration objects as a collection. The returned 
	 * collection is not thread-safe and should be copied if you use this 
	 * collection in the furthermore eradication of your code, for example 
	 * {@link java.util.Collections#synchronizedCollection(Collection)} or by
	 * the unmodifiable collection of 
	 * {@link java.util.Collections#unmodifiableCollection(Collection)}.
	 *
	 * @return Returns all observer enumeration objects as a collection.
	 */
	public abstract Collection<IEData> fetchObservers();

	/**
	 * Returns if this data component is stored in the internal data value.
	 *
	 * @param data_ The data component you are asking for.
	 * @return A boolean value is returned: <tt>true</tt> if the corresponding
	 * 		data component is inside the internal data and <tt>false</tt>
	 * 		otherwise.
	 */
	public abstract boolean contains(final IEData data_);

	/**
	 * Registers the data element to the component subject instance.
	 *
	 * @param data_ Data element.
	 */
	public abstract void register(final IEData data_);

	/**
	 * This will register to an enumeration type a component. If the enumeration
	 * type is not stored it will store it and open a new list with the
	 * component as its first element.
	 *
	 * @param data_ Data element.
	 * @param component_ Is the component element. This should never be null
	 * 		otherwise an <tt>IllegalArgumentException<tt> is thrown.
	 */
	public abstract void register(final IEData data_,
			final IViewComponent component_);

	/**
	 * Removes the enumeration object and destroys all object which are stored
	 * under the enumeration. If the enumeration element is not stored in the
	 * internal map then nothing is done.
	 *
	 * @param data_ The destroyed data element.
	 */
	public abstract void remove(final IEData data_);
}
/* end of file */