/*
 *
 * File: AMonitor.java
 * Date: Dec 15, 2006; 9:31:51 AM
 */
package de.desy.mst.libs.accelerator.model;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;
import java.util.Set;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.IDeviceSubject;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEDataAddress;
import de.desy.mst.libs.accelerator.IEDeviceGroup;
import de.desy.mst.libs.accelerator.Utility;
import de.desy.mst.libs.accelerator.exception.TineException;
import de.desy.mst.libs.framework.mstapp.logging.LogLvl;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

import de.desy.tine.addrUtils.TSrvEntry;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;

/**
 * <p>This is the abstract monitor which allows to set and get the subject design
 * pattern of the observable. Moreover some methods are declared in this
 * class, most of these are for fetching the device names and links or
 * reporting the error (if one occurred).</p>
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: AMonitor.java 41195 2011-01-13 18:30:59Z walla $
 */
public abstract class AMonitor implements IMonitor
{
	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	private static final long serialVersionUID = 4720743595756663841L;

	private static final String CLAZZ = AMonitor.class.getSimpleName();
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	/** Is the subject interface which controls all observers of the observer
	 * design pattern. */
	protected static IDeviceSubject _subjects = null;

	/** The hash table which holds the link index and the link in key-value pair. */
	protected final Map<Integer, TLink> _htLinks =
		new HashMap<Integer, TLink>();

	/** The hash table which holds the link index and the corresponding device
	 * group of device context and server. */
	protected final Map<Integer, IEDeviceGroup> _mapDeviceGroup = 
		new HashMap<Integer, IEDeviceGroup>();;

	/** Creates a map with the the TINE access as a key element and the
	 * corresponding index in the link array as a value element. */
	protected final static Map<EAccess, Short> _htAccess =
		new Hashtable<EAccess, Short>();
	
	static
	{
		_htAccess.put(EAccess.READ, TAccess.CA_READ);
		_htAccess.put(EAccess.WRITE, TAccess.CA_WRITE);
	}

	/** Is the enumeration of devices. */
	protected final IEDeviceGroup _devices;

	/** Link data instance. */
	protected final ILinkData _linkData;
	
	// --- CLASS METHODS -------------------------------------------------------

	/**
	 * Reports the error message and prints it on the Java console.
	 *
	 * @param devices_ The device group.
	 * @param monitor_ The monitor instance.
	 * @param status_ TINE error status.
	 * @param data_ The current data enumeration element; can also be null.
	 */
	public final static void reportError(final IEDeviceGroup devices_,
			final IMonitor monitor_, final int status_, final IEData data_)
	{
		reportError(devices_, monitor_, status_, data_, false);
	}

	/**
	 * Reports the error message and prints it on the Java console.
	 *
	 * @param devices_ The device group.
	 * @param monitor_ The monitor instance.
	 * @param status_ TINE error status.
	 * @param data_ The current data enumeration element; can also be null.
	 * @param isPermanent_ Switches the output logging information. If this field
	 * 		is set to {@code false} then the output is controlled as a
	 * 		debug information and only appears as a transient information on the
	 * 		output. Otherwise by setting this field to {@code true} it is controlled 
	 * 		as an error output and therefore it also appears in the permanent 
	 * 		logging file.
	 */
	public final static void reportError(final IEDeviceGroup devices_,
			final IMonitor monitor_, final int status_, final IEData data_, 
			final boolean isPermanent_)
	{
		reportError(null, devices_, monitor_, status_, data_, isPermanent_);
	}
	
	/**
	 * Reports the error message and prints it on the Java console.
	 *
	 * @param linkData_ Link data instance to get more information about the back
	 * 		ground TINE link, i.e. its full TINE address, access, etc.
	 * @param devices_ The device group.
	 * @param monitor_ The monitor instance.
	 * @param status_ TINE error status.
	 * @param data_ The current data enumeration element; can also be null.
	 */
	public final static void reportError(final ILinkData linkData_,
			final IEDeviceGroup devices_,
			final IMonitor monitor_, final int status_, final IEData data_)
	{
		reportError(linkData_, devices_, monitor_, status_, data_, false);
	}
	
	/**
	 * Reports the error message and prints it on the Java console.
	 *
	 * @param linkData_ Link data instance to get more information about the back
	 * 		ground TINE link, i.e. its full TINE address, access, etc.
	 * @param devices_ The device group.
	 * @param monitor_ The monitor instance.
	 * @param status_ TINE error status.
	 * @param data_ The current data enumeration element; can also be null.
	 * @param isPermanent_ Switches the output logging information. If this field
	 * 		is set to {@code false} then the output is controlled as a
	 * 		debug information and only appears as a transient information on the
	 * 		output. Otherwise by setting this field to {@code true} it is controlled 
	 * 		as an error output and therefore it also appears in the permanent 
	 * 		logging file.
	 */
	public final static void reportError(final ILinkData linkData_,
			final IEDeviceGroup devices_,
			final IMonitor monitor_, final int status_, final IEData data_, 
			final boolean isPermanent_)
	{
		if (CGeneralConstant.ignoreTINEError(status_)
				|| monitor_ == null
				|| devices_ == null)
			return;
		
		final String CLAZZ = monitor_.getClass().getSimpleName();
		final String ERROR = (EErrorExt.getError(status_)==null?TErrorList.getErrorString(status_):EErrorExt.getError(status_).getError());
		
		if (linkData_ == null)
		{
			if (data_ == null)
				Logger.doLog(isPermanent_?LogLvl.error:LogLvl.debug, 
						Logger.SWTine, 
						String.format("%s - error on %s - status=%s (%d)", 
								CLAZZ, 
								devices_.toString(), 
								ERROR,
								status_), 
						null);
			else if (data_ instanceof IEDataAddress
					&& ((IEDataAddress) data_).crate() != -1
					&& ((IEDataAddress) data_).address() != -1)
				Logger.doLog(isPermanent_?LogLvl.error:LogLvl.debug, 
						Logger.SWTine, 
						String.format("%s - error on data=%s - status=%s" +
								" (%d) - SEDAC: crate=%d address=%d subaddress=%d", 
								CLAZZ, data_.toString(), ERROR ,status_, 
								((IEDataAddress) data_).crate(), 
								((IEDataAddress) data_).address(), 
								((IEDataAddress) data_).subaddress()), 
						null);
			else
				Logger.doLog(isPermanent_?LogLvl.error:LogLvl.debug, 
						Logger.SWTine, 
						String.format("%s - error on %s (data=%s) - status=%s (%d)", 
								CLAZZ, devices_.toString(), data_.toString(), ERROR ,status_), 
						null);
			
			return;
		}
		
		String LINK_INFO;
		try
		{
			if (linkData_.hasLink(devices_, EAccess.READ))
				LINK_INFO = linkData_.getLink(devices_, EAccess.READ).toString();
			else if (linkData_.hasLinkGroup(devices_, EAccess.READ))
				LINK_INFO = linkData_.getLinkGroup(devices_, EAccess.READ).values().iterator().next().toString()
					+ " [group]";
			else
				LINK_INFO = "[no info]";
		}
		catch (TineException exception)
		{
			exception.printStackTrace();
			LINK_INFO = "[exception: no info]";
		}
		
		if (data_ == null)
				Logger.doLog(isPermanent_?LogLvl.error:LogLvl.debug, 
						Logger.SWTine, 
						String.format("%s - error on %s (%s) - status=%s (%d)", 
								CLAZZ, 
								devices_.toString(), 
								LINK_INFO, 
								ERROR, 
								status_), 
						null);
		else if (data_ instanceof IEDataAddress
				&& ((IEDataAddress) data_).crate() != -1
				&& ((IEDataAddress) data_).address() != -1)
			Logger.doLog(isPermanent_?LogLvl.error:LogLvl.debug, 
					Logger.SWTine, 
					String.format("%s - error on data=%s (%s) - status=%s" +
							" (%d) - SEDAC: crate=%d address=%d subaddress=%d", 
							CLAZZ, 
							data_.toString(), 
							LINK_INFO, 
							ERROR ,
							status_, 
							((IEDataAddress) data_).crate(), 
							((IEDataAddress) data_).address(), 
							((IEDataAddress) data_).subaddress()), 
					null);
		else
			Logger.doLog(isPermanent_?LogLvl.error:LogLvl.debug, 
					Logger.SWTine, 
					String.format("%s - error on %s (data=%s, %s) - status=%s (%d)", 
							CLAZZ, 
							devices_.toString(), 
							data_.toString(), 
							LINK_INFO,
							ERROR,
							status_), 
					null);
	}
	
	// -------------------------------------------------------------------------

	/**
	 * <p>Returns the current subject of this monitor class. It can also return a
	 * {@code null} value as the subject is not set by the class.</p>
	 *
	 * @return The subject of the class is returned.
	 */
	protected static IDeviceSubject getSubject()
	{
		return AMonitor._subjects;
	}
	
	
	private static final String LINK_INFO_TLINK_HEADER_HTML = 
		"Die folgenden TINE links ...<br><br><ul>";
	private static final String LINK_INFO_TLINK_HEADER_CONSOLE = 
		"Die folgenden TINE links ...\n\n";
	private static final String LINK_INFO_TLINK_ENTITY_HTML = 
		"%s, Zustand %s (%d)<br>";
	private static final String LINK_INFO_TLINK_ENTITY_CONSOLE = 
		"%s - %s (%d)\n";
	private static final String LINK_INFO_TLINK_FOOTER_HTML = 
		"</ul>";
	private static final String LINK_INFO_TLINK_FOOTER_CONSOLE = 
		"";
	
	private static final String LINK_INFO_HOSTS_HEADER_HTML = 
		"<br>... werden diese Hosts und FEC-Namen angesprochen:<br><br><ul>";
	private static final String LINK_INFO_HOSTS_HEADER_CONSOLE = 
		"\n... werden diese Hosts angesprochen:\n\n";
	private static final String LINK_INFO_HOSTS_ENTITY_HTML = 
		"FEC-Name: %s, Host: %s (%s)<br>";
	private static final String LINK_INFO_HOSTS_ENTITY_CONSOLE = 
		"FEC-Name: %s, Host: %s (%s)\n";
	private static final String LINK_INFO_HOSTS_FOOTER_HTML = 
		"</ul>";
	private static final String LINK_INFO_HOSTS_FOOTER_CONSOLE = 
		"";
	
	/**
	 * Returns link information of the delivered TINE links. The returned String
	 * value is an HTML fragment.
	 * 
	 * @param links_ Array of all relevant TINE links. 
	 * @return String value with all information inside it.
	 */
	protected static String getLinkInfo(final TLink... links_)
	{
		return getLinkInfo(true, links_);
	}

	/**
	 * Returns link information of the delivered TINE links.
	 * 
	 * @param html_ The returned String value is an HTML fragment.
	 * @param links_ Array of all relevant TINE links. 
	 * @return String value with all information inside it.
	 */
	protected static String getLinkInfo(final boolean html_, final TLink... links_)
	{
		final StringBuilder builder = new StringBuilder();
		final StringBuilder builderHosts = new StringBuilder();
		final Set<TSrvEntry> servers = new HashSet<TSrvEntry>();
		
		builder.append(html_?LINK_INFO_TLINK_HEADER_HTML:LINK_INFO_TLINK_HEADER_CONSOLE);
		builderHosts.append(html_?LINK_INFO_HOSTS_HEADER_HTML:LINK_INFO_HOSTS_HEADER_CONSOLE);
		for(final TLink lnk: links_)
		{
			if (lnk.srvAddr != null && servers.contains(lnk.srvAddr))
			{
				servers.add(lnk.srvAddr);
				builderHosts.append(String.format(
						html_?LINK_INFO_HOSTS_ENTITY_HTML:LINK_INFO_HOSTS_ENTITY_CONSOLE, 
								lnk.srvAddr.getFecName(), 
								lnk.srvAddr.getFecAddr().fecHost.getHostName(),
								lnk.srvAddr.getFecAddr().fecHost.getHostAddress()));
			}
			builder.append(String.format(
					html_?LINK_INFO_TLINK_ENTITY_HTML:LINK_INFO_TLINK_ENTITY_CONSOLE,
							lnk.toString(),
							TErrorList.getErrorString(lnk.getLinkStatus()), 
							lnk.getLinkStatus()));
		}
		builder.append(html_?LINK_INFO_TLINK_FOOTER_HTML:LINK_INFO_TLINK_FOOTER_CONSOLE);
		builderHosts.append(html_?LINK_INFO_HOSTS_FOOTER_HTML:LINK_INFO_HOSTS_FOOTER_CONSOLE);

		return builder.toString() + builderHosts.toString();
	}
	
	// --- CONSTRUCTOR ---------------------------------------------------------
	
	/**
	 * <p>This is the abstract constructor of the class AMonitor. It will
	 * initialized the hash table of the links, and the device name.</p>
	 * 
	 * <p><b>DEPREACTED</b>: You should call {@link AMonitor#AMonitor(IEDeviceGroup, ILinkData)}!</p>
	 *
	 * @param devices_ Device group.
	 */
	@Deprecated
	protected AMonitor(final IEDeviceGroup devices_)
	{
		if(devices_ == null)
			throw new NullPointerException(CLAZZ + ": argument of IEDeviceGroup" +
					" is null.");

		_devices = devices_;
		_linkData = null;
		
		Logger.SWApp.warn(CLAZZ + ": THIS CONSTRUCTOR IS DEPRECATED. You should call AMonitor(IEDeviceGroup, ILinkData)");
		System.out.println("\tStacktrace:\n" + Utility.getCallingMethod(Thread.currentThread().getStackTrace()));
	}
	
	/**
	 * <p>This is the abstract constructor of the class AMonitor. It will
	 * initialized the hash table of the links, and the device name. More over
	 * it will also provide the TINE type and create a TINE link with the type
	 * information in it. The sorting of the device group is data elements is
	 * dismissed.</p>
	 *
	 * @param devices_ Device group.
	 * @param linkData_ Link data.
	 * @param name_ Name of the sub class.
	 */
	@Deprecated
	protected AMonitor(final IEDeviceGroup devices_,
			final ILinkData linkData_, String name_)
	{
		this(devices_);

		if (name_ == null)
			name_ = "** MONITOR NAME MISSING **";

		if (linkData_ != null)
		{
			TDataType output = new TDataType(_devices.getDataObjects());

			try
			{
				// ATTENTION: I wanted to create a new TINE link with the property
				// names listed as they are ordered in the data element list. So
				// here the value of the unsorted data elements is set to TRUE!
				linkData_.getLink(_devices, EAccess.READ, output, true);
			}
			catch(TineException exception)
			{
				Logger.SWApp.error(CLAZZ + " - initializing " + name_ + " failed - TINE error:" +
						TErrorList.getErrorString(exception.getErrorCode()) + " (" +
						exception.getErrorCode() + ')', exception);
			}
		}
		
		Logger.SWApp.debug(CLAZZ + " - successful passed this constructor");
	}

	/**
	 * <p>This is the abstract constructor of the class AMonitor. It will
	 * initialized the hash table of the links, and the device name.</p>
	 * 
	 * @param devices_ Device group.
	 * @param linkData_ Link data instance.
	 */
	protected AMonitor(final IEDeviceGroup devices_, final ILinkData linkData_)
	{
		if(devices_ == null)
			throw new NullPointerException(CLAZZ + ": argument of IEDeviceGroup" +
					" is null.");
		
		if (linkData_ == null)
			Logger.SWApp.warn(CLAZZ + " - argument of link data is null.");
		
		_devices = devices_;
		_linkData = linkData_;
		
		Logger.SWApp.debug(CLAZZ + " - successful passed this constructor");
	}

	// --- INSTANCE METHODS ----------------------------------------------------
	
	/**
	 * <p>This private method is used for putting the link index and the link in
	 * a hash table. The link can not be null otherwise a runtime exception is
	 * thrown.</p>
	 * <p>It first controls if there exists a connection between the link
	 * index and the link, and if not it will create a new one.</p>
	 *
	 * @param nLinkIndex_ Is the link index which is return by the {@code attach}
	 * 		method of the class {@code de.desy.mst.tine.client.TLink}.
	 * @param lnk_ Is the link from which the {@code attach} method was called.
	 */
	protected void putLink(final int nLinkIndex_, final TLink lnk_)
	{
		if (lnk_ == null)
			throw new IllegalArgumentException("The link is not set, ie it is " +
					" null.");

		if (!_htLinks.containsValue(lnk_) && !_htLinks.containsKey(nLinkIndex_))
			synchronized(_htLinks)
			{
				_htLinks.put(nLinkIndex_ , lnk_);
			}
	}

	/**
	 * <p>This method allows to put the combination of a link index and the
	 * corresponding device name into a hash table. So it is easy to look in the
	 * hash table from the corresponding device name which link index it is. There
	 * can be in the hash table only one link index but multiple device names.</p>
	 *
	 * @param nLinkIndex_ The link index which is returned by the method
	 * 		{@code attach} from the class {@code de.desy.tine.client.TLink}.
	 * @param deviceName_ Device name which belongs to the linked index.
	 * 		It is the device name of a context and its server from
	 * 		TINE. This is created from the common interface
	 * 		{@code IEDeviceGroup} which inherits by a concrete enumeration of
	 * 		{@code EDeviceName}
	 */
	protected void putDeviceName(final int nLinkIndex_,
			final IEDeviceGroup deviceName_)
	{
		if (!_mapDeviceGroup.containsValue(deviceName_) && !_mapDeviceGroup.containsKey(nLinkIndex_))
			synchronized(_htLinks)
			{
				_mapDeviceGroup.put(nLinkIndex_ , deviceName_);
			}
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.model.IMonitor#getDeviceGroup()
	 */
	public IEDeviceGroup getDeviceGroup()
	{
		return _devices;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.petra.temperature.monitor.IMonitor#fetchLink(int)
	 */
	public TLink fetchLink(final int nLinkIndex_) throws TineException
	{
		if (!_htLinks.containsKey(nLinkIndex_))
			throw new TineException(-1, "Could't find the TLink of the link index "
					+ nLinkIndex_);
		return _htLinks.get(nLinkIndex_);
	}


	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.petra.temperature.monitor.IMonitor#fetchProperty(int)
	 */
	public IEDeviceGroup fetchDeviceName(final int nLinkIndex_)
	{
		if (!_mapDeviceGroup.containsKey(nLinkIndex_))
			throw new IllegalArgumentException("Could't find the device name of "
					+ " link index " + nLinkIndex_);
		return _mapDeviceGroup.get(nLinkIndex_);
	}

	// - - LOG TINE STATUS - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	/**
	 * Logs all TINE links to the console output and logging file. The calling
	 * class name is here not set and will be omitted.
	 *  
	 * @param devices_ Device group.
	 */
	protected void logLinkStatus(final IEDeviceGroup devices_)
	{
		logLinkStatus(null, devices_);
	}

	/**
	 * Logs all TINE links to the console output and logging file.
	 *  
	 * @param CLAZZ_ Calling class name.
	 * @param devices_ Device group.
	 */
	protected void logLinkStatus(final String CLAZZ_, final IEDeviceGroup devices_)
	{
		if (_linkData == null)
		{
			if (CLAZZ_ != null && !CLAZZ_.isEmpty())
				Logger.SWApp.warn(CLAZZ_ + " - link data instance is not initialized.");
			else
				Logger.SWApp.warn(CLAZZ + " - link data instance is not initialized.");
			return;
		}
		
		try
		{
			if (_linkData.hasLink(devices_, EAccess.READ))
			{
				final TLink lnk = _linkData.getLink(devices_, EAccess.READ);
				if (CLAZZ_ != null && !CLAZZ_.isEmpty())
					Logger.SWTine.info(CLAZZ_ + " - established link " + lnk + " -> " + TErrorList.getErrorString(lnk.getLinkStatus()));
				else
					Logger.SWTine.info(CLAZZ + " established link " + lnk + " -> " + TErrorList.getErrorString(lnk.getLinkStatus()));
			}
			else if (_linkData.hasLinkGroup(devices_, EAccess.READ))
			{
				final Collection<TLink> links = 
					_linkData.getLinkGroup(devices_, EAccess.READ).values();
				Logger.SWTine.info(CLAZZ_ + " - collected link group:");
				
				if (CLAZZ_ != null && !CLAZZ_.isEmpty())
					for(final TLink lnk: links)
						Logger.SWTine.info(CLAZZ_ + " - established link " + lnk + " -> " + TErrorList.getErrorString(lnk.getLinkStatus()));
				else
					for(final TLink lnk: links)
						Logger.SWTine.info("Established link " + lnk + " -> " + TErrorList.getErrorString(lnk.getLinkStatus()));
			}
			else
				Logger.SWApp.warn(CLAZZ_ + " - no links defined on the devices " + _devices);
		}
		catch (TineException exception)
		{
			Logger.SWTine.error(CLAZZ + ": error on logging link information.", 
					exception);
		}
	}
	
	/**
	 * Logs all TINE links to the console output. The calling
	 * class name is here not set and will be omitted.
	 *  
	 * @param links_ TINE links.
	 */
	protected void logLinkStatus(final TLink... links_)
	{
		logLinkStatus(null, links_);
	}
	
	/**
	 * Logs all TINE links to the console output.
	 *  
	 * @param CLAZZ_ Calling class name.
	 * @param links_ TINE links.
	 */
	protected void logLinkStatus(final String CLAZZ_, final TLink... links_)
	{
		if (CLAZZ_ != null && !CLAZZ_.isEmpty())
			for(final TLink lnk: links_)
				Logger.SWTine.info(CLAZZ_ + " - established link " + lnk + " -> " + TErrorList.getErrorString(lnk.getLinkStatus()));
		else
			for(final TLink lnk: links_)
				Logger.SWTine.info(CLAZZ + " - established link " + lnk + " -> " + TErrorList.getErrorString(lnk.getLinkStatus()));
	}
	
	// - - CALCULATE RESULT - - - - - - - - - - - - - - - - - - - - - - - - - - 
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.model.AMonitor#calculate()
	 */
	public void calculate() throws TineException
	{
		if (_devices == null)
			throw new TineException(-1, "FATAL FAILURE! Should never happen" +
					" - device group is not initialized!");
		throw new TineException(-1, "FATAL FAILURE! Should never be" +
				" called: " + _devices.toString());
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.model.AMonitor#calculate()
	 */
	public void calculate(final IEData component_) throws TineException
	{
		if (_devices == null)
			throw new TineException(-1, "FATAL FAILURE! Should never happen" +
					" - device group is not initialized!");
		throw new TineException(-1, "FATAL FAILURE! Should never be" +
				" called: " + _devices.toString());
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return _devices.toString();
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(final Object object_)
	{
		if (object_ instanceof IMonitor)
			return ((IMonitor) object_).getDeviceGroup().toString().equals(
					_devices.toString());
		return super.equals(object_);
	}
}
/* end of file */