/*
 *
 * File: DataExchange.java
 * Date: Jul 3, 2007; 4:14:58 PM
 */
package de.desy.mst.libs.accelerator.model;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.EServer;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEDataSendDataType;
import de.desy.mst.libs.accelerator.IEDataSendNumber;
import de.desy.mst.libs.accelerator.exception.EnumerationNotUsedException;
import de.desy.mst.libs.accelerator.exception.TineException;
import de.desy.mst.libs.accelerator.exception.TinePathException;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.ser.tclientlib.ITineAccessor;
import de.desy.mst.libs.ser.tclientlib.TineLinkException;
import de.desy.mst.libs.ser.tclientlib.readers.DoubleReader;
import de.desy.mst.libs.ser.tclientlib.readers.FloatReader;
import de.desy.mst.libs.ser.tclientlib.readers.IntReader;
import de.desy.mst.libs.ser.tclientlib.writers.DoubleArrayWriter;
import de.desy.mst.libs.ser.tclientlib.writers.DoubleWriter;
import de.desy.mst.libs.ser.tclientlib.writers.FLTINTArrayWriter;
import de.desy.mst.libs.ser.tclientlib.writers.FLTINTWriter;
import de.desy.mst.libs.ser.tclientlib.writers.FloatArrayWriter;
import de.desy.mst.libs.ser.tclientlib.writers.FloatWriter;
import de.desy.mst.libs.ser.tclientlib.writers.INTINTArrayWriter;
import de.desy.mst.libs.ser.tclientlib.writers.INTINTWriter;
import de.desy.mst.libs.ser.tclientlib.writers.IntArrayWriter;
import de.desy.mst.libs.ser.tclientlib.writers.IntWriter;
import de.desy.mst.libs.ser.tclientlib.writers.NAME16IIArrayWriter;
import de.desy.mst.libs.ser.tclientlib.writers.NAME16IIWriter;
import de.desy.mst.libs.ser.tclientlib.writers.NAME32IArrayWriter;
import de.desy.mst.libs.ser.tclientlib.writers.NAME32IWriter;
import de.desy.mst.libs.ser.tclientlib.writers.ShortArrayWriter;
import de.desy.mst.libs.ser.tclientlib.writers.ShortWriter;
import de.desy.mst.libs.ser.tclientlib.writers.VoidWriter;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.types.FLTINT;
import de.desy.tine.types.INTINT;
import de.desy.tine.types.NAME16II;
import de.desy.tine.types.NAME32I;

/**
 * <p>This is the data exchange class which holds different static methods for
 * reading or writing different TINE and primitive types (here it is
 * <b>integer</b>,  <b>float</b> and <b>double</b>) via TINE and CDI calls.
 * It distinguishes between a normal TINE call to CDI by the method
 * {@link de.desy.mst.libs.accelerator.IEData#isCDI()}.</p>
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: DataExchange.java 39999 2010-10-26 12:59:22Z walla $
 */
public final class DataExchange
{
	public static final String CLAZZ = DataExchange.class.getSimpleName();

	/** Is the max. delay time before the static method returns from its
	 * execution. */
	public static final int MAX_DELAY_TIME = 10000;

	/**
	 * The following enumeration is for controlling the transaction through CDI.
	 * It validates if you wanted to send the data with the calibration of CDI
	 * or without.
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: DataExchange.java 39999 2010-10-26 12:59:22Z walla $
	 */
	public enum ESend
	{
		CDI_SEND_CLBR,
		CDI_SEND
	}

	/**
	 * The following enumeration is for controlling the transaction through CDI.
	 * It validates if you wanted to receive the data with the calibration of
	 * CDI or without.
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: DataExchange.java 39999 2010-10-26 12:59:22Z walla $
	 */
	public enum ERecv
	{
		CDI_RECV_CLBR,
		CDI_RECV
	}

	/**
	 * Sends one integer value via TINE to the TINE path of the corresponding
	 * device server. Returns by success of the connection with a 0 else the
	 * TINE error code is returned. The value is send to the CDI server with the
	 * property of {@code SEND.CLBR}!
	 *
	 * @param tinePath_ Is the TINE path of the device server.
	 * @param value_ Is the value you wanted to send.
	 * @return Returns the TINE success code. If it is unequal to zero an
	 * 		error has occurred.
	 */
	public final static int sendTineInt(final String tinePath_,
			final int value_)
	{
		int[] value = {value_};
		int result = TErrorList.success;
		final TDataType output = new TDataType(value);
		try
		{
			final TLink lnkWriter = new TLink(tinePath_ + '/' +
						ETineProperty.SEND.getName(EName.PROPERTY),
						null, output, TAccess.CA_WRITE);
			if (CGeneralConstant.isWriteArmed(EServer.CDI))
				result = lnkWriter.execute();
			lnkWriter.close();
		}
		catch(final RuntimeException exception)
		{
			Logger.SWApp.debug(CLAZZ + " - TLink exception: " + tinePath_ + '/' +
					ETineProperty.SEND.getName(EName.PROPERTY) +
					" with access WRITE doesn't exist.", exception);
		}
		return result;
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) nothing to the server.
	 * The data element is only used to get the corresponding device name of
	 * the TINE call. This method has no delay time and will return
	 * immediately.</p>
	 *
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param send_  This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final ESend send_)
		throws TineException
	{
		final VoidWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new VoidWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new VoidWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValue();
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}
		
		close(writer);
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) <b>NO</b> value to the
	 * server.
	 * The data element is only used to get the corresponding device name of
	 * the TINE call.</p>
	 * <p>This method will wait after sending the value via TINE. If the value
	 * is zero or lower it will return immediately after sending the value. If
	 * the delay time is beyond the value of {@link DataExchange#MAX_DELAY_TIME}
	 * then  it will wait for {@link DataExchange#MAX_DELAY_TIME} milliseconds.
	 * </p>
	 *
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_  This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_
			, final int delay_, final ESend send_)
		throws TineException
	{
		final VoidWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new VoidWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new VoidWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				// FIXME The following send statement should be corrected in
				// future versions!!!
				writer.setValue();
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}

		close(writer);
		
		if (delay_ <= 0)
			return;

		try
		{
			Thread.sleep(delay_ > MAX_DELAY_TIME ? 0 : delay_);
		}
		catch(InterruptedException exception)
		{
			Logger.SWApp.debug(CLAZZ + " - interrupting the sleep process.");
		}
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) <b>NO</b> value to the
	 * server.
	 * The data element is only used to get the corresponding device name of
	 * the TINE call.</p>
	 * <p>This method will wait after sending the value via TINE. If the value
	 * is zero or lower it will return immediately after sending the value. If
	 * the delay time is beyond the value of {@link DataExchange#MAX_DELAY_TIME}
	 * then  it will wait for {@link DataExchange#MAX_DELAY_TIME} milliseconds.
	 * </p>
	 *
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_  This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSendWithNumber(final IEData data_
			, final int delay_, final ESend send_)
		throws TineException
	{
		if (data_ == null)
			throw new NullPointerException("Data is null.");
		if (!(data_ instanceof IEDataSendNumber))
			throw new IllegalArgumentException("Data is not an instance of" +
					" IEDataSendNumber.");

		final TDataType input =
			new TDataType(((IEDataSendNumber) data_).sendNumber());
		final TLink link;
		int linkStatus;

		try
		{
			if (data_.isCDI())
			{
				if (!CGeneralConstant.isWriteArmed(EServer.CDI))
					return;

				link = new TLink(
						data_.getName(EName.PRE_TINEADDRESS) + '/' +
						data_.getName(EName.PROPERTY) + '/' +
						getTransactionSend(send_),
						null, input,
						TAccess.CA_WRITE);
			}
			else
			{
				if (!CGeneralConstant.isWriteArmed(EServer.TINE))
					return;

				link = new TLink(
						data_.getName(EName.PRE_TINEADDRESS) + '/' +
						data_.getName(EName.PROPERTY),
						null, input,
						TAccess.CA_WRITE);
			}
		}
		catch(final RuntimeException exception)
		{
			Logger.SWApp.error(CLAZZ + " - TLink exception: " +
					data_.getName(EName.PRE_TINEADDRESS) + '/' +
					data_.getName(EName.PROPERTY) +
					" with access WRITE doesn't exist.");
			throw new TineException(-1, data_.getName(EName.PRE_TINEADDRESS)
					+ '/' +	data_.getName(EName.PROPERTY) + " doesn't exist");

		}

		try
		{
			// FIXME The following send statement should be corrected in
			// future versions!!!
			linkStatus = link.execute((delay_ >= 100)?delay_:1000);
			link.close();
		}
		catch(RuntimeException exception)
		{
			throw new TineException(link != null ? link.getLinkStatus() : 0,
					"Catch a runtime exception: " + exception.getMessage(), exception);
		}
		
		if (linkStatus != 0)
			throw new TineException(linkStatus, "TINE error - " +
					TErrorList.getErrorString(linkStatus) + " (" + linkStatus +
					")");
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) value to the server
	 * which are stored in the {@link IEData} parameters. These send
	 * data type is of the form {@code TDataType} form the TINE framework.</p>
	 * <p>This method will wait after sending the value via TINE. If the value
	 * is zero or lower it will return immediately after sending the value. If
	 * the delay time is beyond the value of {@link DataExchange#MAX_DELAY_TIME}
	 * then  it will wait for {@link DataExchange#MAX_DELAY_TIME} milliseconds.
	 * </p>
	 *
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_  This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSendWithDataType(final IEData data_
			, final int delay_, final ESend send_)
		throws TineException
	{
		if (data_ == null)
			throw new NullPointerException("Data is null.");
		if (!(data_ instanceof IEDataSendDataType))
			throw new IllegalArgumentException("Data is not an instance of" +
					" IEDataSendDataType.");

		final TDataType input = ((IEDataSendDataType) data_).sendDataType();
		final TLink link;
		int linkStatus;

		try
		{
			if (data_.isCDI())
			{
				if (!CGeneralConstant.isWriteArmed(EServer.CDI))
					return;

				link = new TLink(
						data_.getName(EName.PRE_TINEADDRESS) + '/' +
						data_.getName(EName.PROPERTY) + '/' +
						getTransactionSend(send_),
						null, input,
						TAccess.CA_WRITE);
			}
			else
			{
				if (!CGeneralConstant.isWriteArmed(EServer.TINE))
					return;

				link = new TLink(
						data_.getName(EName.PRE_TINEADDRESS) + '/' +
						data_.getName(EName.PROPERTY),
						null, input,
						TAccess.CA_WRITE);
			}
		}
		catch(final RuntimeException exception)
		{
			Logger.SWApp.error(CLAZZ + " - TLink exception: " +
					data_.getName(EName.PRE_TINEADDRESS) + '/' +
					data_.getName(EName.PROPERTY) +
					" with access WRITE doesn't exist.");
			throw new TineException(-1, data_.getName(EName.PRE_TINEADDRESS)
					+ '/' +	data_.getName(EName.PROPERTY) + " doesn't exist");

		}

		try
		{
			// FIXME The following send statement should be corrected in
			// future versions!!!
			linkStatus = link.execute((delay_ >= 100)?delay_:1000);
			link.close();
		}
		catch(RuntimeException exception)
		{
			throw new TineException(link != null?link.getLinkStatus():0, 
					"Catch a runtime exception: " + exception.getMessage(), exception);
		}

		if (linkStatus != 0)
			throw new TineException(linkStatus, "TINE error - " +
					TErrorList.getErrorString(linkStatus) + " (" + linkStatus +
					")");
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>integer</b> value of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * <p>This method will wait after sending the value via TINE. A value of
	 * zero or lower it will return immediately. If the delay time is beyond
	 * the value of {@link DataExchange#MAX_DELAY_TIME} then it will wait
	 * for {@link DataExchange#MAX_DELAY_TIME} milliseconds.</p>
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param value_ Is the <b>integer</b> value to be send.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_  This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final int value_, final int delay_, final ESend send_)
		throws TineException
	{
		if (value_ < 0 && data_ == null)
			throw new IllegalArgumentException("The value or the data element"
					+ " are invalid.");

		final IntWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new IntWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new IntWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValue(value_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}

		close(writer);
		
		if (delay_ <= 0)
			return;

		try
		{
			Thread.sleep(delay_ > MAX_DELAY_TIME ? 0 : delay_);
		}
		catch(InterruptedException exception)
		{
			Logger.SWApp.warn(CLAZZ + " - interrupting the sleep process.");
		}
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>short</b> value of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * <p>This method will wait after sending the value via TINE. A value of
	 * zero or lower it will return immediately. If the delay time is beyond
	 * the value of {@link DataExchange#MAX_DELAY_TIME} then it will wait
	 * for {@link DataExchange#MAX_DELAY_TIME} milliseconds.</p>
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param value_ Is the <b>short</b> value to be send.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_  This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final short value_, final int delay_, final ESend send_)
		throws TineException
	{
		if (value_ < 0 && data_ == null)
			throw new IllegalArgumentException("The value or the data element"
					+ " are invalid.");

		final ShortWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new ShortWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new ShortWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValue(value_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}

		close(writer);
		
		if (delay_ <= 0)
			return;

		try
		{
			Thread.sleep(delay_ > MAX_DELAY_TIME ? 0 : delay_);
		}
		catch(InterruptedException exception)
		{
			Logger.SWApp.warn(CLAZZ + " - interrupting the sleep process.");
		}
		
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>short</b> values via the TINE Client library. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param send_  This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @param values_ Is the <b>short</b> array value to be send.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final ESend send_, final short... values_)
		throws TineException
	{
		if (values_.length == 0)
			throw new IllegalArgumentException("Short values have a length of zero.");
		if (data_ == null)
			throw new IllegalArgumentException("Data element is invalid.");

		final ShortArrayWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new ShortArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.DEVICENAME) + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new ShortArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValues(values_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}
		
		close(writer);
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>integer</b> values via the TINE Client library. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param send_  This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @param values_ Is the <b>integer</b> array value to be send.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final ESend send_, final int... values_)
		throws TineException
	{
		if (values_.length == 0)
			throw new IllegalArgumentException("Integer values have a length of zero.");
		if (data_ == null)
			throw new IllegalArgumentException("Data element is invalid.");

		final IntArrayWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new IntArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.DEVICENAME) + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new IntArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValues(values_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}
		
		close(writer);
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>integer</b> value of the corresponding data element. The sending
	 * is done with enumeration of {@code ESend}. These enumeration tells the
	 * send method if it should use calibration or not. The data element is
	 * only used to get the corresponding device name of the TINE call.</p>
	 * <p>This method will wait after sending the value via TINE. A value of
	 * zero or lower it will return immediately. If the
	 * delay time is beyond the value of {@link DataExchange#MAX_DELAY_TIME} then
	 * it will wait for {@link DataExchange#MAX_DELAY_TIME} milliseconds.</p>
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param value_ Is the <b>integer</b> value to be send.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_ This is an enumeration of {@code ESend} which includes if
	 * 		the data should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSendCDI(final IEData data_,
			final int value_, final int delay_, final ESend send_)
		throws TineException
	{
		if (value_ < 0 && data_ == null)
			throw new IllegalArgumentException("The value or the data element"
					+ " are invalid.");

		final IntWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;


			writer = new IntWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
			throw new TinePathException(-1,
					"The TINE path is not a CDI path: "
					+ data_.getName(EName.PRE_TINEADDRESS));
		try
		{
			if (!writer.hasError())
				writer.setValue(value_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}

		close(writer);
		
		if (delay_ <= 0)
			return;

		try
		{
			Thread.sleep(delay_ > MAX_DELAY_TIME ? 0 : delay_);
		}
		catch(InterruptedException exception)
		{
			Logger.SWApp.warn(CLAZZ + " - interrupting the sleep process.");
		}
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current TINE data
	 * <b>INTINT</b> value of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * <p>This method will wait after sending the value via TINE. A value of
	 * zero or lower it will return immediately.  If the
	 * delay time is beyond the value of {@link DataExchange#MAX_DELAY_TIME} then
	 * it will wait for {@link DataExchange#MAX_DELAY_TIME} milliseconds.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param value_ Is the TINE <b>INTINT</b> value to be send.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_ This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final INTINT value_, final int delay_, final ESend send_)
		throws TineException
	{
		if (value_.i1val < 0 && data_ == null)
			throw new IllegalArgumentException("The value or the data element"
					+ " are invalid.");

		final INTINTWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new INTINTWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new INTINTWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValue(value_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}

		close(writer);
		
		if (delay_ <= 0)
			return;

		try
		{
			Thread.sleep(delay_ > MAX_DELAY_TIME ? 0 : delay_);
		}
		catch(InterruptedException exception)
		{
			Logger.SWApp.warn(CLAZZ + " - interrupting the sleep process.");
		}
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current TINE data
	 * <b>INTINT</b> values of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param send_ This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @param values_ Is the TINE <b>INTINT</b> array value to be send.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final ESend send_, final INTINT... values_)
		throws TineException
	{
		if (values_.length == 0)
			throw new IllegalArgumentException("The length of the INTINT" +
					" array is null.");
		if (data_ == null)
			throw new NullPointerException("The data element is invalid.");

		final INTINTArrayWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new INTINTArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new INTINTArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValues(values_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}
		
		close(writer);
	}
	
	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>float</b> value of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * <p>This method will wait after sending the value via TINE. A value of
	 * zero or lower it will return immediately after sending the value. If the
	 * delay time is beyond the value of {@link DataExchange#MAX_DELAY_TIME} then
	 * it will wait for {@link DataExchange#MAX_DELAY_TIME} milliseconds.</p>
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param value_ Is the <b>float</b> value to be send.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_ This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final float value_, final int delay_, final ESend send_)
		throws TineException
	{
		if (value_ < 0 && data_ == null)
			throw new NullPointerException("The value or the data element" +
					" are invalid.");

		final FloatWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new FloatWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new FloatWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValue(value_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}
		
		close(writer);

		if (delay_ <= 0)
			return;

		try
		{
			Thread.sleep(delay_ > MAX_DELAY_TIME ? 0 : delay_);
		}
		catch(InterruptedException exception)
		{
			Logger.SWApp.warn(CLAZZ + " - interrupting the sleep process.");
		}
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>float</b> values of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param send_ This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @param values_ Is the <b>float</b> array value to be send.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			 final ESend send_, final float... values_)
		throws TineException
	{
		if (values_.length == 0)
			throw new IllegalArgumentException("The length of the send float" +
					" values is null.");
		if (data_ == null)
			throw new NullPointerException("The data element is null.");

		final FloatArrayWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new FloatArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new FloatArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValues(values_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}
		
		close(writer);
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current TINE data
	 * <b>FLTINT</b> value of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * <p>This method will wait after sending the value via TINE. A value of
	 * zero or lower it will return immediately after sending the value. If the
	 * delay time is beyond the value of {@link DataExchange#MAX_DELAY_TIME} then
	 * it will wait for {@link DataExchange#MAX_DELAY_TIME} milliseconds.</p>
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param value_ Is the TINE <b>FLTINT</b> value to be send.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_ This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final FLTINT value_, final int delay_, final ESend send_)
		throws TineException
	{
		if (data_ == null)
			throw new NullPointerException("The data element is invalid.");

		final FLTINTWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new FLTINTWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new FLTINTWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValue(value_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}

		close(writer);
		
		if (delay_ <= 0)
			return;

		try
		{
			Thread.sleep(delay_ > MAX_DELAY_TIME ? 0 : delay_);
		}
		catch(InterruptedException exception)
		{
			Logger.SWApp.warn(CLAZZ + " - interrupting the sleep process.");
		}
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current TINE data
	 * <b>FLTINT</b> values of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param send_ This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @param values_ Is the TINE <b>FLTINT</b> array value to be send.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final ESend send_, final FLTINT... values_)
		throws TineException
	{
		if (values_.length == 0)
			throw new IllegalArgumentException("The length of the FLTINT" +
					" array is null.");
		if (data_ == null)
			throw new NullPointerException("The data element is invalid.");

		final FLTINTArrayWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new FLTINTArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new FLTINTArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValues(values_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}
		
		close(writer);
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>double</b> value of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * <p>This method will wait after sending the value via TINE. A value of
	 * zero or lower it will return immediately after sending the value. If the
	 * delay time is beyond the value of {@link DataExchange#MAX_DELAY_TIME} then
	 * it will wait for {@link DataExchange#MAX_DELAY_TIME} milliseconds.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param value_ Is the <b>double</b> value to be send.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_ This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final double value_, final int delay_, final ESend send_)
		throws TineException
	{
		if (value_ < 0 && data_ == null)
			throw new NullPointerException("The value or the data element" +
					" are invalid.");

		final DoubleWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new DoubleWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new DoubleWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValue(value_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}

		close(writer);
		
		if (delay_ <= 0)
			return;

		try
		{
			Thread.sleep(delay_ > MAX_DELAY_TIME ? 0 : delay_);
		}
		catch(InterruptedException exception)
		{
			Logger.SWApp.warn(CLAZZ + " - interrupting the sleep process.");
		}
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>double</b> values of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param send_ This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @param values_ Is the <b>double</b> array value to be send.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final ESend send_, final double... values_)
		throws TineException
	{
		if (values_.length == 0)
			throw new IllegalArgumentException("The length of the double" +
					" array is null.");
		if (data_ == null)
			throw new NullPointerException("The data element is invalid.");

		final DoubleArrayWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new DoubleArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new DoubleArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValues(values_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}
		
		close(writer);
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>NAME16II</b> value of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * <p>This method will wait after sending the value via TINE. A value of
	 * zero or lower it will return immediately after sending the value. If the
	 * delay time is beyond the value of {@link DataExchange#MAX_DELAY_TIME} then
	 * it will wait for {@link DataExchange#MAX_DELAY_TIME} milliseconds.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param value_ Is the <b>NAME16II</b> value to be send.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_ This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final NAME16II value_, final int delay_, final ESend send_)
		throws TineException
	{
		if (value_ == null && data_ == null)
			throw new NullPointerException("The value or the data element" +
					" are invalid.");

		final NAME16IIWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new NAME16IIWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new NAME16IIWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValue(value_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}

		close(writer);
		
		if (delay_ <= 0)
			return;

		try
		{
			Thread.sleep(delay_ > MAX_DELAY_TIME ? 0 : delay_);
		}
		catch(InterruptedException exception)
		{
			Logger.SWApp.warn(CLAZZ + " - interrupting the sleep process.");
		}
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>NAME16II</b> values of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param send_ This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @param values_ Is the <b>NAME16II</b> array value to be send.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final ESend send_, final NAME16II... values_)
		throws TineException
	{
		if (values_.length == 0)
			throw new IllegalArgumentException("The length of the NAME16II" +
					" values is null.");
		if (data_ == null)
			throw new NullPointerException("The value or the data element" +
					" are invalid.");

		final NAME16IIArrayWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new NAME16IIArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new NAME16IIArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValues(values_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}
		
		close(writer);
	}
	
	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>NAME32I</b> value of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * <p>This method will wait after sending the value via TINE. A value of
	 * zero or lower it will return immediately. If the delay time is beyond
	 * the value of {@link DataExchange#MAX_DELAY_TIME} then it will wait
	 * for {@link DataExchange#MAX_DELAY_TIME} milliseconds.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param value_ Is the <b>NAME32I</b> value to be send.
	 * @param delay_ The waiting time after the value is send.
	 * @param send_  This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final NAME32I value_, final int delay_, final ESend send_)
		throws TineException
	{
		if (value_.ival < 0 && data_ == null)
			throw new IllegalArgumentException("The value or the data element"
					+ " is invalid.");

		final NAME32IWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new NAME32IWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new NAME32IWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValue(value_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}

		close(writer);
		
		if (delay_ <= 0)
			return;

		try
		{
			Thread.sleep(delay_ > MAX_DELAY_TIME ? 0 : delay_);
		}
		catch(InterruptedException exception)
		{
			Logger.SWApp.warn(CLAZZ + " - interrupting the sleep process.");
		}
	}

	/**
	 * <p>Fires an event to TINE, which will write (send) the current data
	 * <b>NAME32I</b> values of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 * 
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param send_  This is an enumeration of {@code ESend}. Its contents tells
	 * 		this method if the data value should be calibrated or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#READ} is used.
	 * @param values_ Is the <b>NAME32I</b> array value to be send.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static void fireEventSend(final IEData data_,
			final ESend send_, final NAME32I... values_)
		throws TineException
	{
		if (values_.length == 0)
			throw new IllegalArgumentException("The length of the NAME32I" +
					" values is null.");
			if (data_ == null)
			throw new NullPointerException("The data element is invalid.");

		final NAME32IArrayWriter writer;

		if (data_.isCDI())
		{
			if (!CGeneralConstant.isWriteArmed(EServer.CDI))
				return;

			writer = new NAME32IArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionSend(send_));
		}
		else
		{
			if (!CGeneralConstant.isWriteArmed(EServer.TINE))
				return;

			writer = new NAME32IArrayWriter(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!writer.hasError())
				writer.setValues(values_);
			else
				throwTineException(writer, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(writer, data_);
		}
		
		close(writer);
	}
	
	/**
	 * <p>Fires an event over TINE, which will read (receive) the current data
	 * <b>integer</b> value of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 *
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param receive_ This is the enumeration which tells this method if it
	 * 		should use the calibration or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#WRITE} is used.
	 * @return Is the returned <b>integer</b> value.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static int fireEventRecvINT(final IEData data_,
			final ERecv receive_)
		throws TineException
	{
		if (data_ == null)
			throw new NullPointerException("The value or the data element" +
					" are invalid.");

		final IntReader reader;

		int value = 0;

		if (data_.isCDI())
		{
			reader = new IntReader(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionRecv(receive_));
		}
		else
		{
			reader = new IntReader(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!reader.hasError())
				value = reader.getValue();
			else
				throwTineException(reader, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(reader, data_);
		}

		close(reader);
		
		return value;
	}

	/**
	 * <p>Fires an event to TINE, which will read (receive) the current
	 * <b>float</b> value of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 *
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param receive_ This is the enumeration which tells this method if it
	 * 		should use the calibration or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#WRITE} is used.
	 * @return Is the returned <b>float</b> value.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static float fireEventRecvFLT(final IEData data_,
			final ERecv receive_)
		throws TineException
	{
		if (data_ == null)
			throw new NullPointerException("The value or the data element" +
					" are invalid.");

		final FloatReader reader;

		float value = 0.0f;

		if (data_.isCDI())
		{
			reader = new FloatReader(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionRecv(receive_));
		}
		else
		{
			reader = new FloatReader(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!reader.hasError())
				value = reader.getValue();
			else
				throwTineException(reader, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(reader, data_);
		}
		
		close(reader);

		return value;
	}

	/**
	 * <p>Fires an event to TINE, which will read (receive) the current data
	 * <b>double</b> value of the corresponding data element. The data
	 * element is only used to get the corresponding device name of the TINE
	 * call.</p>
	 *
	 * @param data_ Is the current data element which will be send via TINE.
	 * @param receive_  This is the enumeration which tells this method if it
	 * 		should use the calibration or not. If this
	 * 		argument is {@code null} and you execute the data element on a CDI
	 * 		server the element {@link ETineProperty#WRITE} is used.
	 * @return Is the returned <b>double</b> value.
	 * @throws TineException Is thrown if an error occurred and installing or
	 * 		executing the synchronous TINE link failed.
	 */
	public final static double fireEventRecvDBL(final IEData data_,
			final ERecv receive_)
		throws TineException
	{
		if (data_ == null)
			throw new NullPointerException("The data element is invalid.");

		final DoubleReader reader;

		double value = 0.0;

		if (data_.isCDI())
		{
			reader = new DoubleReader(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.toString() + '/' + getTransactionRecv(receive_));
		}
		else
		{
			reader = new DoubleReader(data_.getName(EName.PRE_TINEADDRESS) + '/'
					+ data_.getName(EName.PROPERTY));
		}

		try
		{
			if (!reader.hasError())
				value = reader.getValue();
			else
				throwTineException(reader, null);
		}
		catch(TineLinkException exception)
		{
			throwTineException(reader, data_);
		}
		
		close(reader);

		return value;
	}

	/**
	 * Close the TINE link.
	 * 
	 * @param assessor_ TINE assessor.
	 */
	private static void close(final ITineAccessor assessor_)
	{
		if (assessor_ != null)
			assessor_.close();
	}
	
	/**
	 * Throws a TINE exception depending on the delivered arguments.
	 *
	 * @param accessor_ Is the {@code ITineAccessor} from the library
	 * 		{@code TClientLib}. This can also be {@code null}.
	 * @param data_ Is the data element. This can also be {@code null}.
	 * @throws TineException This exception is thrown by this method independent
	 * 		from the delivered arguments.
	 */
	private static void throwTineException(final ITineAccessor accessor_,
			final IEData data_)
		throws TineException
	{
		close(accessor_);
		
		if (accessor_ != null && data_ != null)
			throw new TineException(accessor_.getLinkStatus(),
					"An error occurred on executing the" +
					" TINE link of data element " + data_.toString() + ": "
					+ accessor_.getErrorMessage()
					+ " (" + accessor_.getLinkStatus() + ")");
		else if (accessor_ != null)
			throw new TineException(accessor_.getLinkStatus(),
					"An error occurred on initializing the" +
					" TINE link: " + accessor_.getErrorMessage());
		else if (data_ != null)
			throw new TineException(-1, "An error occurred on executing the" +
					" TINE link of data element " + data_.toString() + ".");
		else
			throw new TineException(-1, "An error occurred on executing the" +
					" TINE link.");
	}

	/**
	 * Gets the corresponding string value of the property of a call to a
	 * CDI server.
	 *
	 * @param send_ The enumeration value of the send command. This determines
	 * 		if you use the calibration function or not. If this
	 * 		argument is {@code null} the element {@code WRITE} of the
	 * 		enumeration {@link ETineProperty} is returned.
	 * @return Returns the corresponding property value in the TINE jargon.
	 */
	private static String getTransactionSend(final ESend send_)
	{
		if (send_ == null)
			return ETineProperty.WRITE.getName(EName.PROPERTY);

		switch(send_)
		{
		case CDI_SEND_CLBR:
			return ETineProperty.SEND.getName(EName.PROPERTY);
		case CDI_SEND:
			return ETineProperty.WRITE.getName(EName.PROPERTY);
		default:
			throw new EnumerationNotUsedException("The enumeration of "
					+ send_.toString() + " is not used.");
		}
	}

	/**
	 * Gets the corresponding string value of the property of a call to a
	 * CDI server.
	 *
	 * @param receive_ The enumeration value of the receive command. This
	 * 		determines if you use the calibration function or not. If this
	 * 		argument is {@code null} the element {@code READ} of the
	 * 		enumeration {@link ETineProperty} is returned.
	 * @return Returns the corresponding property value in the TINE jargon.
	 */
	private static String getTransactionRecv(final ERecv receive_)
	{
		if (receive_ == null)
			return ETineProperty.READ.getName(EName.PROPERTY);

		switch(receive_)
		{
		case CDI_RECV_CLBR:
			return ETineProperty.RECEIVE.getName(EName.PROPERTY);
		case CDI_RECV:
			return ETineProperty.READ.getName(EName.PROPERTY);
		default:
			throw new EnumerationNotUsedException("The enumeration of "
					+ receive_.toString() + " is not used.");
		}
	}

}
/* end of file */