/*
 *
 * File: EAccess.java
 * Date: Dec 15, 2006; 12:36:09 PM
 */
package de.desy.mst.libs.accelerator.model;

import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.IEEnum;
import de.desy.mst.libs.accelerator.exception.EnumerationNotUsedException;
import de.desy.tine.definitions.TAccess;

/**
 * This enumeration describes the access over a network connection via TINE.
 *
 * <table cellpadding="4" cellspacing="4">
 * 
 * <tr bgcolor="#E0E0E0"><th>{@link EAccess} instance</th><th>Description</th></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #READ}</td><td>This will read information from the server or hardware.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #WRITE}</td><td>This will write information to the server or hardware.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #TCP}</td><td>This is focused on a TCP connection.</td></tr>
 * 
 * </table>
 * 
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: EAccess.java 34783 2010-04-01 16:20:12Z walla $
 */
public enum EAccess implements IEEnum 
{
	READ ("This will read information from the server or hardware."
			, TAccess.CA_READ),
	WRITE ("This will write information to the server or hardware."
			, TAccess.CA_WRITE),
	TCP   ("This is focused on a TCP connection.", TAccess.CA_CONNECT);
	
	/** A description of the corresponding network access. */
	private final String _description;
	
	/** Is the TINE access. */
	private final short _tineAccess;
	
	/**
	 * The private constructor of a TINE access instance.  
	 * 
	 * @param description_ A description.
	 * @param tineAccess_ The TINE access.
	 */
	private EAccess(final String description_, short tineAccess_)
	{
		validate(description_);
		_description = description_;
		_tineAccess = tineAccess_;
	}
	
	/**
	 * Validates all fields.
	 * 
	 * @param description_ A description.
	 */
	private void validate(final String description_)
	{
		if (description_ == null)
			throw new NullPointerException("The description is null.");
		if (description_.isEmpty())
			throw new IllegalArgumentException("The description should be empty");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IEEnum#description()
	 */
	public String description()
	{
		return _description;
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IEEnum#id()
	 */
	public String id()
	{
		return this.toString();
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IEEnum#getName(de.desy.mst.libs.accelerator.EName)
	 */
	public String getName(final EName description_) 
		throws EnumerationNotUsedException
	{
		switch(description_)
		{
		case NAME:
		case NAME_TABLE:
		case NAME_CAVITY:
			return this.toString();
		case DESCRIPTION:
			return _description;
		default:
			throw new EnumerationNotUsedException("The enumeration indeed" +
					" exists but is not used: " + description_.toString());
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.IEEnum#putName(de.desy.mst.libs.accelerator.EName, java.lang.String)
	 */
	public void putName(final EName description_, final String name_)
	{
		throw new EnumerationNotUsedException("No enumeration is used in this" +
				" enumeration.");
	}
	
	/**
	 * This will return a boolean value. Every entry of the enumeration also 
	 * gets the corresponding TINE access codes. If only one is accessible in 
	 * the TINE access code a {@code true} is returned otherwise a {@code false}. 
	 * The TINE values are take from the class called 
	 * {@link de.desy.tine.definitions.TAccess}.
	 *
	 * @param accesses_ Is an array of TINE values of the type {@code TAccess}.
	 * @return Returns the corresponding boolean value if the access is accessible
	 * 		through the TINE access or not.
	 */
	public boolean isTineAccess(short... accesses_)
	{	
		for(short currentAccess: accesses_)
			if ( (_tineAccess & currentAccess) == currentAccess)
				return true;
		return false;
	}
	
	/**
	 * Returns the corresponding TINE access. This can be controlled by operating
	 * every value with a bitwise AND operator. If every condition returns with a
	 * {@code true} the hole querying will return {@code true} otherwise 
	 * {@code false}.
	 * 
	 * @return Returns the TINE access code of the corresponding enumeration 
	 * 		entry.
	 */
	public short tineAccess()
	{
		return _tineAccess;
	}
}
/* end of file */