/*
 *
 * File: ICDIDevice.java
 * Date: Aug 24, 2007; 4:54:01 PM
 */
package de.desy.mst.libs.accelerator.model;

import java.util.Collection;

import de.desy.mst.libs.accelerator.IEExpectList;
import de.desy.mst.libs.accelerator.exception.EnumerationNotFoundException;
import de.desy.mst.libs.accelerator.exception.TineException;

/**
 * This is the interface for holding information of all relevant CDI device 
 * servers. It will represent all methods for getting all devices and there 
 * properties (number and name etc.). 
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: ICDIDevice.java 53477 2013-01-25 15:34:42Z walla $
 */
public interface ICDIDevice {	
	
	/** The method {@link #setTINETimeout(int)} allows to set a time out for
	 * requesting meta information from a server. This is the lower time out limit. */
	public static final int LOWER_LIMIT_REQUEST_FOR_TIMEOUT = 1000;
	/** The method {@link #setTINETimeout(int)} allows to set a time out for
	 * requesting meta information from a server. This is the upper time out limit. */
	public static final int UPPER_LIMIT_REQUEST_FOR_TIMEOUT = 10000;
	
	/**
	 * The application model. This can be one part of the traditionally 
	 * client-server model.
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: ICDIDevice.java 53477 2013-01-25 15:34:42Z walla $
	 */
	public enum EApplicationModel { CLIENT, SERVER };

	/**
	 * Allows to set for requesting data from a TINE server a time out. If the
	 * requesting link exceeds this time limit a {@link TineException} is thrown
	 * with the error code 45 (link timeout).
	 *  
	 * @param timeout Time out in milliseconds should be between 1000 up to 
	 * 		10000 ms. If it exceeds one of these limit is forced to the 
	 * 		corresponding limit.
	 */
	public void setTINETimeout(final int timeout);

	/**
	 * Returns the time out limit for requesting data from a TINE server.
	 *  
	 * @return timeout Time out in milliseconds.
	 */
	public int getTINETimeout();
	
	/**
	 * This method is <b>deprecated</b>. You should use the method {@link #checkEnumElements(EApplicationModel)}
	 * instead.
	 * <p>By default checks the <b>server</b> application model.</p>
	 *  
	 * @throws TineException Throws a <tt>TineException</tt> if the TINE 
	 * 		connection failed for any reasons.
	 * @throws EnumerationNotFoundException If one or more elements does not
	 * 		exits or are misspelled.
	 * @see #checkEnumElements(EApplicationModel)
	 */
	@Deprecated
	public abstract void checkEnumElements() throws TineException,
			EnumerationNotFoundException;
	
	/**
	 * <p>All enumeration elements in the delivered collection of the
	 * constructor will be checked if they exists for the corresponding CDI device 
	 * server element in the table. If not an {@code EnumerationNotFoundException}
	 * will be thrown. It will get all elements from CDI and set them to the
	 * corresponding device name.</p>
	 *  
	 * @param model_ Devides the two possible application model for initializing
	 * 		the application. Internally this method will call the refresh method
	 * 		which reflects via a synchronous TINE call the meta information
	 * 		from the server, which depends on the application model (for example
	 * 		the client would be interested in the unit of a value where the
	 * 		server would be interested in the range of a value). 
	 * @throws TineException Throws a <tt>TineException</tt> if the TINE 
	 * 		connection failed for any reasons.
	 * @throws EnumerationNotFoundException If one or more elements does not
	 * 		exits or are misspelled.
	 */
	public abstract void checkEnumElements(
			final EApplicationModel model_) 
		throws TineException, EnumerationNotFoundException;
	
	/**
	 * This method is <b>deprecated</b>. You should use the method {@link #checkEnumElements(EApplicationModel, String)}
	 * instead.
	 * <p>By default checks the <b>server</b> application model.</p>
	 *  
	 * @param tineAddress_ Preliminary TINE address (i.e. the TINE context and
	 * 		server name) allows to specify the
	 * 		CDI server name to be checked. This can also be null, which means
	 * 		to check all elements of all CDI servers. If this string value
	 * 		is set and the corresponding CDI server doesn't exist then a
	 * 		{@code IllegalArgumentException} is thrown.  
	 * @throws TineException Throws a <tt>TineException</tt> if the TINE 
	 * 		connection failed for any reasons.
	 * @throws EnumerationNotFoundException If one or more elements does not
	 * 		exits or are misspelled.
	 * @see #checkEnumElements(EApplicationModel, String)
	 */
	@Deprecated
	public abstract void checkEnumElements(final String tineAddress_) 
		throws TineException, EnumerationNotFoundException;

	/**
	 * <p>All enumeration elements in the delivered collection of the
	 * constructor will be checked if they exists the corresponding CDI device 
	 * server element in the table. If not an {@code EnumerationNotFoundException}
	 * will be thrown. It will get all elements from CDI and set them to the
	 * corresponding device name.</p>
	 *  
	 * @param model_ Devides the two possible application model for initializing
	 * 		the application. Internally this method will call the refresh method
	 * 		which reflects via a synchronous TINE call the meta information
	 * 		from the server, which depends on the application model (for example
	 * 		the client would be interested in the unit of a value where the
	 * 		server would be interested in the range of a value). 
	 * @param tineAddress_ Preliminary TINE address (i.e. the TINE context and
	 * 		server name) allows to specify the
	 * 		CDI server name to be checked. This can also be null, which means
	 * 		to check all elements of all CDI servers. If this string value
	 * 		is set and the corresponding CDI server doesn't exist then a
	 * 		{@code IllegalArgumentException} is thrown.  
	 * @throws TineException Throws a <tt>TineException</tt> if the TINE 
	 * 		connection failed for any reasons.
	 * @throws EnumerationNotFoundException If one or more elements does not
	 * 		exits or are misspelled.
	 */
	public abstract void checkEnumElements(
			final EApplicationModel model_,
			final String tineAddress_)
		throws TineException, EnumerationNotFoundException;
	
	
	/**
	 * This method is <b>deprecated</b>. You should use the method {@link #checkEnumElements(EApplicationModel, String, boolean, boolean)}
	 * instead.
	 *  
	 * @param tineAddress_ Preliminary TINE address (i.e. the TINE context and
	 * 		server name) allows to specify the
	 * 		CDI server name to be checked. This can also be null, which means
	 * 		to check all elements of all CDI servers. If this string value
	 * 		is set and the corresponding CDI server doesn't exist then a
	 * 		{@code IllegalArgumentException} is thrown.  
	 * @param checkEverytime_ Checks if this method executes every time you call
	 *  	this method. If you use <tt>true</tt> independent of
	 *  	every other field, it will try to get all device 
	 *  	information over TINE, otherwise it will do nothing. 
	 * @param initAllDevices_ Checks if all device elements should be initialized
	 * 		(<tt>true</tt>) or not (<tt>false</tt>). Is this boolean value 
	 * 		{@code false} then only the device names and numbers are initialize. If
	 * 		it set to {@code true} will initialize all device elements by getting the device
	 * 		names, device numbers, units, precision, minimum and max values (range),
	 * 		the addresses (crate and sub-address), and the description.
	 * @throws TineException Throws a <tt>TineException</tt> if the TINE 
	 * 		connection failed for any reasons.
	 * @throws EnumerationNotFoundException If one or more elements does not
	 * 		exits or are misspelled.
	 * @see #checkEnumElements(EApplicationModel, String, boolean, boolean)
	 */
	@Deprecated
	public abstract void checkEnumElements(final String tineAddress_,
			final boolean checkEverytime_, final boolean initAllDevices_) 
		throws TineException, EnumerationNotFoundException;

	/**
	 * All enumeration elements in the delivered collection of the
	 * constructor will be checked if they exists for the corresponding CDI device 
	 * server element in the table. If not an {@code EnumerationNotFoundException}
	 * is thrown.
	 *  
	 * @param model_ Devides the two possible application model for initializing
	 * 		the application. Internally this method will call the refresh method
	 * 		which reflects via a synchronous TINE call the meta information
	 * 		from the server, which depends on the application model (for example
	 * 		the client would be interested in the unit of a value where the
	 * 		server would be interested in the range of a value). 
	 * @param tineAddress_ Preliminary TINE address (i.e. the TINE context and
	 * 		server name) allows to specify the
	 * 		CDI server name to be checked. This can also be null, which means
	 * 		to check all elements of all CDI servers. If this string value
	 * 		is set and the corresponding CDI server doesn't exist then a
	 * 		{@code IllegalArgumentException} is thrown.  
	 * @param checkEverytime_ Checks if this method executes every time you call
	 *  	this method. If you use <tt>true</tt> independent of
	 *  	every other field, it will try to get all device 
	 *  	information over TINE, otherwise it will do nothing. 
	 * @param initAllDevices_ Checks if all device elements should be initialized
	 * 		(<tt>true</tt>) or not (<tt>false</tt>). Is this boolean value is
	 * 		{@code false} then only the device names and numbers are initialize. When
	 * 		it is {@code true} it will initialize all device elements by getting the device
	 * 		names, device numbers, units, precision, minimum and max values (range),
	 * 		the addresses (crate and sub-address), and the description.
	 * @throws TineException Throws a <tt>TineException</tt> if the TINE 
	 * 		connection failed for any reasons.
	 * @throws EnumerationNotFoundException If one or more elements does not
	 * 		exits or are misspelled.
	 */
	public abstract void checkEnumElements(
			final EApplicationModel model_, 
			final String tineAddress_,
			final boolean checkEverytime_, 
			final boolean initAllDevices_) 
		throws TineException, EnumerationNotFoundException;

	/**
	 * Fetches the current device by its current CDI number.
	 * 
	 * @param tineAddress_ Is the preliminary TINE address string.
	 * @param deviceNumber_ Is the current CDI device number.
	 * @return Returns a string value as its result. If the current CDI number
	 * 		is not in the current map a {@code null} value can also be returned. 
	 */
	public abstract String fetchDevice(final String tineAddress_,
			final int deviceNumber_);

	/**
	 * Fetches all device names and returns them as a collection.
	 * 
	 * @param tineAddress_ The preliminary TINE address context is needed to
	 * 		get all devices.
	 * @return Returns a collection of all device names as a collection. 
	 */
	public abstract Collection<String> fetchDevices(final String tineAddress_);

	/**
	 * Fetches the current device by its current CDI name and returns its number.
	 * 
	 * @param tineAddress_ Is the preliminary TINE address string.
	 * @param deviceName_ Is the current CDI device name.
	 * @return Returns an integer value as its result. If the current CDI name
	 * 		is not in the current map a -1 is yield. 
	 */
	public abstract int fetchNumber(final String tineAddress_, 
			final String deviceName_);

	/**
	 * Returns the number of all devices.
	 * 
	 * @return Returns the number of devices.
	 */
	public abstract int fetchNumberOfDevices();

	/**
	 * Returns the executing interface.
	 * 
	 * @param exert_ Is the exerted value.
	 * @return Returns an instance of the executing object.
	 */
	@Deprecated
	public abstract IExecute getExec(final IEExpectList exert_);
	
}
/* end of file */