/*
 *
 * File: IMonitor.java
 * Date: Sep 28, 2006; 10:17:21 AM
 */
package de.desy.mst.libs.accelerator.model;

import java.io.Serializable;

import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEDeviceGroup;
import de.desy.mst.libs.accelerator.exception.TineException;
import de.desy.tine.client.TLink;
import de.desy.tine.types.TCompoundDataObject;

/**
 * <p>This is the java interface of monitoring all hardware values. It will watch
 * all corresponding values whichever this interface will get from the hardware by
 * pulling it. An asynchronous link will return all values in pretend time. 
 * On the other hand a synchronous link will return all values only once.
 * This interface is for <b>asynchronous</b> links only.</p>
 *  
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: IMonitor.java 12444 2008-06-09 07:49:03Z walla $
 */
public interface IMonitor extends Serializable
{

	/**
	 * Returns the current device group enumeration which is used to created 
	 * the monitor.
	 * 
	 * @return The current device group is returned.
	 */
	public abstract IEDeviceGroup getDeviceGroup();
	
	/**
	 * Returns the current device name enumeration of the give link index. 
	 * Internally it will examine a hash table and look if there exists a key
	 * value of the corresponding link index. If yes it will return the 
	 * interface (which inherits the corresponding enumeration) otherwise an 
	 * exception is thrown.
	 * 
	 * @param nLinkIndex_ Is the link index value from the {@code attach} method
	 * 		of the class {@code de.desy.tine.client.TLink}.
	 * @return Returns the interface of the device name. If it doesn't exist
	 * 		the runtime exception {@code IllegalArgumentException} is thrown.
	 */
	public abstract IEDeviceGroup fetchDeviceName(final int nLinkIndex_);
	
	/**
	 * Will fetch the current link index that correspond to the link. If there
	 * does'nt exist such a link then a {@code null} value is returned. Will look
	 * in the internal hash table. In this associated table it will fetch the 
	 * link and link index and return the stored value of the key-value pair.
	 * 
	 * @param nLinkIndex_ Is the link index value from the {@code attach} method
	 * 		of the class {@code de.desy.tine.client.TLink}.
	 * @return Returns the corresponding {@code TLink} instance.
	 * @throws TineException A TINE exception is thrown if the current link
	 * 		does not exist.
	 */
	public abstract TLink fetchLink(final int nLinkIndex_) throws TineException;
	
	/**
	 * Attache a monitor to an asynchronous TINE link. 
	 *
	 * @throws TineException A TINE exception is thrown if an error occurred
	 * 		during getting information or developing of information from TINE.
	 */
	public abstract void attach() throws TineException;
	
	/**
	 * <p>Returns the data of the initialized device group directly from the 
	 * TINE protocol. The returned information can also be invalid if you call 
	 * it a few milliseconds after creating this instance; it is better to
	 * wait for up to 100 milliseconds before executing this method.</p>
	 * <p>Currently I return here a {@code Collection} element with the generic 
	 * type of {@code TCompoundDataObject}. It is a composite type of primitive 
	 * data which TINE supports.</p>
	 * 
	 * @return Returns a list of the current device name. This can also be 
	 * 		invalid if you call it a few milliseconds after creating it.
	 */
//	@SuppressWarnings("unchecked")
	public abstract TCompoundDataObject[] getData();

	/**
	 * <p>In most cases this method will read the data from a buffered
	 * storage and not over a network connection by TINE.</p> 
	 * 
	 * @param deviceGroup_ Is the interface of the device group from which you
	 * 		want to resolve the data.
	 * @return Returns a collection of the calculated result values.
	 */
//	public abstract Collection<TCompoundDataObject> fetchResult();
	
	/**
	 * <p>Will calculate the result from its current value list. This is only 
	 * responsible for a valid result if all links are opened first.</p>
	 * <p>Moreover it will throw a runtime exception if the argument of the subject
	 * is not set. The subject is part of the observer design pattern. It uses
	 * this to inform all observers which are stored in the subject.
	 * There exist two class method in the class {@code AMonitor} which will set
	 * and get the subject. See at link {@code AMonitor.setSubject(IDeviceSubject)}
	 * and {@code AMonitor.getSubject()}</p>
	 * 
	 * @throws TineException A TINE exception is thrown if an error occurred
	 * 		during getting information or developing of information from TINE.
	 */
	public abstract void calculate() throws TineException;
	
	/**
	 * <p>This method will calculate the delivered data component and set
	 * internally their values. These values can therefore be read by the
	 * {@code IMonitor.fetchResult(IEDeviceGroup)} or direct from the TINE
	 * data type parameter via {@code IMonitor.fetchData(IEDeviceGroup)}.</p>
	 * 
	 * @param data_ Is the enumeration interface of the data component.
	 * @throws TineException Returns a <tt>TineException</tt> if an error 
	 * 		occurred.
	 */
	public abstract void calculate(final IEData data_) throws TineException;

	/**
	 * <p>Sets the error vehicle (both, on the GUI device or on the console). 
	 * Over given is the link status from the TINE call-back function (includes
	 * the error code) and the data component.</p>
	 * 
	 * @param linkStatus_ Is the link status from the TINE call-back method.
	 * @param data_ Is the enumeration interface to which the error belongs.
	 */
	public abstract void showError(final int linkStatus_, 
			final IEData data_);
//
//	/**
//	 * <p>Sets the error vehicle (both, on the GUI device or on the console). 
//	 * Over given is the link status from the TINE call-back function (includes
//	 * the error code).</p>
//	 * 
//	 * @param linkStatus_ Is the link status from the TINE call-back method.
//	 */
//	public abstract void showError(int linkStatus_);
}
/* end of file */