/*
 *
 * File: ECavityElement.java
 * Date: Jan 27, 2008; 5:29:31 PM
 */
package de.desy.mst.libs.accelerator.model.cavity;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.exception.EnumerationNotUsedException;


/**
 * <p>This is the standard cavity elements, which nearly every cavity application
 * should include in their GUI interface. The following arguments can be selected
 * in this enumeration:</p>
 *
 * <table cellpadding="4" cellspacing="4">
 * <tr><th>Enumeration {@link ECavityElement}</th><th>Description</th></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#EINKOPPLER_TEMPERATURE}</td><td>Einkoppler temperature (Grad).</td></tr>
 * <tr bgcolor="A0A0A0"><td>{@link ECavityElement#EINKOPPLER_DELTA_TEMPERATURE}</td><td>Einkoppler temperature difference (Grad).</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#CAVITY_VOLTAGE}</td><td>Cavity voltage (volt).</td></tr>
 * <tr bgcolor="A0A0A0"><td>{@link ECavityElement#CAVITY_VOLTAGE_PHASE}</td><td>Cavity voltage, phase (degree).</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#PHASE_DIFFERENCE}</td><td>Phase difference between U cavity - U pre voltage (degree).</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#STAMP_1_WEGGEBER}</td><td>Stamp 1 position (millimeters).</td></tr>
 * <tr bgcolor="A0A0A0"><td>{@link ECavityElement#STAMP_1_BOUND}</td><td>Stamp 1 has reached its minimum or maximum position (bit value). In other words: has reached Fmin or Fmax.</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#STAMP_1_FMIN}</td><td>Stamp 1 has reached its minimal position (boolean value). In other words: has reached FMin.</td></tr>
 * <tr bgcolor="A0A0A0"><td>{@link ECavityElement#STAMP_1_FMAX}</td><td>Stamp 1 has reached its maximum position (boolean value). In other words: has reached FMax.</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#STAMP_2_WEGGEBER}</td><td>Stamp 2 position (millimeters).</td></tr>
 * <tr bgcolor="A0A0A0"><td>{@link ECavityElement#STAMP_2_BOUND}</td><td>Stamp 2 has reached its minimum or maximum position (bit value). In other words: has reached Fmin or Fmax.</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#STAMP_2_FMIN}</td><td>Stamp 2 has reached its minimal position (boolean value). In other words: has reached FMin.</td></tr>
 * <tr bgcolor="A0A0A0"><td>{@link ECavityElement#STAMP_2_FMAX}</td><td>Stamp 2 has reached its maximum position (boolean value). In other words: has reached FMax.</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#PRE_POWER}</td><td>Pre power (watt).</td></tr>
 * <tr bgcolor="A0A0A0"><td>{@link ECavityElement#PRE_PHASE}</td><td>Pre phase (degree).</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#POST_POWER}</td><td>Post power (watt).</td></tr>
 * <tr bgcolor="A0A0A0"><td>{@link ECavityElement#POST_PHASE}</td><td>Post phase (degree).</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#CELL_2_TEMPERATURE}</td><td>Cell 2, temperature (celsius).</td></tr>
 * <tr bgcolor="A0A0A0"><td>{@link ECavityElement#CELL_2_VOLTAGE}</td><td>Cell 2, voltage (volt).</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#CELL_4_TEMPERATURE}</td><td>Cell 4, temperature (celsius).</td></tr>
 * <tr bgcolor="A0A0A0"><td>{@link ECavityElement#CELL_4_VOLTAGE}</td><td>Cell 4, voltage (volt).</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link ECavityElement#CELL_6_TEMPERATURE}</td><td>Cell 6, temperature (celsius).</td></tr>
 * <tr bgcolor="A0A0A0"><td>{@link ECavityElement#CELL_6_VOLTAGE}</td><td>Cell 2, voltage (volt).</td></tr>
 * </table>
 *
 * Returns ({@link IEData#getName(EName)}) or sets ({@link IEData#putName(EName,String)})
 * on the {@link EName}:
 *
 * <table cellpadding="4" cellspacing="4">
 * <tr><th>Enumeration {@link EName}</th><th>Description</th></tr>
 * <tr bgcolor="A0A0A0"><td>{@link EName#NAME}</td><td>Name of the enumeration element without HTML styled tags.</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link EName#NAME_CAVITY}</td><td>A HTML styled enumeration element, here the cavity value.</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link EName#NAME_TABLE}</td><td>A HTML styled enumeration element, here the table value.</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link EName#UNIT}</td><td>Returns the unit.</td></tr>
 * <tr bgcolor="E0E0E0"><td>{@link EName#PRECISION}</td><td>A C-like styled precision string. See <a href="http://java.sun.com/javase/6/docs/api/java/util/Formatter.html#syntax">here</a> for more information.</td></tr>
 * </table>
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: ECavityElement.java 25572 2009-05-19 15:31:07Z walla $
 */
public enum ECavityElement implements IECavityElement
{

	/** Temperature EINKOPPLER. */
	EINKOPPLER_TEMPERATURE(
			"Einkoppler Temperature RL", "T<sub>Einkoppler</sub>", "T<sub>Einkoppler</sub>",
			"C"  , "% 9.1f")
	,
	/** Temperature difference of EINKOPPLER.*/
	EINKOPPLER_DELTA_TEMPERATURE(
			"Einkoppler Delta Temperature", "&Delta;T<sub>Einkoppler</sub>", "&Delta;T<sub>Einkoppler RL</sub>",
			"C", "% 9.1f"
			)
	,
	/** Cavity voltage (volt). */
	CAVITY_VOLTAGE(
			"Cavityspannung", "U<sub>Cavity</sub>", "U<sub>Cavity</sub>",
			"kV"  , "% 9.1f")
	,
	/** Cavity voltage, phase (degree).*/
	CAVITY_VOLTAGE_PHASE(
			"Cavityspannung Phase", "&phi;<sub>U Cavity</sub>", "&phi;<sub>U Cavity</sub>",
			"Grad", "% 9.1f"
			)
	,
	/** Phase difference between U cavity - U pre voltage (degree).*/
	PHASE_DIFFERENCE(
			"Phase Cavity", "&Delta;&phi;<sub>Cavity</sub>", "&Delta;&phi;<sub>Cavity</sub>",
			"Grad", "% 9.1f"
			)
	,
	/** Stamp position 1 (millimeters).*/
	STAMP_1_WEGGEBER(
			"Plunger 1", "Plunger 1", "Plunger 1",
			"mm"  , "% 9.1f"
			)
	,
	/** Stamp 1 has reached its minimal position (boolean value). In other words: has reached FMin.*/
	STAMP_1_FMIN(
			"Plunger 1 on FMin", "F<sub>min, Plunger 1</sub>", "F<sub>min, Plunger 1</sub>",
			""  , "%s"
			)
		,
	/** Stamp 1 has reached its maximum position (boolean value). In other words: has reached FMax.*/
	STAMP_1_FMAX(
			"Plunger 1 on FMax", "F<sub>max, Plunger 1</sub>", "F<sub>max, Plunger 1</sub>",
			""  , "%s"
			)
		,
	/** Stamp 1 bound position, requests the {@link ECavityElement#STAMP_1_WEGGEBER} and the location of FMin or FMax.*/
	STAMP_1_BOUND(
			"Plungerstellung 1", "Plungerstellung 1", "Plungerstellung 1",
			"", "%s"
			)
	,
	/** Stamp 2 position (millimeters).*/
	STAMP_2_WEGGEBER(
			"Plunger 2", "Plunger 2", "Plunger 2",
			"mm"  , "% 9.1f"
			)
	,
	/** Stamp 2 has reached its minimal position (boolean value). In other words: has reached FMin.*/
	STAMP_2_FMIN(
			"Plunger 2 on FMin", "F<sub>min, Plunger 2</sub>", "F<sub>min, Plunger 2</sub>",
			""  , "%s"
			)
		,
	/** Stamp 2 has reached its maximum position (boolean value). In other words: has reached FMax.*/
	STAMP_2_FMAX(
			"Plunger 2 on FMax", "F<sub>max, Plunger 2</sub>", "F<sub>max, Plunger 2</sub>",
			""  , "%s"
			)
		,
	/** Stamp 2 bound position, requests the {@link ECavityElement#STAMP_2_WEGGEBER} and the location of FMin or FMax.*/
	STAMP_2_BOUND(
			"Plungerstellung 2", "Plungerstellung 2", "Plungerstellung 2",
			"", "%s"
			)
	,
	/** Pre power (watt).*/
	PRE_POWER(
			"Vorlaufleistung", "P<sub>Vorlauf</sub>", "P<sub>Vorlauf</sub>",
			"kW"  , "% 9.1f"
			)
	,
	/** Pre phase (degree).*/
	PRE_PHASE(
			"Vorlaufphase", "&phi;<sub>Vorlauf</sub>", "&phi;<sub>Vorlauf</sub>",
			"Grad"  , "% 9.1f"
			)
	,
	/** Post power (watt).*/
	POST_POWER(
			"Rcklaufleistung", "P<sub>Rcklauf</sub>", "P<sub>Rcklauf</sub>",
			"kW"  , "% 9.1f"
			)
	,
	/** Post phase (degree).*/
	POST_PHASE(
			"Rcklaufphase", "&phi;<sub>Rcklauf</sub>", "&phi;<sub>Rcklauf</sub>",
			"Grad"  , "% 9.1f"
			)
	,
	/** Cell 2, temperature (celsius).*/
	CELL_2_TEMPERATURE(
			"Temperature Zelle 2", "T<sub>Zelle 2</sub>", "T<sub>Zelle 2</sub>",
			"C"  , "% 9.1f")
	,
	/** Cell 2, voltage (volt).*/
	CELL_2_VOLTAGE(
			"Spannung Zelle 2", "U<sub>Zelle 2</sub>", "U<sub>Zelle 2</sub>",
			"kV"  , "% 9.1f")
	,
	CELL_4_TEMPERATURE(
			"Temperature Zelle 4", "T<sub>Zelle 4</sub>", "T<sub>Zelle 4</sub>",
			"C"  , "% 9.1f")
	,
	/** Cell 4, voltage (volt). */
	CELL_4_VOLTAGE(
			"Spannung Zelle 4", "U<sub>Zelle 4</sub>", "U<sub>Zelle 4</sub>",
			"kV"  , "% 9.1f")
	,
	/** Cell 6, temperature (celsius).*/
	CELL_6_TEMPERATURE(
			"Temperature Zelle 6", "T<sub>Zelle 6</sub>", "T<sub>Zelle 6</sub>",
			"C"  , "% 9.1f")
	,
	/** Cell 6, voltage (volt).*/
	CELL_6_VOLTAGE(
			"Spannung Zelle 6", "U<sub>Zelle 6</sub>", "U<sub>Zelle 6</sub>",
			"kV"  , "% 9.1f")
	,
	/** Pressure in the cavity (mbar). */
	PRESSURE(
			"Druck Vaccum", "p<sub>Vaccum</sub>", "p<sub>Vaccum</sub>",
			"mbar", "% 8.2g")
	,
	/** Pre pressure in the cavity (mbar). */
	PRESSURE_PRE(
			"<html>Druck<sub>1</sub></html>", "p<sub>1</sub>", "p<sub>1</sub>",
			"mbar", "% 8.2g")
	,
	/** Post pressure in the cavity (mbar). */
	PRESSURE_POST(
			"<html>Druck<sub>2</sub></html>", "p<sub>2</sub>", "p<sub>2</sub>",
			"mbar", "% 8.2g")
	,
	/** Temperature (celsius). */
	TEMPERATURE(
			"T Fenster", "T<sub>Fenster</sub>", "T<sub>Fenster</sub>",
			"C"  , "% 9.1f")
	,
	/** Current cavity frequency state (fmin, fres, fmax). */
	STATE_FREQUENCY(
			"Cavity-Regelung", "Cavity-Regelung", "Cavity-Regelung",
			"", "%s")
	;

	// -------------------------------------------------------------------------

	/** An HTML free name of the enumeration element. */
	private String _name;

	/** Cavity name, probably with HTML styled code. */
	private String _nameCavity;

	/** Table description name, probably in HTML styled code. */
	private String _nameTable;

	/** Unit of the enumeration element. */
	private String _unit;

	/** Precision of the enumeration element. */
	private String _precision;

	// -------------------------------------------------------------------------

	/**
	 * Constructs the enumeration instance.
	 *
	 * @param name_ An HTML free name of the enumeration instance.
	 * @param nameCavity_ Cavity name, probably in HTML styled code.
	 * @param nameTable_ Table description name, probably in HTML styled code.
	 * @param unit_ Unit of the enumeration instance.
	 * @param precision_ Precision of the enumeration instance.
	 */
	ECavityElement(
			final String name_,
			final String nameCavity_,
			final String nameTable_,
			final String unit_,
			final String precision_)
	{
		validate(name_, nameCavity_, nameTable_, unit_, precision_);
		initialize(name_, nameCavity_, nameTable_, unit_, precision_);
	}

	// --- INSANCE METHODS -----------------------------------------------------
	/**
	 * Validates the enumeration instance.
	 *
	 * @param name_ An HTML free name of the enumeration instance.
	 * @param nameCavity_ Cavity name, probably in HTML styled code.
	 * @param nameTable_ Table description name, probably in HTML styled code.
	 * @param unit_ Unit of the enumeration instance.
	 * @param precision_ Precision of the enumeration instance.
	 */
	private void validate(
			final String name_,
			final String nameCavity_,
			final String nameTable_,
			final String unit_,
			final String precision_) {
		if (name_ == null)
			throw new NullPointerException("Name is null.");
		if (name_.isEmpty())
			throw new IllegalArgumentException("Name is empty.");

		if (nameCavity_ == null)
			throw new NullPointerException("Name cavity is null.");
		if (nameCavity_.isEmpty())
			throw new IllegalArgumentException("Name cavity is empty.");

		if (nameTable_ == null)
			throw new NullPointerException("Name table is null.");
		if (nameTable_.isEmpty())
			throw new IllegalArgumentException("Name table is empty.");

		if (unit_ == null)
			throw new NullPointerException("Unit is null.");

		if (precision_ == null)
			throw new NullPointerException("Precision is null.");
		if (precision_.isEmpty())
			throw new IllegalArgumentException("Precision is empty.");
		if (!precision_.matches(CGeneralConstant.PATTERN_CAVITY_PRECISION))
			throw new IllegalArgumentException("Precision of " + this.toString()
					+ " doesn't match the cavity precision: " + precision_);
	}

	/**
	 * Initialize the enumeration instance.
	 *
	 * @param name_ An HTML free name of the enumeration instance.
	 * @param nameCavity_ Cavity name, probably in HTML styled code.
	 * @param nameTable_ Table description name, probably in HTML styled code.
	 * @param unit_ Unit of the enumeration instance.
	 * @param precision_ Precision of the enumeration instance.
	 */
	private void initialize(
			final String name_,
			final String nameCavity_,
			final String nameTable_,
			final String unit_,
			final String precision_) {
		_name = name_;
		_nameCavity = nameCavity_;
		_nameTable = nameTable_;
		_unit = unit_;
		_precision = precision_;
	}

	@Override public String getName(final EName description_) throws EnumerationNotUsedException {
		switch(description_) {
		case NAME:
			return _name;
		case NAME_CAVITY:
			return _nameCavity;
		case NAME_TABLE:
			return _nameTable;
		case UNIT:
			return _unit;
		case PRECISION:
			return _precision;
		default:
			throw new EnumerationNotUsedException("Enumeration of " +
					description_.toString() + " exists but is not used on " +
					this.toString() + ".");
		}
	}

	/** DON'T DELETE THIS CONTENT BECAUSE IT IS NEEDED BY SOME CALLS! */
	@Override public void putName(final EName description_, final String name_) throws EnumerationNotUsedException {
		if (name_ == null)
			throw new NullPointerException("Setting " + description_.toString()
					+ " of " + this.toString() + " is null.");

		switch(description_)
		{
		case NAME:
			if (name_.isEmpty())
				throw new IllegalArgumentException("Setting " + description_.toString()
						+ " of " + this.toString() + " is empty.");
			_name = name_;
			break;
		case NAME_CAVITY:
			if (name_.isEmpty())
				throw new IllegalArgumentException("Setting " + description_.toString()
						+ " of " + this.toString() + " is empty.");
			_nameCavity = name_;
			break;
		case NAME_TABLE:
			if (name_.isEmpty())
				throw new IllegalArgumentException("Setting " + description_.toString()
						+ " of " + this.toString() + " is empty.");
			_nameTable = name_;
			break;
		case UNIT:
			_unit = name_;
			break;
		case PRECISION:
			if (name_.isEmpty())
				throw new IllegalArgumentException("Setting " + description_.toString()
						+ " of " + this.toString() + " is empty.");
			if (!name_.matches(CGeneralConstant.PATTERN_CAVITY_PRECISION))
				throw new IllegalArgumentException("Precision of " + this.toString()
						+ " doesn't match the cavity precision: " + name_);
			_precision = name_;
			break;
		default:
			throw new EnumerationNotUsedException("Enumeration of " +
					description_.toString() + " exists but is not used on " +
					this.toString() + ".");
		}
	}

	@Override public int size() { return 1; }
	@Override public boolean isAdjusting() { return false; }
	@Override public boolean isCDI() { return false; }
	@Override public boolean isFireEvent() { return false; }
}
/* end of file */