/*
 *
 * File: RFControl.java
 * Date: Jan 25, 2007; 9:17:59 AM
 */
package de.desy.mst.libs.accelerator.view.container;

import java.awt.Dimension;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.GroupLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EDebug;
import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.EViewElement;
import de.desy.mst.libs.accelerator.IComponentSubject;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEValue;
import de.desy.mst.libs.accelerator.Utility;
import de.desy.mst.libs.accelerator.view.CColor;
import de.desy.mst.libs.accelerator.view.CFrame;
import de.desy.mst.libs.accelerator.view.ComponentSubject;
import de.desy.mst.libs.accelerator.view.IViewComponent;
import de.desy.mst.libs.accelerator.view.IViewContainer;
import de.desy.mst.libs.accelerator.view.generic.RFButton;
import de.desy.mst.libs.accelerator.view.generic.RFPanel;

/**
 * <p>This is the public class of a RF container which contains the part of
 * the control panel. Each control panel can hold one or more entries, which are
 * included in the array list of the type {@code IEData}. There can
 * also be two or more elements in this array with the same type, but this is
 * normally not done. All entries are horizontal displayed with a gap of the
 * width {@code width} between them.</p>
 * <p>Moreover you are forced to deliver also a string value for the action
 * which each element is supposed to. This can be recalled by the action in the
 * {@code ActionEvent} with the method {@code ActionEvent.getActionCommand()}.
 * </p>
 * <p>The array element of the constructor are satisfied with the static class
 * called {@link RFControl#RFControl(Dimension, de.desy.mst.libs.accelerator.view.container.RFControl.RFControlHolder[])}
 * which can be found in this class ({@link RFControl.RFControlHolder}).
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFControl.java 15589 2008-08-27 07:55:58Z walla $
 */
public abstract class RFControl extends RFPanel implements IViewContainer
{
	/** The serial version UID, which is used by the java.io.Serializable
	 * interface. */
	public static final long serialVersionUID = 4530108951921842358l;

	/** Is the maximum delay time. */
	public static final int MAX_DELAY = 60000;

	/** This is the map which associates the enumeration of the data component
	 * and the view component, here the check box. */
	protected final Map<IEData, IViewComponent> _mapDataComponent;

	/**
	 * New holder class for carrying different values (most of it, the data
	 * component value is most important) for its supervisor class.
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @since 1.0.0
	 * @version 1.0.0, Jan 22, 2008 - 5:05:52 PM
	 *
	 */
	public static final class RFControlHolder
	{
		/** Is the data component enumeration interface which holds the data. */
		private IEData _data;

		/** Is the width between two displayed components. */
		private int _width;

		/** Is a boolean value, which informs the user of a dangerous
		 *  situations which can occur on further execution. */
		private boolean _dialog;

		/** Is the delay time. */
		private int _delay;

		/**
		 * The constructor which gets all relevant values.
		 *
		 * @param data_ The data component.
		 * @param delay_ Delay time after sending the value.
		 * @param dialog_ The program represents a dialog box with
		 * 		a YES/NO questions before sending the value. By pressing the
		 * 		YES button the value is send otherwise not.
		 * @param width_ The width between two drawn component elements.
		 */
		public RFControlHolder(final IEData data_, final int delay_,
				final boolean dialog_, final int width_)
		{
			validate(data_);
			initialize(data_, delay_, dialog_, width_);
		}

		/**
		 * Validates all fields.
		 *
		 * @param data_ The data component.
		 */
		private void validate(final IEData data_)
		{
			if (data_ == null)
				throw new NullPointerException("The delivered data component" +
						" is null.");
		}

		/**
		 * Initializes this component by setting all of its values to its
		 * internal fields.
		 *
		 * @param data_ The data component.
		 * @param width_ The width between two drawn component.
		 */
		private void initialize(final IEData data_, final int delay_,
				final boolean dialog_, final int width_)
		{
			_data = data_;
			if (width_ < 0 || width_ > 50)
				_width = 10;
			else
				_width = width_;
			_delay = (delay_ < 0) ? 0 :
				((delay_ > MAX_DELAY)? MAX_DELAY : delay_);
			_dialog = dialog_;
		}

		/**
		 * Returns the data component enumeration element.
		 *
		 * @return The data component is returned.
		 */
		public IEData getData()
		{
			return _data;
		}

		/**
		 * Returns the width between two component elements of the array. The
		 * array is defined in its supervisor method.
		 *
		 * @return Returns the width between two component elements.
		 */
		public int getWidth()
		{
			return _width;
		}

		/**
		 * The boolean dialog value is returned by this method. It informs the
		 * user that the following action will take place a longer time or that
		 * miscellaneous danger situations can occur.
		 *
		 * @return Returns the boolean dialog value.
		 */
		public boolean hasDialog()
		{
			return _dialog;
		}

		/**
		 * Returns the delay time between pressing the control button and
		 * performing the result of the status.
		 *
		 * @return Returns the delay time.
		 */
		public int getDelay()
		{
			return _delay;
		}
	}

	/**
	 * This public constructor sets the control panel of the RF container.
	 * Also an action command will be set to the delivered element, so that you
	 * can recall its action by performing the equality with the string value.
	 *
	 * @param dim_ Is the dimension if this container. Can also be {@code null}.
	 * @param data_ Is an array of the holder on data enumeration interfaces
	 * 		and its width from the next.
	 */
	protected RFControl(final Dimension dim_, final RFControlHolder[] data_)
	{
		super(dim_);
		checkArguments(data_);
		_mapDataComponent = new HashMap<IEData, IViewComponent>();

		initialize(dim_, data_);
	}

	/**
	 * This private method checks all arguments about its validity. If not an
	 * <tt>IllegalArgumentException</tt> is thrown.
	 *
	 * @param data_ Is an array of the data enumeration interfaces.
	 */
	private void checkArguments(final RFControlHolder[] data_)
	{
		if (data_ == null)
			throw new NullPointerException("The holder array is null.");
		if (data_.length == 0)
			throw new IllegalArgumentException("There should be one argument" +
					" set in the holder array.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#debug()
	 */
	public void debug(final JComponent component_)
	{
		if (CGeneralConstant.isDebug(EDebug.DEBUG_GRAPHIC))
			component_.setBorder(BorderFactory.createLineBorder(CColor.RED));
	}

	/**
	 * This private method will initialize the whole container.
	 *
	 * @param dim_ Is the dimension if this container. Can also be {@code null}.
	 * @param data_ Is the enumeration interface.
	 */
	private void initialize(final Dimension dim_, final RFControlHolder[] data_)
	{
		CFrame.DEBUG_BORDER(this);

    	IComponentSubject subject = ComponentSubject.getInstance();

    	JPanel control = new JPanel();
    	GroupLayout layout = new GroupLayout(control);
    	control.setLayout(layout);

    	GroupLayout.SequentialGroup horizontalButton = layout.createSequentialGroup();
		GroupLayout.ParallelGroup verticalButton = layout.createParallelGroup(GroupLayout.Alignment.BASELINE);

		for (int i = 0; i < data_.length; i++)
		{
			IEData data = data_[i].getData();
			int width = data_[i].getWidth();
	        RFButton button = new RFButton(data.getName(EName.NAME),
	        		CFrame.FONT_CONTROL, null);
			subject.register(data, button);
			subject.notify(data, EViewElement.TOOLTIP, "<html>" +
					Utility.getAddress(data)
					+ "<br>Schaltet den Sender auf "
					+ data.getName(EName.NAME) + ".</html>");
			button.setOpaque(true);

			_mapDataComponent.put(data, button);

			horizontalButton.addComponent(button);
			if (i < data_.length - 1)
				horizontalButton.addGap(width);
			verticalButton.addComponent(button);
		}

		layout.setHorizontalGroup(horizontalButton);
		layout.setVerticalGroup(verticalButton);

		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		add(Box.createVerticalGlue());
		add(control);
		add(Box.createVerticalGlue());
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#reset(de.desy.mst.libs.accelerator.IEValue)
	 */
	public void reset(final IEValue container_)
	{
		throw new RuntimeException("This method is currently not implemented.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getPanel()
	 */
	public JComponent getPanel()
	{
		return this;
	}


	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getValue(de.desy.mst.libs.accelerator.IEValue)
	 */
	public Number getValue(final IEValue container_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#setValue(de.desy.mst.libs.accelerator.IEValue, java.lang.Number)
	 */
	public void setValue(final IEValue container_, final Number value_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * <p>This allows to set a boolean value on the enumeration interface of
	 * {@link IEValue}. For example, you can get the enabling of the
	 * container by calling it with <tt>EViewContainer.BUTTON</tt> and the
	 * enumeration interface type of <tt>EViewElement.ENABLE</tt>.</p>
	 *
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement)
	 */
	public boolean getBoolean(final IEValue container_,
			final EViewElement element_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

	 /**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	  * (non-Javadoc)
	  * @see de.desy.mst.libs.accelerator.view.IViewContainer#setBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement, boolean)
	  */
	public void setBoolean(final IEValue container_,
			final EViewElement element_, final boolean value_)
	{
		throw new RuntimeException("This method is not implemented.");
	}

}
/* end of file */