/*
 *
 * File: RFRadioButton.java
 * Date: Jul 1, 2007; 3:38:48 PM
 */
package de.desy.mst.libs.accelerator.view.generic;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.util.EventListener;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JRadioButton;
import javax.swing.border.Border;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EDebug;
import de.desy.mst.libs.accelerator.EGraphicComponent;
import de.desy.mst.libs.accelerator.ESeverity;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.exception.OperationNotAllowedException;
import de.desy.mst.libs.accelerator.view.CColor;
import de.desy.mst.libs.accelerator.view.IViewComponent;

/**
 * This represents a radio button with its dimension and font stored. This
 * derivates the class of {@code JRadioButton}.
 *
 * @author <a href="mailto:marcus.walla(at}desy.de">walla</a>
 * @version $Id: RFRadioButton.java 36465 2010-06-04 08:04:07Z walla $
 */
public class RFRadioButton extends JRadioButton implements IViewComponent
{
	// --- CLASS FIELDS --------------------------------------------------------
	
	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	final static long serialVersionUID = 6493920042341236346l;

	/** Class name. */
	public final static String CLAZZ = RFRadioButton.class.getSimpleName();
	
	// --- INSTANCE FIELDS -----------------------------------------------------
	
	/** This is the dimension of this container. */
    private Dimension _dim;

    /** The severity of the radio button component. This has no direct effect on the
     * GUI element. */
    private ESeverity _severity = ESeverity.NORMAL;

    // --- CONSTRUCTOR ---------------------------------------------------------
    
    /**
	 * Creates a JRadioButton instance without any special arguments.
	 * 
	 * The dimension is currently {@code null} and therefore this instance can
	 * vary in its size. Also the default font is used.
	 * 
	 * @see #RFRadioButton(Font, Dimension)
	 */
	public RFRadioButton()
	{
		super();
		initialization(null, null);
		debug();
	}

    /**
	 * This will create a new radio button as an instance. As one of its
	 * arguments it will create only a text without any icon field. It will
	 * set the dimension and font to its default values.
	 * 
	 * The dimension is currently {@code null} and therefore this instance can
	 * vary in its size. Also the default font is used.
	 *
	 * @param name_ The text to be displayed by the label.
	 * @see #RFRadioButton(Font, Dimension)
     */
    public RFRadioButton(final String name_)
    {
		super(name_);
		initialization(null, null);
    	debug();
    }

    /**
     * A radio button is created here with the font and dimension as its only
     * parameters.
     *
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
     */
	public RFRadioButton(final Font font_, final Dimension dim_)
	{
		super();
		initialization(dim_, font_);
    	debug();
    }

	/**
	 * This will create a new radio button as an instance. As one of its
	 * arguments it will create only an icon without any text field.
	 *
	 * @param icon_ The image to be displayed by the label.
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFRadioButton(final Icon icon_, final Font font_,
			final Dimension dim_)
	{
		super(icon_);
		initialization(dim_, font_);
    	debug();
    }

	/**
	 * This will create a new radio button as an instance. As one of its
	 * arguments it will create only a text without any icon field.
	 *
	 * @param name_ The text to be displayed by the label.
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFRadioButton(final String name_, final Font font_,
			final Dimension dim_)
	{
		super(name_);
		initialization(dim_, font_);
    	debug();
    }

	/**
	 * This will create a new radio button as an instance. As one of its
	 * arguments it will create with an icon and a text field.
	 *
	 * @param name_ The text to be displayed by the label.
	 * @param icon_ The image to be displayed by the label.
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFRadioButton(final String name_, final Icon icon_, final Font font_,
			final Dimension dim_)
	{
		super(name_, icon_);
		initialization(dim_, font_);
    	debug();
    }

	// --- INSTANCE METHODS ----------------------------------------------------

	/**
	 * Initialization.
	 *
	 * @param dim_ The dimension.
	 * @param font_ The font.
	 */
	private void initialization(
			final Dimension dim_, 
			final Font font_)
	{
		if (font_ != null)
			setFont(font_);
	
		_dim = dim_;
	}

	@Override public Dimension getMinimumSize()
	{
		if (_dim == null)
			return super.getMinimumSize();
		return _dim;
	}

	@Override public Dimension getMaximumSize()
	{
		if (_dim == null)
			return super.getMaximumSize();
		return _dim;
	}

	@Override public Dimension getPreferredSize()
	{
		if (_dim == null)
			return super.getPreferredSize();
		return _dim;
	}

	@Override public void setSize(final Dimension dim_)
	{
		_dim = dim_;
		super.setSize(dim_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#debug()
	 */
	public void debug()
	{
		if (CGeneralConstant.isDebug(EDebug.DEBUG_GRAPHIC))
			super.setBorder(BorderFactory.createLineBorder(CColor.RED));
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSeverity(int)
	 */
	public void putSeverity(final ESeverity severity_)
	{
		_severity = severity_;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchSeverity()
	 */
	public ESeverity fetchSeverity()
	{
		return _severity;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchGraphicComponent()
	 */
	public EGraphicComponent fetchGraphicComponent()
	{
		return EGraphicComponent.RADIOBUTTON_FIXED;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchText(de.desy.mst.libs.accelerator.IEData)
	 */
	public String fetchText(IEData view_)
	{
		return super.getText();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#hasOpaque()
	 */
	public boolean hasOpaque()
	{
		return super.isOpaque();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isEnable()
	 */
	public boolean isEnable()
	{
		return super.isEnabled();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isValueAdjusting(de.desy.mst.libs.accelerator.IEData)
	 */
	public boolean isValueAdjusting(IEData view_)
	{
		return false;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isVisual()
	 */
	public boolean isVisual()
	{
		return super.isVisible();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putListener(java.util.EventListener)
	 */
	public void putListener(final EventListener listener_)
	{
		if (listener_ instanceof ActionListener)
			super.addActionListener((ActionListener) listener_);
		else
			throw new IllegalArgumentException("The listener model you are" +
					" adding doesn't exist for this instance: " +
					listener_.toString());
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBackground(java.awt.Color)
	 */
	public void putBackground(Color background_)
	{
		super.setBackground(background_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putEnable(boolean)
	 */
	public void putEnable(boolean enable_)
	{
		super.setEnabled(enable_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putForeground(java.awt.Color)
	 */
	public void putForeground(Color foreground_)
	{
		super.setForeground(foreground_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putNumber(java.lang.Number, de.desy.mst.libs.accelerator.IEData)
	 */
	public void putNumber(Number number_, IEData view_)
	{
		throw new OperationNotAllowedException("The operation is not allowed.");
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putOpaque(boolean)
	 */
	public void putOpaque(boolean opaque_)
	{
		super.setOpaque(opaque_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putText(java.lang.String, de.desy.mst.libs.accelerator.IEData)
	 */
	public void putText(String label_, IEData view_)
	{
		super.setText(label_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putTooltip(java.lang.String)
	 */
	public void putTooltip(String tooltip_)
	{
		this.setToolTipText(tooltip_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putVisual(boolean)
	 */
	public void putVisual(boolean visual_)
	{
		super.setVisible(visual_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSelected(boolean)
	 */
	public void putSelected(boolean isSelected_)
	{
		this.setSelected(isSelected_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBorder(javax.swing.border.Border)
	 */
	public void putBorder(final Border border_)
	{
		super.setBorder(border_);
	}
	
	@Override
	public String toString()
	{
		return CLAZZ + "[text" + this.getText() + ']';
	}
}
/* end of file */