/*
*
* File: RFScrollPane.java
* Date: Jan 22, 2007; 5:13:50 PM
*/
package de.desy.mst.libs.accelerator.view.generic;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.util.EventListener;

import javax.swing.JScrollPane;
import javax.swing.border.Border;

import de.desy.mst.libs.accelerator.EGraphicComponent;
import de.desy.mst.libs.accelerator.ESeverity;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.exception.OperationNotAllowedException;
import de.desy.mst.libs.accelerator.view.CFrame;
import de.desy.mst.libs.accelerator.view.IViewComponent;

/**
 * This is the manipulated {@code JScrollPane} which allows beside calling
 * the constructor with its normal java values also to provide its dimension.
 * Moreover there are also there inherited methods from the class
 * {@code JComponent} which all will return the delivered dimension value, so
 * that there no resize will happen.
 *
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFScrollPane.java 36465 2010-06-04 08:04:07Z walla $
 * @see javax.swing.JScrollPane
 */
public class RFScrollPane extends JScrollPane implements IViewComponent
{
	// --- CLASS FIELDS --------------------------------------------------------
	
	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	static final long serialVersionUID = 5952997466759345990L;

	/** Class name. */
	public static final String CLAZZ = RFScrollPane.class.getSimpleName();
	
	// --- INSTANCE FIELDS -----------------------------------------------------
	
	/** This is the dimension of this container. */
	private Dimension _dim;

	// --- CONSTRUCTOR ---------------------------------------------------------
	
	/**
	 * Creates an empty (no view port view) {@code JScrollPane}  where both
	 * horizontal and vertical scroll bars appear when needed. The dimension
	 * is set to {@code null} and therefore this instance can vary in its size.
	 * 
	 * @see #RFScrollPane(Dimension)
	 */
	public RFScrollPane()
	{
		this(null);
	}
	
	/**
	 * Creates an empty (no view port view) {@code JScrollPane}  where both
	 * horizontal and vertical scroll bars appear when needed. The dimension
	 * is possibly also set. If it is null then the dimension of the current
	 * drawn figure is return by its inherited methods.
	 *
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFScrollPane(final Dimension dim_)
	{
		super();
		initialize(dim_);
    	debug();
	}

	/**
	 * Creates a JScrollPane that displays the contents of the specified
	 * component, where both horizontal and vertical scroll bars appear whenever
	 * the component's contents are larger than the view. The dimension is
	 * possibly also set. If not it will draw this component with its current
	 * drawing settings and return it as a {@code Dimension}.
	 *
	 * @param view_ The component to display in the scrollpane's view port.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFScrollPane(final Component view_, final Dimension dim_)
	{
		super(view_);
		initialize(dim_);
    	debug();
	}

	/**
	 * Creates an empty (no view port view) JScrollPane  with specified scroll bar
	 * policies. The dimension is possibly also set. If not, it will return the
	 * current drawn dimension.
	 *
	 * @param vsbPolicy_ An integer that specifies the vertical scroll bar policy.
	 * @param hsbPolicy_ An integer that specifies the horizontal scroll bar policy.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFScrollPane(final int vsbPolicy_, final int hsbPolicy_,
			final Dimension dim_)
	{
		super(vsbPolicy_, hsbPolicy_);
		initialize(dim_);
    	debug();
	}

	/**
	 * Creates an empty (no view port view) JScrollPane  with specified scroll bar
	 * policies. The dimension is possibly also set. If not, it will return the
	 * current drawn dimension.
	 *
	 * @param view_ The component to display in the scrollpane's view port.
	 * @param vsbPolicy_ An integer that specifies the vertical scroll bar policy.
	 * @param hsbPolicy_ An integer that specifies the horizontal scroll bar policy.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFScrollPane(final Component view_, final int vsbPolicy_,
			final int hsbPolicy_, final Dimension dim_)
	{
		super(view_, vsbPolicy_, hsbPolicy_);
		initialize(dim_);
    	debug();
	}

	// --- INSTANCE METHODS ----------------------------------------------------
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#debug()
	 */
	public void debug()
	{
		CFrame.DEBUG_BORDER(this);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSeverity(int)
	 */
	public void putSeverity(final ESeverity severity_)
	{
		throw new IllegalArgumentException("This method is not implemented.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchSeverity()
	 */
	public ESeverity fetchSeverity()
	{
		return ESeverity.NORMAL;
	}

	/**
	 * Initialization.
	 *
     * @param dim_ This is the dimension of this container. If it is null the
     * 		dimension of the current container is selected.
	 */
	private void initialize(final Dimension dim_)
	{
		_dim = dim_;
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#getMinimumSize()
	 */
	@Override public Dimension getMinimumSize()
	{
		if (_dim == null)
			return super.getMinimumSize();
		return _dim;
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#getMaximumSize()
	 */
	@Override public Dimension getMaximumSize()
	{
		if (_dim == null)
			return super.getMaximumSize();
		return _dim;
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#getPreferredSize()
	 */
	@Override public Dimension getPreferredSize()
	{
		if (_dim == null)
			return super.getPreferredSize();
		return _dim;
	}
	
	@Override public void setSize(final Dimension dim_)
	{
		_dim = dim_;
		super.setSize(dim_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchGraphicComponent()
	 */
	public EGraphicComponent fetchGraphicComponent()
	{
		return EGraphicComponent.SCROLLPANE_FIXED;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchLabel()
	 */
	public String fetchText(IEData view_)
	{
		throw new OperationNotAllowedException("The operation is not allowed.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isEnable()
	 */
	public boolean isEnable()
	{
		return super.isEnabled();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isVisual()
	 */
	public boolean isVisual()
	{
		return super.isVisible();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#hasOpaque()
	 */
	public boolean hasOpaque()
	{
		return super.isOpaque();
	}

	/*
	 * Will always return the {@code false} value.
	 *
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isValueAdjusting(de.desy.mst.libs.accelerator.model.IEDataComponent)
	 */
	public boolean isValueAdjusting(IEData view_)
	{
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putListener(java.util.EventListener)
	 */
	public void putListener(final EventListener listener_)
	{
		throw new IllegalArgumentException("The listener model you are" +
				" adding doesn't exist for this instance: " +
				listener_.toString());
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putLabel(java.lang.String)
	 */
	public void putText(String label_, IEData view_)
	{
		throw new OperationNotAllowedException("The operation is not allowed.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putNumber(java.lang.Number, de.desy.mst.libs.accelerator.model.IEDataComponent)
	 */
	public void putNumber(Number number_, IEData view_)
	{
		throw new OperationNotAllowedException("The operation is not allowed.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putEnable(boolean)
	 */
	public void putEnable(boolean enable_)
	{
		super.setEnabled(enable_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putOpaque(boolean)
	 */
	public void putOpaque(boolean opaque_)
	{
		super.setOpaque(opaque_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putVisual(boolean)
	 */
	public void putVisual(boolean visual_)
	{
		super.setVisible(visual_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putForeground(java.awt.Color)
	 */
	public void putForeground(Color foreground_)
	{
		super.setForeground(foreground_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBackground(java.awt.Color)
	 */
	public void putBackground(Color background_)
	{
		super.setBackground(background_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putTooltip(java.lang.String)
	 */
	public void putTooltip(String tooltip_)
	{
		this.setToolTipText(tooltip_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSelected(boolean)
	 */
	public void putSelected(boolean isSelected_)
	{
		throw new RuntimeException("This method makes no sense here!");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBorder(javax.swing.border.Border)
	 */
	public void putBorder(final Border border_)
	{
		super.setBorder(border_);
	}
	
	@Override
	public String toString()
	{
		return CLAZZ;
	}
}
/* end of file */