/**
 * File: RFSpinner.java
 * Date: Mar 9, 2010
 */
package de.desy.mst.libs.accelerator.view.generic;

import java.awt.Color;
import java.awt.Dimension;
import java.util.EventListener;

import javax.swing.BorderFactory;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.Border;
import javax.swing.event.ChangeListener;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EDebug;
import de.desy.mst.libs.accelerator.EGraphicComponent;
import de.desy.mst.libs.accelerator.ESeverity;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.exception.NotImplementedException;
import de.desy.mst.libs.accelerator.view.CColor;
import de.desy.mst.libs.accelerator.view.IViewComponent;

/**
 * Spinner component implements the {@link IViewComponent} interface.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFSpinner.java 36465 2010-06-04 08:04:07Z walla $
 */
public final class RFSpinner extends JSpinner implements IViewComponent
{
	// --- CLASS FIELDS --------------------------------------------------------
	
	/** Class name */
	public final static String CLAZZ = RFSpinner.class.getSimpleName();
	
	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	private static final long serialVersionUID = -9020091435418732732L;

	// --- INSTANCE FIELDS -----------------------------------------------------
	
    /** The severity of the label component. This has no direct effect on the
     * GUI element. */
    private ESeverity _severity = ESeverity.NORMAL;
    
    /** Dimension. */
    private Dimension _dim = null;

    // --- CONSTRUCTOR ---------------------------------------------------------
	
	/**
	 * Constructor.
	 * 
	 * @param model_ Spinner model.
	 */
	public RFSpinner(final SpinnerModel model_)
	{
		this(model_, null);
	}
	
	/**
	 * Constructor.
	 * 
	 * @param model_ Spinner model.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFSpinner(final SpinnerModel model_, final Dimension dim_)
	{
		super(model_);
		initialize(null);
	}

	// --- INSTANCE METHODS ----------------------------------------------------

	/**
	 * Initialization.
	 *
     * @param dim_ This is the dimension of this container. If it is null the
     * 		dimension of the current container is selected.
	 */
	private void initialize(final Dimension dim_)
	{
		_dim = dim_;
	}

    /*
     * (non-Javadoc)
     * @see javax.swing.JComponent#getMinimumSize()
     */
	@Override public Dimension getMinimumSize()
	{
		if (_dim != null)
			return _dim;
		return super.getMinimumSize();
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#getMaximumSize()
	 */
	@Override public Dimension getMaximumSize()
	{
		if (_dim != null)
			return _dim;
		return super.getMaximumSize();
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#getPreferredSize()
	 */
	@Override public Dimension getPreferredSize()
	{
		if (_dim != null)
			return _dim;
		return super.getPreferredSize();
	}
	
	@Override
	public EGraphicComponent fetchGraphicComponent()
	{
		return EGraphicComponent.SPINNER;
	}

	@Override
	public String fetchText(IEData view_)
	{
		throw new NotImplementedException();
	}

	@Override
	public boolean isEnable()
	{
		return super.isEnabled();
	}

	@Override
	public boolean isVisual()
	{
		return super.isVisible();
	}
	
	@Override
	public boolean hasOpaque()
	{
		return super.isOpaque();
	}

	@Override
	public boolean isValueAdjusting(IEData view_)
	{
		return false;
	}

	/**
	 * Only change listener are accepted.
	 */
	@Override
	public void putListener(final EventListener listener_)
	{
		if (listener_ instanceof ChangeListener)
			super.addChangeListener((ChangeListener) listener_);
		else
			throw new IllegalArgumentException("The listener model you are" +
					" adding doesn't exist for this instance: " +
					listener_.toString());
	}

	/**
	 * The spinner has no text field and therefore this method does really 
	 * nothing.
	 */
	@Override
	public void putText(String label_, IEData view_)
	{
	}

	/**
	 * If the spinner model is of the type {@code SpinnerNumberModel} then the
	 * model is casted to the inherited type and sets the value. Otherwise 
	 * nothing is done and the method will return immediately without doing 
	 * anything.
	 */
	@Override
	public void putNumber(Number number_, IEData data_)
	{
		if (super.getModel() instanceof SpinnerNumberModel)
			super.getModel().setValue(number_);
	}

	@Override
	public void putEnable(boolean enable_)
	{
		super.setEnabled(enable_);
	}

	@Override
	public void putVisual(boolean visual_)
	{
		super.setVisible(visual_);
	}

	@Override
	public void putOpaque(boolean opaque_)
	{
		super.setOpaque(opaque_);
	}

	/**
	 * Currently the foreground color is set to the internal editor instance
	 * if it is of the kind {@code JSpinner.DefaultEditor}. Otherwise it will
	 * directly set to the color of the spinner instance itself.
	 */
	@Override
	public void putForeground(Color foreground_)
	{
        if (super.getEditor() instanceof JSpinner.DefaultEditor) 
            ((JSpinner.DefaultEditor) this.getEditor()).getTextField().setForeground(foreground_);
        else 
        	super.setForeground(foreground_);
	}

	/**
	 * Currently the background color is set to the internal editor instance
	 * if it is of the kind {@code JSpinner.DefaultEditor}. Otherwise it will
	 * directly set to the color of the spinner instance itself.
	 */
	@Override
	public void putBackground(Color background_)
	{
        if (super.getEditor() instanceof JSpinner.DefaultEditor) 
            ((JSpinner.DefaultEditor) this.getEditor()).getTextField().setBackground(background_);
        else 
        	super.setBackground(background_);
	}

	@Override
	public void putTooltip(String tooltip_)
	{
		this.setToolTipText(tooltip_);
	}

	/**
	 * Returns the current name of this label by just calling the {@code getName()}
	 * method.
	 *
	 * @return Returns the name of this label.
	 */
	@Override public String toString()
	{
		return CLAZZ + "[value=" + this.getValue() + "]";
	}

	@Override
	public void putSelected(boolean isSelected_)
	{
		throw new NotImplementedException();
	}

	@Override
	public void putBorder(final Border border_)
	{
		super.setBorder(border_);
	}

	@Override
	public void debug()
	{
		if (CGeneralConstant.isDebug(EDebug.DEBUG_GRAPHIC))
			super.getEditor().setBorder(BorderFactory.createLineBorder(CColor.RED));
	}

	@Override
	public ESeverity fetchSeverity()
	{
		return _severity;
	}

	@Override
	public void putSeverity(final ESeverity severity_)
	{
		_severity = severity_;
	}
}
/* end of file */