/**
 *  Created on	Jan 12, 2006 - Jan 27, 2006
 *	Jul 28, 2006 - Aug 01, 2006	ws: new: set, equals, hashcode
 *  Oct 15, 2008 - Nov  4, 2008	ws:	added setFromSaveString(...), getSaveString(...), makeKey(...)
 * 	Jun  8, 2009 - Jun 26, 2009	ws:	pass(...) to final
 */
package de.desy.mst.libs.framework.mstapp.logging.filter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.desy.mst.libs.framework.mstapp.ini.PrefStorage;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.util.xml.AXmlBeschreibung;

/**
 * Basis Implementation eines Filters.<br>
 * In abgeleiteten Klassen muss nur noch
 * <li>getName()
 * <li>decorateElement(final Document document, final Element element)
 * <li>passOnStandardModus()
 * <br>
 * implementiert werden.
 * @author ws
 * @version 1.0.0
 */
public abstract class ALogFilter extends AXmlBeschreibung implements ILogFilter {
	private static final String MODUS = "MODUS";
//-----------------------------------------------------------------------------
	private FilterModus modus = FilterModus.STANDARD;

//=============================================================================

	/**wird nur aufgerufen, wenn der Filter in Standard Modus<br>Implementation des Filterverhaltens im Standard Modus*/
	abstract protected boolean passOnStandardModus(final LogMsg logMsg);
	/**Name des Filters (XML Namenskompatibel, ohne Blanks etc.)*/
	abstract protected String getName();
	/**Element mit Filterspezifischen Infos, jenseits von getName() versehen<br>Argumente nie null*/
	abstract protected void decorateElement(final Document document, final Element element);

//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.util.xml.IXmlBeschreibung#getBeschreibungAsElement(org.w3c.dom.Document)
	 */
	@Override public final Element getBeschreibungAsElement(Document document) {
		if(document == null) document = createDocument();
		if(document == null) {return null;}	//in real trouble on a real minor feature

		final Element element = document.createElement(getName());
		if(getModus() != FilterModus.STANDARD)element.setAttribute(MODUS, getModus().toString());
		decorateElement(document, element);
		return element;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ILogFilter#pass(de.desy.mst.mstapp.logging.LogMsg)
	 */
	@Override public final boolean pass(final LogMsg logMsg) {
		if(getModus() == FilterModus.ALWAYS_PASS) 	{return true;}
		if(getModus() == FilterModus.NEVER_PASS)  	{return false;}
		if(logMsg 	  == null)  					{return false;}
		return passOnStandardModus(logMsg);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ILogFilter#filter(de.desy.mst.mstapp.logging.LogMsg)
	 */
	@Override public final boolean filter(final LogMsg logMsg) {return !pass(logMsg);}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ILogFilter#getModus()
	 */
	@Override public final FilterModus getModus() {return this.modus;}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.logging.filter.ILogFilter#setModus(de.desy.mst.mstapp.logging.filter.FilterModus)
	 */
	@Override public final void setModus(final FilterModus modus) {if(modus != null) this.modus = modus;}
	/**Clones the Setting of the wegWirfFilter*/
	public void set(final ALogFilter wegWirfFilter) {
		if(wegWirfFilter != null)
			this.setModus(wegWirfFilter.getModus());
	}

	@Override public boolean equals(final Object obj) {
		if(obj instanceof ALogFilter){
			final ALogFilter filter = (ALogFilter) obj;
			return this.getModus() == filter.getModus();
		}
		return false;
	}
	@Override public int hashCode() {
		return getModus().hashCode();
	}

	/**
	 * @param appenderGruppe dieser spezifische Appender (Std Datei Logging mehrere Dateinamen)
	 * @param saveString	 Serialisierung dieses Filters fuer PrefStorage
	 * @param optionalDefault	 wird nicht beachtet
	 */
	protected void setFromSaveString(final String appenderGruppe, final String saveString, final Object optionalDefault) {
		final String key = makeKey(appenderGruppe);
		final String value = PrefStorage.extract(saveString, key);
		try {
			final FilterModus modus = (value == null || value.isEmpty())
									?  FilterModus.STANDARD
									:  FilterModus.valueOf(value);
			this.setModus(modus);
		} catch (final IllegalArgumentException ex) {
			Logger.SWFW.warn("Kann den Filter " + this.getName()
						   + " nicht zu dem Modus '" + value
						   + "' setzen ignorier den Wert und belasse ihn auf '"
						   + getModus().toString() + "'.");
		} catch (final NullPointerException ex) {
			Logger.SWFW.warn("Kann den Filter " + this.getName()
						   + " nicht zu dem Modus 'null' setzen"
						   + ", belasse ihn auf '" + getModus().toString()
						   + "'.");
		}
	}
	/**
	 * @param appenderGruppe dieser spezifische Appender (Std Datei Logging mehrere Dateinamen)
	 * @param saveString	 Serialisierung von Eigenschaften anderer Klassenobjekte (do not touch or duplicate)
	 * @return				 appended diese Serialisierung
	 */
	protected String getSaveString(final String appenderGruppe, final String saveString){
		if(getModus() == FilterModus.STANDARD)
			return saveString == null ? "" : saveString;
		final String key = makeKey(appenderGruppe);
		final String value = getModus().toString();
		return PrefStorage.serialize(saveString, key, value);
	}
	/**@return	combined key*/
	protected final String makeKey(final String appenderGruppe, final String valueKey) {
		return appenderGruppe + ": " + valueKey;
	}
	//-------------------------------------------------------------------------
	private String makeKey(final String appenderGruppe) {
		return makeKey(appenderGruppe, MODUS);
	}
}
