/**
 *  Created on	Jun 20, 2006 - Jun 22, 2006
 */
package de.desy.mst.libs.framework.mstapp.mode;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Hilfsklasse (Package Privat) Fuer MaschineMgr.
 * Haelt die GUELTIGE Liste erlaubter Maschinen.<br>
 * Mann kann sich zur Benachrichtigung ueber Anderungen registrieren lassen<br>
 * Threadsafe.
 * 
 * @author ws
 * @version 1.0.0
 */
public class MaschinenKonfiguration {
	private final List<Maschine> erlaubteMaschinen = new ArrayList<Maschine>();
	private final List<IMaschinenKonfigurationObserver> observers = new ArrayList<IMaschinenKonfigurationObserver>();
	private final KonfigChangeCounterObserver konfigChangeCounterObserver = new KonfigChangeCounterObserver();

//=============================================================================
	
	/**
	 * @param maschine  ist erlaubte Maschine (Convenience XTor fuer singulaere Erlaubnisse)
	 * @throws NullpointerException falls maschine == null
	 */
	public MaschinenKonfiguration(final Maschine maschine) {
		this(createList(maschine));
	}
	/**
	 * @param erlaubteMaschinen liste von erlaubten Maschine != null
	 * @throws NullpointerException, 		falls erlaubteMaschinen == null
	 * @throws IllegalArgumentException,	falls erlaubteMaschinen keine Maschinen enthaelt
	 */
	public MaschinenKonfiguration(final List<Maschine> erlaubteMaschinen) {
		this.set(erlaubteMaschinen);
		this.registerIMaschinenKonfigurationObserver(this.konfigChangeCounterObserver);
	}
	/**
	 * @param erlaubteMaschinen liste von erlaubten Maschine != null
	 * @throws NullpointerException, 		falls erlaubteMaschinen == null
	 * @throws IllegalArgumentException,	falls erlaubteMaschinen keine Maschinen enthaelt
	 */
	public MaschinenKonfiguration(final Maschine[] erlaubteMaschinen) {
		this.set(erlaubteMaschinen);
		this.registerIMaschinenKonfigurationObserver(this.konfigChangeCounterObserver);
	}
	
//=============================================================================
	
	public synchronized final boolean istErlaubt(final Maschine maschine){
		return this.erlaubteMaschinen.contains(maschine);
	}
	/**
	 * @param maschine maschine ist zusaetzlich erlaubt
	 */
	public synchronized void add(final Maschine maschine) {
		if(!this.istErlaubt(maschine) && maschine != null){
			this.erlaubteMaschinen.add(maschine);
			fireConfigChange();
		}
	}
	/**
	 * @param erlaubteMaschinen maschinen die zusaetzlich erlaubt sind
	 */
	public void add(final List<Maschine> erlaubteMaschinen) {
		for (final Maschine maschine : erlaubteMaschinen) {
			this.add(maschine);
		}
	}
	
	/**
	 * @param maschine entfernt die maschine aus der Erlaubnisliste
	 * @throws IllegalStateException falls man versucht die letzte Maschine zu entfernen
	 */
	public synchronized void remove(final Maschine maschine) {
		if(istErlaubt(maschine)){
			if(this.erlaubteMaschinen.size() <= 1){
				throw new IllegalStateException("Kann die letzte Maschine nicht entfernen.");
			}
			this.erlaubteMaschinen.remove(maschine);
			fireConfigChange();
		}
	}
	/**
	 * @param maschine nur maschine ist erlaubt
	 * @throws NullpointerException falls maschine == null
	 */
	public void set(final Maschine maschine) {
		final List<Maschine> list = createList(maschine);
		this.set(list);
	}
	/** 
	 * @param erlaubteMaschinen voellig neuer Array erlaubter Maschinen
	 * @throws NullpointerException 	falls erlaubteMaschinen == null
	 * @throws IllegalArgumentException	falls erlaubteMaschinen keine Maschinen enthaelt
	 */
	public void set(final Maschine[] erlaubteMaschinen) {
		this.set(Arrays.asList(erlaubteMaschinen));
	}
	/**
	 * @param erlaubteMaschinen voellig neue Liste erlaubter Maschinen
	 * @throws NullpointerException 	falls erlaubteMaschinen == null
	 * @throws IllegalArgumentException	falls erlaubteMaschinen keine Maschinen enthaelt
	 */
	public synchronized void set(final List<Maschine> erlaubteMaschinen) {
		if(erlaubteMaschinen == null){
			throw new NullPointerException("erlaubteMaschinen darf nicht null sein");
		}
		if(isSameList(erlaubteMaschinen)){
			return;
		}
		this.erlaubteMaschinen.clear();	//fires automatisch nach naechstem Statement
		this.add(erlaubteMaschinen);
		if(this.erlaubteMaschinen.size() <= 0){
			throw new IllegalArgumentException("erlaubteMaschinen muss mindestens eine Maschine != null enthalten");
		}
		
	}

	/**
	 * @return Alle Erlaubten Maschinen als ReadOnlyListe, nie null und .size() > 0
	 */
	public synchronized List<Maschine> getASReadOnlyList() {
		final List<Maschine> unmodifiableList = Collections.unmodifiableList(this.erlaubteMaschinen);
		return unmodifiableList;
	}
	
	/**Framework veraendert von sich aus nur fuer jungfraeuliche Konfigurationen die Default Konfiguration*/
	public final boolean istJungfraeulich(){return this.konfigChangeCounterObserver.counter <= 1;}
	
	/**
	 * @param observer wird bei Konfigurationsaenderungen benachrichtigt
	 */
	public void registerIMaschinenKonfigurationObserver(final IMaschinenKonfigurationObserver observer) {
		if(observer == null){
			return;	//nothing to do
		}
		synchronized (this.observers) {
			if(this.observers.contains(observer)) {
				return;	//nothing to do
			}
			observer.onMaschinenKonfigurationChange(this);
			this.observers.add(observer);
		}
	}
	/**
	 * @param observer wird deregistriert, keine referenzen werden gehalten, keine callbacks mehr
	 */
	public void deregisterIMaschineObserver(final IMaschinenKonfigurationObserver observer) {
		if(observer != null){
			synchronized (this.observers) {
				this.observers.remove(observer);
			}
		}
	}
	
	
	@Override public synchronized String toString() {
		return super.toString() + " " + this.erlaubteMaschinen.toString();
	}
	
//=============================================================================
	
	/**support routine fuer XTor, call once only. Erzeugt Liste mit maschine*/	
	private static List<Maschine> createList(final Maschine maschine) {
		if(maschine == null){
			throw new NullPointerException("Die maschine darf nicht null sein");
		}
		final List<Maschine> list = new ArrayList<Maschine>();
		list.add(maschine);
		return list;
	}
	private synchronized void fireConfigChange(){
		synchronized (this.observers) {
			for (final IMaschinenKonfigurationObserver observer: this.observers) {
				observer.onMaschinenKonfigurationChange(this);
			}
		}
	}

	/**synchronisiert da nur aus genau einer synchronisierten Methode gerufen*/
	private boolean isSameList(final List<Maschine> erlaubteMaschinen) {
		if(this.erlaubteMaschinen.size() == 0){
			return false;	//eine Leere Liste muss immer ueberschrieben werden, da ungueltig
		}
		int countNonNullEntries = 0;
		for (final Maschine maschine : erlaubteMaschinen) {
			if(maschine == null){ continue;}
			countNonNullEntries++;
			if(!this.erlaubteMaschinen.contains(maschine)){
				return false;
			}
		}
		return countNonNullEntries == this.erlaubteMaschinen.size();
	}
	
//=============================================================================
	
	private class KonfigChangeCounterObserver implements IMaschinenKonfigurationObserver{
		private int counter = 0;
		@Override public void onMaschinenKonfigurationChange(final MaschinenKonfiguration neueMaschinenKonfiguration) {
			this.counter++;
		}	
	}
}
