package de.desy.mst.libs.ser.launcher;



/**
 * @author labudda
 * Ueber Optionen kann das Verhalten der Applikation gesteuert werden.
 * Jede Option ist als Umgebungsvariable oder als Property der jnlp Datei definierbar.  
 * Abgesehen von Optionen zur Fehlersuche soll die Option in der jnlp Datei definiert werden.
 * Um eine Interferenz mit Optionen anderer Applikationen zu verhindern 
 * beginnen alle Launcher Optionen mit 'launcher.'
 * Seit dem Einsatz von Java 7u51 im Kontrollsystem muss beruecksichtigt werden, 
 * dass properties mit 'jnlp.' beginnen muessen. 
 * Z.Zt. werden von dieser Klasse properties mit und ohne jnlp prefix unterstuetzt
 */
public enum EOptions {
	/**
	 * Nur fuer die Fehlersuche.
	 * Beendet den Java Launcher nicht, nachdem die Applikation erfolgreich gestartet wurde.
	 */
	StayOnSuccess,			 
	/**
	 * Nur fuer die Fehlersuche.
	 * Beendet den Launcher im Fehlerfall  nicht. 
	 */
	StayOnFailure,			
	/**
	 * Startet die Applikation nicht sondern kopiert nur die Dateien in den lokalen Cache
	 */
	IgnoreLaunch,			
	/**
	 * Das erste Argument entspricht einer nativen Applikation auf der Plattform.
	 * Deshalb wird das erste Argument nicht in den lokalen cache geladen.
	 * Es ist Aufgabe der jnlp Datei eine gueltige Plattform zu definieren.
	 */
	UseNativeCommand,			
	/**
	 * Definiert die Art wie eine Applikation gestartet wird. Default ist Execute. 
	 * Mit dieser Option wird ueber die Dateierweiterung entschieden womit die Datei geoeffnet wird.
	 * Argumente koennen nicht uebergeben werden, 
	 * wenn die Datei ueber die Dateierweiterung geoeffnet wird.  
	 */
	UseOpen,
	/**
	 * Eine Liste von Argumenten, die der zu startenden Applikation uebergeben werden.
	 * Die Argumente werden nicht uebergeben, wenn eine Datei ueber die Dateierweiterung
	 * geoeffnet wird (UseOpen) 
	 */
	MainArgument;				
	
	/**
	 * @author labudda
	 * Hashtables, deren keys alle lowercase sind 
	 */
	private final static class DefinedOptions  {
		private final static java.util.logging.Logger log = java.util.logging.Logger.getLogger(DefinedOptions.class.getName());
		private final java.util.HashMap<String, String> options;
		/**
		 * Sucht die relevanten optionen aus den system properties und dem environment heraus
		 * Zwischen java7u21 und java7u51 hat sich die behandlung von properties durch webstart geaendert
		 * sichere properties, die der java vm uebergeben werden, muessen mit jnlp beginnen 
		 */
		private DefinedOptions(final String LAUNCHER_PREFIX) {
			this.options = new java.util.HashMap<String, String>();
			final java.util.HashMap<String, String> properties = new java.util.HashMap<String, String>();
			for (String name : System.getProperties().stringPropertyNames()) {
				properties.put(name, System.getProperties().getProperty(name));
			}
			DefinedOptions.log.info("Going to define options by systemproperties");
			DefinedOptions.defineOptions(this.options, LAUNCHER_PREFIX, properties);
			DefinedOptions.defineOptions(this.options, "jnlp." + LAUNCHER_PREFIX, properties);
			
			DefinedOptions.log.info("Going to define options by environment");
			DefinedOptions.defineOptions(this.options, LAUNCHER_PREFIX, System.getenv());
			DefinedOptions.defineOptions(this.options, "jnlp." + LAUNCHER_PREFIX, System.getenv());

		}
		
		/**
		 * @param relevant options to be returned.
		 * The map will be modified
		 * @param prefix of keys to look for
		 * @param all key/values to search in
		 */
		private final static void defineOptions(final java.util.Map<String,String> relevant, final String prefix, final java.util.Map<String, String> all) {
			final String lcasePrefix = prefix.toLowerCase();
			final int prefixSize = prefix.length();
			for (String key: all.keySet()) {
				if (key.length() <= prefixSize) {
					// the size of key must always be larger than size of prefix
					// to be relevant
					// so ignore
				} else if (lcasePrefix.equalsIgnoreCase(key.substring(0, prefixSize))) {
					final String option = key.substring(prefixSize).toLowerCase();
					final String value = all.get(key);
					if (relevant.containsKey(option)) {
						DefinedOptions.log.warning(option + " will be overwritten by " + key);
					} else {
						// not defined, yet
					}
					DefinedOptions.log.info(option + " defined " + value);
					relevant.put(option, value);
				} else {
					// ignore
				}
			}
		}
		/**
		 * @param name of option
		 * @param defaultValue
		 * @return value of option or default
		 */
		public String getValue(String name, String defaultValue) {
			final String key = name.toLowerCase();
			if (this.options.containsKey(key)) {
				return this.options.get(key);
			} else {
				return defaultValue;
			}
		}
	}
	
	private final static DefinedOptions options = new DefinedOptions("launcher.");

	/**
	 * @return True, wenn diese Option definiert ist
	 */
	public boolean isDefined() {
		return this.getValue(null) != null;
	}
	
	/**
	 * @param defaultValue if not defined
	 * @return value of option or default
	 */
	public String getValue(final String defaultValue) {
		return EOptions.options.getValue(this.name(), defaultValue);
	}
}