package de.desy.mst.libs.ser.tclientlib;

import de.desy.mst.libs.ser.tclientlib.utils.TineStringUtils;
import de.desy.tine.client.TLink;
import de.desy.tine.dataUtils.TDataType;
import de.desy.tine.definitions.TAccess;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.definitions.TMode;

/**
 * An Abstract Base Class for TINE Writers.
 * 
 * @author jwlg
 */
public abstract class ATineWriter implements ITineAccessor {

    public static final int DEFAULT_TIMEOUT = 1000;

    private int timeoutMs = DEFAULT_TIMEOUT;
    private int errorCode = 0;
    private TLinkAddress linkAddress = null;
    private String lastErrorMessage = "";
    private TLink outputLink = null;
    private TConnectionMode connectionMode = TConnectionMode.DATAGRAM;

    public ATineWriter(String fullDevicePath) {
        if (fullDevicePath == null)
            throw new NullPointerException();
        this.linkAddress = new TLinkAddress(fullDevicePath);
    }

    public ATineWriter(TLinkAddress linkAddress) {
        if (linkAddress == null)
            throw new NullPointerException();
        this.linkAddress = linkAddress;
    }

    protected int writeData(TDataType outputData) throws TineLinkException {
        try {
            close();
            outputLink = new TLink(linkAddress.getFullPath(), null, outputData, getAccessFlags());
            outputLink.removeOnClose = false;
            
            if (this.connectionMode == TConnectionMode.STREAM) {
                errorCode = outputLink.execute(timeoutMs, TMode.CM_CONNECT);
            }  else {
                errorCode = outputLink.execute(timeoutMs);
            }

            lastErrorMessage = TineStringUtils.getInstance().removeTrailingZero(outputLink.getLastError());

        } catch (Exception e) {
            String errorMessage = "Error in TINE";
            if (e.getMessage() != null) {
                errorMessage += ": " + e.getMessage();
            }
            throw new TineLinkException(this.linkAddress, TErrorList.runtime_error, errorMessage, e);
        } finally {
            close();
        }
        return errorCode;
    }

    /**
     * Returns the TINE access flags 
     * used by this writer.
     * @see TAccess
     * @return A combination of TAccess modes.
     * Normally only CA_WRITE and CA_CONNECT are used.
     */
    // Called by writeData()
    public short getAccessFlags() {
        short result = TAccess.CA_WRITE;
        //if (this.connectionMode == TConnectionMode.STREAM) {
        //    result |= TAccess.CA_CONNECT;
        //}
        return result;
    }

    /**
     * Closes the underlying TINE link.
     * 
     */
    public void close() {
        if (outputLink != null) {
            outputLink.close();
            outputLink = null;
        }
    }

    public int getTimeoutMs() {
        return timeoutMs;
    }

    public void setTimeoutMs(int timeoutMs) {
        this.timeoutMs = timeoutMs;
    }

    public int getOutputStatus() {
        return errorCode;
    }

    public boolean hasError() {
        return (errorCode != 0);
    }

    public int getLinkStatus() {
        return errorCode;
    }

    public String getErrorMessage() {
        return lastErrorMessage;
        /*
         * if (outputLink != null) { return outputLink.getError(errorCode); }
         * else { return TErrorList.getErrorString(errorCode); }
         */
    }

    /**
     * Queries the input array size from the device server. This method is
     * useful for array writers to find out their buffer size dynamically. It
     * works only if the property is not overloaded.
     * 
     * @return The array size of the property.
     * 
     @Deprecated public int queryArraySize() throws TineLinkException { // TODO
     *             make usable for overloaded properties (filter type) int
     *             result;
     * 
     *             try { TPropertyQuery propertyInfos[] =
     *             TQuery.getPropertyInformation( linkAddress.getContext(),
     *             linkAddress.getServer(), linkAddress.getDevice(),
     *             linkAddress.getProperty());
     * 
     *             if ((propertyInfos != null) && (propertyInfos.length > 0)) {
     *             result = propertyInfos[0].prpSizeIn; } else { result = 1; } }
     *             catch (Exception e) { throw new TineLinkException(e); }
     *             return result; }
     */

    /**
     * Cleans up resources on deletion. Closes the underlying TLink.
     */
    protected void finalize() throws Throwable {
        try {
            close();
        } finally {
            super.finalize();
        }
    }

    /**
     * Returns the address to which the underlying TINE link connects.
     */
    public TLinkAddress getLinkAddress() {
        return linkAddress;
    }

    /**
     * Returns the connection mode.
     */
    public TConnectionMode getConnectionMode() {
        return connectionMode;
    }

    /**
     * Sets the connection mode. The default value is TConnectionMode.DATAGRAM.
     */
    public void setConnectionMode(TConnectionMode connectionMode) {
        this.connectionMode = connectionMode;
    }

}
