/*
 * Created on May 22, 2007
 *
 */
package de.desy.mst.libs.ser.tclientlib;

/**
 * A TINE Address.
 * 
 * This class resolves a complete TINE address into components and vice versa.
 * 
 * @author jwlg
 */
public class TLinkAddress {

    String context = "";
    String server = "";
    String device = "";
    String property = "";

    /**
     * Default Constructor. Creates an empty TLinkAddress.
     */
    public TLinkAddress() {

    }

    /**
     * Copy Constructor
     * 
     * @param tLinkAddress
     *            a <code>TLinkAddress</code> object
     */
    public TLinkAddress(TLinkAddress tLinkAddress) {

        this.context = tLinkAddress.context;
        this.server = tLinkAddress.server;
        this.device = tLinkAddress.device;
        this.property = tLinkAddress.property;
    }

    /**
     * Constructor with full path.
     * 
     * @param fullPath
     *            A full TINE address path in the form
     *            "/<context>/<server>/<device>/<property>" Note that the
     *            context is mandatory. The old form without a context is not
     *            supported.
     *            <P>
     *            <B>NOTE</B> The validity of the address is not checked here.
     *            The constructor accepts addresses in the form
     *            "/<context>[/<server>[/<device>[/<property>]]]". It fills the
     *            fields context, server, device, and property in this sequence.
     *            Missing fields are left empty.
     */
    public TLinkAddress(String fullPath) {

        // TODO replace by setAddress() in next version!
        setAddress(fullPath);
        
        /*
        String[] components = fullPath.split("/");
        if (components.length > 4) {
            context = components[1];
            server = components[2];
            device = components[3];
            property = components[4];
        } else if (components.length == 4) {
            context = components[1];
            server = components[2];
            device = components[3];
            // property = components[3];
        } else if (components.length == 3) {
            context = components[1];
            server = components[2];
        } else if (components.length == 2) {
            context = components[1];
        }
        */

    }
    
    /** 
     * Returns a String array element
     * with range checking. 
     * @param array
     * @param index
     * @return Array element or empty string ("") if the index is out of range.
     */
    private String getElement(String[] array, int index) {
        String result;
        if ((index >= 0) && (index < array.length)) {
            result = array[index];
        } else {
            result = "";
        }
        return result;
    }

    
    /**
     * Assigns an address as a full path.
     * @param fullPath Full path.
     */
    public  void setAddress(String fullPath) {

        String[] components = fullPath.split("/",-1);
        String propertyComponent = components[components.length - 1];
        int currentIndex = 0;
        
        if (propertyComponent.matches(".*\\[.*\\]")) {
            int idxStart = propertyComponent.indexOf('[');
            int idxEnd = propertyComponent.lastIndexOf(']');
            if (idxStart < idxEnd) {
                property = propertyComponent.substring(idxStart+1, idxEnd);
                device = propertyComponent.substring(0,idxStart);
            } else {
                property = "";
                device = propertyComponent;
            }
            
            currentIndex = components.length - 1;
            server = getElement(components, --currentIndex);
            context = getElement(components, --currentIndex);
            
        } else {
            
            currentIndex = components.length;
            
            property = getElement(components, --currentIndex);
            device = getElement(components, --currentIndex);
            server = getElement(components, --currentIndex);
            context = getElement(components, --currentIndex);
        }
 
   }

    /**
     * Constructor.
     * 
     * Constructs a TLINK to a property which is not assigned to a device.
     * 
     * @param context
     *            The context.
     * @param server
     *            The export name of the equipment module.
     * @param property
     *            The property name.
     */
    public TLinkAddress(String context, String server, String property) {

        super();
        setContext(context);
        setServer(server);
        this.device = "";
        setProperty(property);
    }

    /**
     * Constructor.
     * 
     * @param context
     *            The context.
     * @param server
     *            The export name of the equipment module.
     * @param device
     *            The device name.
     * @param property
     *            The property name.
     */
    public TLinkAddress(String context, String server, String device, String property) {

        super();
        setContext(context);
        setServer(server);
        setDevice(device);
        setProperty(property);
    }

    /**
     * Returns the full TINE address path.
     */
    public String getFullPath() {

        StringBuffer resultBuffer = new StringBuffer();
        resultBuffer.append("/");
        resultBuffer.append(context);
        resultBuffer.append("/");
        resultBuffer.append(server);
        if ((device != null) && (!device.equals(""))) {
            resultBuffer.append("/");
            resultBuffer.append(device);
        }
        if ((property != null) && (!property.equals(""))) {
            resultBuffer.append("/");
            resultBuffer.append(property);
        }
        return resultBuffer.toString();
    }
    
    /**
     * Returns the device path in the form /CONTEXT/SERVER/DEVICE.
     */
    public String getDevicePath() {
         StringBuffer resultBuffer = new StringBuffer();
         resultBuffer.append("/");
         resultBuffer.append(context);
         resultBuffer.append("/");
         resultBuffer.append(server);
         if ((device != null) && (!device.equals(""))) {
             resultBuffer.append("/");
             resultBuffer.append(device);
         }
         return resultBuffer.toString();
    }
    
    /**
     * Returns the server path in the form /CONTEXT/SERVER.
     */
    public String getServerPath() {
        StringBuffer resultBuffer = new StringBuffer();
        resultBuffer.append("/");
        resultBuffer.append(context);
        resultBuffer.append("/");
        resultBuffer.append(server);
        return resultBuffer.toString();
    }

    /**
     * Returns the context.
     */
    public String getContext() {

        return context;
    }

    /**
     * Assigns the context.
     * 
     * @param context
     *            The new context name.
     */
    public void setContext(String context) {

        if (context != null)
            this.context = context;
        else
            this.context = "";
    }

    /**
     * Returns the device name.
     */
    public String getDevice() {

        return device;
    }

    /**
     * Assigns the device name.
     * 
     * @param device
     */
    public void setDevice(String device) {

        if (device != null)
            this.device = device;
        else
            this.device = "";
    }

    /**
     * Returns the property.
     */
    public String getProperty() {

        return property;
    }

    /**
     * Assigns the property.
     * 
     * @param property
     */
    public void setProperty(String property) {

        if (property != null)
            this.property = property;
        else
            this.property = "";
    }

    /**
     * Returns the server (=export name).
     */
    public String getServer() {

        return server;
    }

    /**
     * Assigns the server (=export name)
     * 
     * @param server
     */
    public void setServer(String server) {

        if (server != null)
            this.server = server;
        else
            this.server = "";
    }

    /**
     * Constructs a <code>String</code> with all attributes in name = value
     * format.
     * 
     * @return a <code>String</code> representation of this object.
     */
    public String toString() {

        return getFullPath();
    }

    /**
     * Returns true if a given TINE path is syntactically valid. A full TINE
     * path has the form "/<context>/<server>/<device>/<property>" The device is
     * allowed to be an empty string. All other fields must not be empty.
     * 
     * @param fullTinePath
     *            The TINE path to be checked.
     * @return true if the path is valid.
     */
    public static boolean isValidTinePath(String fullTinePath) {

        if (fullTinePath == null) {
            return false;
        }
        String[] components = fullTinePath.split("/");
        return (fullTinePath.startsWith("/") && (components.length == 5)
            && (!components[1].isEmpty()) && (!components[2].isEmpty()) && (!components[4]
            .isEmpty()));
        
        // NOTE: components[3] (device) is allowed to be empty!

    }

    /**
     * Returns true if this is a proper TINE address.
     */
    public boolean isProperAddress() {

        return !((context == null) || context.isEmpty() || (server == null) || server.isEmpty()
            || (property == null) || property.isEmpty());

        // NOTE: device may be empty.

    }

    /**
     * Creates a valid TLinkAddress. This method guarantees that a TLinkAddress
     * will only be created if the supplied TINE path is syntactically correct.
     * Otherwise an InvalidAddressException will be thrown.
     * 
     * @param fullTinePath
     *            A full TINE path.
     * @return A TLinkAddress
     * @throws InvalidAddressException
     *             Thrown if the TINE path is not valid.
     */
    public static TLinkAddress createValidated(String fullTinePath) throws InvalidAddressException {

        TLinkAddress result = null;
        if (isValidTinePath(fullTinePath)) {
            result = new TLinkAddress(fullTinePath);
        } else {
            throw new InvalidAddressException();
        }
        return result;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((context == null) ? 0 : context.hashCode());
        result = prime * result + ((device == null) ? 0 : device.hashCode());
        result = prime * result + ((property == null) ? 0 : property.hashCode());
        result = prime * result + ((server == null) ? 0 : server.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        TLinkAddress other = (TLinkAddress) obj;
        if (context == null) {
            if (other.context != null)
                return false;
        } else if (!context.equals(other.context))
            return false;
        if (device == null) {
            if (other.device != null)
                return false;
        } else if (!device.equals(other.device))
            return false;
        if (property == null) {
            if (other.property != null)
                return false;
        } else if (!property.equals(other.property))
            return false;
        if (server == null) {
            if (other.server != null)
                return false;
        } else if (!server.equals(other.server))
            return false;
        return true;
    }

}
