/*
 * Created on May 10, 2007
 *
 */
package de.desy.mst.libs.ser.tclientlib;

import de.desy.mst.libs.ser.tclientlib.utils.TineStringUtils;
import de.desy.tine.client.TLink;
import de.desy.tine.definitions.TErrorList;

/**
 * An Exception class for TINE clients. This exception is used for all data
 * access methods in this library. it is thrown when no valid data could be sent
 * or received, and it contains the error message and the error code from TINE.
 * 
 * @author jwlg
 */
public class TineLinkException extends Exception {

    private static final long serialVersionUID = 1L;
    private int errorCode = 0;
    private TLinkAddress linkAddress = null;

    public enum ErrorMap {

        UNKNOWN_ARRAY_SIZE(800, "Unknown Array Size"), 
        TINE_RUNTIME_EXCEPTION(
            TErrorList.runtime_error, "Runtime Error in TINE");

        private final int code;
        private final String message;

        private ErrorMap(int code, String message) {
            this.code = code;
            this.message = message;
        }

        public int getCode() {
            return code;
        }

        public String getMessage() {
            return message;
        }

    }

    /**
     * Default Constructor.
     */
    public TineLinkException(TLinkAddress linkAddress) {
        super();
        errorCode = 0;
        this.linkAddress = linkAddress;
    }

    /**
     * Constructor. Takes error code and message from link.
     * 
     * @param link
     *            A TLink.
     */
    public TineLinkException(TLink link, TLinkAddress linkAddress) {
        super(TineStringUtils.getInstance().removeTrailingZero(link.getLastError()));
        this.errorCode = link.getLinkStatus();
        this.linkAddress = linkAddress;
    }

    /**
     * Constructor. Takes error code and message from a TINE accessor.
     * 
     * @param accessor
     *            The TINE accessor.
     */
    public TineLinkException(ITineAccessor accessor) {
        super(accessor.getErrorMessage());
        this.errorCode = accessor.getLinkStatus();
        this.linkAddress = accessor.getLinkAddress();
    }

    /**
     * Constructor with message and error number.
     */
    public TineLinkException(TLinkAddress linkAddress, int errorCode, String message) {
        super(message);
        this.errorCode = errorCode;
        this.linkAddress = linkAddress;
    }

    /**
     * Constructor for exception wrapping.
     * 
     * @param errorCode
     *            TINE error code
     * @param message
     *            Message text
     * @param cause
     *            Wrapped exception
     */
    public TineLinkException(TLinkAddress linkAddress, int errorCode, String message, Throwable cause) {
        super(message, cause);
        this.errorCode = errorCode;
        this.linkAddress = linkAddress;
       
    }

    public TineLinkException(TLinkAddress linkAddress, ErrorMap errorCode, Throwable cause) {
        super(errorCode.getMessage(), cause);
        this.errorCode = errorCode.getCode();
        this.linkAddress = linkAddress;
   }

    public TineLinkException(TLinkAddress linkAddress, ErrorMap errorCode) {
        super(errorCode.getMessage());
        this.errorCode = errorCode.getCode();
        this.linkAddress = linkAddress;
   }

    /**
     * Creates an exception wrapper. This constructor is used to throw an
     * exception when TINE throws an unforeseen runtime exception. The exception
     * contains the error code TErrorList.runtime_error and the error message
     * "Runtime Error in TINE".
     * 
     * @param cause
     *            Cause
     */
    public TineLinkException(TLinkAddress linkAddress, Throwable cause) {
        this(linkAddress,ErrorMap.TINE_RUNTIME_EXCEPTION, cause);
    }

    /**
     * Returns the error code.
     */
    public int getErrorCode() {
        return errorCode;
    }

    public TLinkAddress getLinkAddress() {
        return linkAddress;
    }

    
}
