/*
 * Created on May 10, 2007
 *
 */
package de.desy.mst.libs.ser.tclientlib;

import java.util.LinkedList;

import de.desy.mst.libs.ser.tclientlib.utils.TineStringUtils;
import de.desy.tine.client.TLink;
import de.desy.tine.client.TLinkCallback;

/**
 * A reader group.
 * 
 * This class supports grouped reading of TINE data. A special feature of TINE
 * is the CM_GROUPED flag, which was invented to pack multiple packets into one
 * and thus make data transmission more efficient. All readers in a group are
 * set to asynchronous mode with a common update rate. Listeners are notified on
 * each update. They may either be assigned to the group or to each reader
 * individually.
 * 
 * The update rate of all assigned readers is defined by the group. A reader is
 * assigned to a group by calling TineReaderGroup.addReader(). This sets the
 * reader to asynchronous mode with a group trigger. A reader is removed from a
 * group by calling TineReaderGroup.removeReader(). Grouped readers are closed
 * by calling TineReaderGroup.close().
 * 
 * <p>
 * <b>Usage:</b>
 * </p>
 * 
 * <pre>
 * TineReaderGroup group = new TineReaderGroup();
 * IntReader reader1 = new IntReader(&quot;/CONTEXT/SERVER/DEVICE/PROPERTY-1&quot;);
 * FloatReader reader2 = new FloatReader(&quot;/CONTEXT/SERVER/DEVICE/PROPERTY-2&quot;);
 * 
 * group.setUpdateInterval(1000); // update interval for all readers in this group.
 * group.addReader(reader1); // add reader1 to group - sets reader 1 in asynchronous mode 
 * group.addReader(reader2); // add reader2 to group - sets reader 2 in asynchronous mode
 * </pre>
 * 
 * <p>
 * <b>Note:</b>
 * <ul>
 * <li>
 * Rumours say that grouped links are prone to strange connection errors.</li>
 * <li>
 * It makes no sense to add a reader to a group which has a different server
 * than the other readers - although it's not prohibited.</li>
 * <li>
 * Grouping overrides some of a reader's private settings with the group
 * settings and makes it impossible to set a reader to synchronous/asynchronous
 * mode manually.</li>
 * <li>
 * Don't close a reader that's in a group. Close the group instead.</li>
 * </ul>
 * 
 * @see ATineReader
 * @author jwlg
 */
public class TineReaderGroup implements TLinkCallback {

    private int linkStatus = 0; /* buffer for link status */
    private String errorMessage = ""; /* buffer for error message */
    private int updateInterval = 1000;
    private TConnectionMode connectionMode = TConnectionMode.DATAGRAM;

    private LinkedList<ATineReader> readers = new LinkedList<ATineReader>();
    private LinkedList<IReaderGroupListener> listeners = new LinkedList<IReaderGroupListener>();

    /**
     * TINE callback method. This method is called by the TINE link to which
     * this group is attached.
     */
    public final void callback(TLink link) {

        linkStatus = link.getLinkStatus();
        errorMessage = TineStringUtils.getInstance().removeTrailingZero(link.getLastError());

        for (ATineReader reader : readers) {
            reader.callbackFromGroup(link);
        }

        for (IReaderGroupListener l : listeners) {
            l.notifyUpdate(this);
        }
    }

    /**
     * Adds a listener.
     * 
     * @param listener
     *            The new listener.
     */
    public final void addListener(IReaderGroupListener listener) {
        listeners.add(listener);
    }

    /**
     * Removes a listener.
     * 
     * @param listener
     *            The listener to be removed.
     */
    public final void removeListener(IReaderGroupListener listener) {
        listeners.remove(listener);
    }

    /**
     * Returns the status of the TINE link.
     * 
     * @return The status of the TINE link.
     */
    public final int getLinkStatus() {
        return linkStatus;
    }

    /**
     * Returns the last error message of the TINE link.
     * 
     * @return The last error message of the TINE link.
     */
    public final String getLinkError() {
        return errorMessage;
    }

    /**
     * Returns true if the calling TINE link has an error.
     */
    public final boolean hasError() {
        return (linkStatus != 0);
    }

    /**
     * Adds a reader to a group.
     * <p>
     * <b>Note:</b>
     * </p>
     * When a reader is added to a group, the reader is closed and re-opened
     * with the group settings. The group settings override the reader's
     * settings like asynchronous/synchronous, update interval, and connection
     * mode.
     * 
     * @param reader
     * @throws TineLinkException
     */
    public synchronized void addReader(ATineReader reader) throws TineLinkException {
        if (!readers.contains(reader)) {
            reader.close();
            readers.add(reader);
            reader.setGrouped(this);
        }
    }

    /**
     * Removes a reader from a group. Closes the reader.
     */
    public synchronized void removeReader(ATineReader reader) {
        if (readers.contains(reader)) {
            readers.remove(reader);
            reader.setUngrouped();
            reader.close();
        }
    }

    /**
     * Closes all readers and removes them from the group.
     */
    public synchronized void close() {
        for (ATineReader reader : readers) {
            reader.close();
        }
        readers.clear();
    }

    /**
     * Returns the update interval.
     */
    public int getUpdateInterval() {
        return updateInterval;
    }

    /**
     * Assigns the update interval. Sets the common update interval which all
     * readers use. The update interval is normally assigned to a group before
     * readers are added. When readers are already attached, this method closes
     * and re-opens them.
     * 
     * @param updateInterval
     * @throws TineLinkException
     */
    public void setUpdateInterval(int updateInterval) throws TineLinkException {
        if (this.updateInterval != updateInterval) {
            this.updateInterval = updateInterval;
            for (ATineReader reader : readers) {
                reader.close();
                reader.setGrouped(this);
            }
        }
    }

    /**
     * Closes the group.
     * 
     * @see #close()
     */
    public final void finalize() {
        close();
    }

    /**
     * Returns the connection mode.
     */
    public TConnectionMode getConnectionMode() {
        return connectionMode;
    }

    /**
     * Sets the connection mode. The connection mode must be set before readers
     * are added to the group. The connection mode of the group overrides the
     * connection mode in a reader. The default value is
     * TConnectionMode.DATAGRAM.
     */
    public void setConnectionMode(TConnectionMode connectionMode) {
        this.connectionMode = connectionMode;
    }

}
