package de.desy.mst.libs.ser.tclientlib.readers;

import de.desy.tine.dataUtils.TDataType;
import de.desy.mst.libs.ser.tclientlib.utils.TineQueryUtils;
import de.desy.mst.libs.ser.tclientlib.utils.TineStringUtils;
import de.desy.mst.libs.ser.tclientlib.ATineReader;
import de.desy.mst.libs.ser.tclientlib.TineLinkException;
import de.desy.mst.libs.ser.tclientlib.TLinkAddress;


/**
 * A Text Reader.
 *
 * The TextReader reads a TINE character array 
 * and maps it to a Java String.
 *
 */
public class TextReader extends ATineReader {
    
    private int arraySize = 1;
    private char[] inputBuffer = null; 
    private char[] outputBuffer = null; 
    
    /**
     * Constructor.
     * Constructs a TextReader with a specified array size.
     * @param linkAddress The TINE address
     * @param arraySize The array size.
     */
    public TextReader(TLinkAddress linkAddress, int arraySize) {
        super(linkAddress);
        if (arraySize > 0) {
            this.arraySize = arraySize;
            createBuffers();
        }
        setup(createTDataType());
    }

    /**
     * Constructor.
     * Constructs a TextReader. The array size is 
     * determined dynamically by a server request.
     * <P>
     * <B>NOTE:</B> This constructor is unsafe. 
     * The array size can only be determined
     * correctly when the server runs at the time when
     * the constructor is called. Otherwise TClientLib 
     * sets the array size to the default value 1.
     * For safe operation, you must know the array
     * size in advance.
     * </P>
     * @param linkAddress The TINE address
     * @throws TineLinkException if the array size cannot be determined.
     */
    public TextReader(TLinkAddress linkAddress) throws TineLinkException {
        super(linkAddress);
       	this.arraySize = TineQueryUtils.getInstance().getPropertyArraySize(linkAddress);
        if (this.arraySize > 0) {
        	createBuffers();
        }
        setup(createTDataType());
    }

    
    /**
     * Constructor.
     * Constructs a TextReader with a specified array size.
     * @param fullDevicePath The full TINE path in the 
     *  form /context/server/device/property
     * @param arraySize The array size.
     */
    public TextReader(String fullDevicePath, int arraySize) {
        super(fullDevicePath);
        if (arraySize > 0) {
            this.arraySize = arraySize;
            createBuffers();
        } 
        setup(createTDataType());
    }

    /**
     * Constructor.
     * Constructs a TextReader. The array size is 
     * determined dynamically by a server request.
     * @param fullDevicePath The full TINE path in the 
     *  form /context/server/device/property
     * @throws TineLinkException if the array size cannot be determined.
     */
    public TextReader(String fullDevicePath) throws TineLinkException {
        super(fullDevicePath);
        this.arraySize = TineQueryUtils.getInstance().getPropertyArraySize(new TLinkAddress(fullDevicePath));
        createBuffers();
        setup(createTDataType());
    }

    /**
     * Returns the values.
     * If the reader is in asynchronous mode (attached), 
     * the last updated values are returned. If the reader
     * in in synchronous mode, a synchronous read operation
     * (execute) is performed.
     * In both cases, when an error occured, no valid data
     * ara available, no values are returned,
     * and a TineLinkException is thrown instead.
     * @throws TineLinkException
     */
    public String getValue() throws TineLinkException {
        String result = null;
        if (isSynchronous()) {
            readData();
            copyData();
        }
        if (hasError()) {
            throw new TineLinkException(getLinkAddress(), getLinkStatus(), getErrorMessage());
        } 
        result = TineStringUtils.getInstance().convertToString(outputBuffer);
        return result;
    }
     
    /**
     * Copies the input array to the output array.
     * This method is called when the reader is in asynchronous mode.
     */
    protected synchronized void copyData() {
        outputBuffer = new char[arraySize];
        for (int i = 0; i< arraySize; i++) {
             outputBuffer[i] = inputBuffer[i];
        }
    }
    
    private void createBuffers() {
	if (arraySize > 0) {
          inputBuffer = new char[arraySize];
          copyData();
	} else {
	    inputBuffer = null;
	    outputBuffer = null;
	}
    }
    
    private char[] getInputBuffer() {
        if (inputBuffer == null) createBuffers();
        return inputBuffer;
    }
    
    private TDataType createTDataType() {
        TDataType result = new TDataType(getInputBuffer());
        return result;
    }
    
}
