/*
 * Created on Nov 4, 2009
 *
 */
package de.desy.mst.libs.ser.tclientlib.utils;

import de.desy.mst.libs.ser.tclientlib.TLinkAddress;
import de.desy.mst.libs.ser.tclientlib.TineLinkException;
import de.desy.tine.definitions.TErrorList;
import de.desy.tine.queryUtils.TPropertyQuery;
import de.desy.tine.queryUtils.TQuery;

/**
 * Utilities in connection with TINE services. Checks addresses and services.
 * 
 * @author jwlg
 */
public class TineQueryUtils {

    private static TineQueryUtils theInstance = new TineQueryUtils();

    /**
     * This is a singleton. Constructor is private.
     */
    private TineQueryUtils() {
    }

    /**
     * Returns the instance.
     */
    public static TineQueryUtils getInstance() {
        return theInstance;
    }


    /**
     * Returns true if a connection to a property is possible. Tests if a
     * server runs and has the requested property. [Doesn't test the device
     * name!]
     * 
     * @param fullPath
     *            TINE address as full path.
     * @return True if the connection is possible.
     */
    public boolean isPropertyAvailable(String fullPath) {
        TLinkAddress address = new TLinkAddress(fullPath);
        return isPropertyAvailable(address);
    }

    /**
     * Returns true if a connection to a property is possible. Tests if a
     * server runs and has the requested property. [Doesn't test the device
     * name!]
     * 
     * @param address
     *            TINE address.
     * @return True if the connection is possible.
     */
    public boolean isPropertyAvailable(TLinkAddress address) {
        boolean result = false;

        try {
            TPropertyQuery[] info = TQuery.getPropertyInformation(
                address.getContext(),
                address.getServer(),
                address.getDevice(),
                address.getProperty(), 500);
            result = ((info != null) && (info.length != 0));
        } catch (Exception e) { // Be aware of unforeseen runtime exceptions
            result = false;
        }
        return result;
    }
    
    /**
     * Queries the output array size from the device server. This works only if
     * the property is not overloaded.
     * 
     * @return The array size of the property.
     */
    public int getPropertyArraySize(TLinkAddress linkAddress) throws TineLinkException {

        // TODO make usable for overloaded properties (filter type)
        int result = 0;
        TPropertyQuery[] propertyInfos = null;

        try {
            propertyInfos = TQuery.getPropertyInformation(linkAddress.getContext(), linkAddress
                .getServer(), linkAddress.getDevice(), linkAddress.getProperty());
            if ((propertyInfos != null) && (propertyInfos.length > 0)) {
                result = propertyInfos[0].prpSize;
            } else {
                result = 0;
            }
        } catch (Exception e) {
            String errorMessage = "Runtime Error in TINE";
            if (e.getMessage() != null) {
                errorMessage += ": " + e.getMessage();
            }
            throw new TineLinkException(linkAddress, TErrorList.runtime_error, "Runtime Error in TINE", e);
        }

        return result;
    }
    
    /**
     * Returns true if data from a stock property can be read. 
     * 
     * @param address
     *            TINE address.
     * @return True if the server responds within 1s.
     */
    public boolean isServerAvailable(TLinkAddress address) {
        boolean result = false;
        try {
            result = (TQuery.pingServer(address.getContext(), address.getServer(), 1000) == 0);
        } catch (Exception e) { // Be aware of unforeseen runtime exceptions
            result = false;
        }
        return result;
    }

}
