package de.desy.mst.libs.ser.tclientlib.writers;
import java.util.Arrays;

import de.desy.tine.dataUtils.TDataType;
import de.desy.mst.libs.ser.tclientlib.ATineWriter;
import de.desy.mst.libs.ser.tclientlib.TineLinkException;
import de.desy.tine.definitions.TFormat;
import de.desy.mst.libs.ser.tclientlib.TLinkAddress;

/**
 * A text writer.
 *
 * The TextWriter writes Java String as a TINE 
 * character array to to a TINE server.
 *
 */
public class TextWriter extends ATineWriter {
    
    private int arraySize = 0;
    private char[] outputBuffer = null; // Data buffer for TINE

    /**
     * Constructor.
     * Constructs a TextWriter with a specified array size.
     * @param linkAddress The TINE address
     * @param arraySize The array size.
     */
    public TextWriter(TLinkAddress linkAddress, int arraySize) {
        super(linkAddress);
        this.arraySize = arraySize;
    }

    /**
     * Constructor.
     * Constructs a TextWriter. The array size is 
     * determined dynamically by a server request.
     * @param linkAddress The TINE address
     */
    public TextWriter(TLinkAddress linkAddress) {
        super(linkAddress);
    }


    
    /**
     * Constructor.
     * Constructs a TextWriter with a specified array size.
     * @param fullDevicePath The full TINE path in the 
     *  form /context/server/device/property
     * @param arraySize The array size.
     */
    public TextWriter(String fullDevicePath, int arraySize) {
        super(fullDevicePath);
        this.arraySize = arraySize;
    }

    /**
     * Constructor.
     * Constructs a TextWriter. The array size is 
     * determined dynamically by a server request.
     * @param fullDevicePath The full TINE path in the 
     *  form "/context/server/device/property"
     */
    public TextWriter(String fullDevicePath) {
        super(fullDevicePath);
    }

    
    /**
     * Writes an array of values to the server.
     * If the write operation was not successful,
     * a TineLinkException is thrown.
     * <BR><B>Note:</B> This method uses the size of
     * the given array. To determine which array size the
     * server expects, use getArraySize().
     * @throws TineLinkException
     */
    public synchronized void setValue(String newValue) throws TineLinkException {
        
        char[] convertedText = null;
        
        convertedText = newValue.toCharArray();

        if (arraySize > 0) {
            outputBuffer = Arrays.copyOf(convertedText, arraySize);
        } else {
            outputBuffer = Arrays.copyOf(convertedText, convertedText.length);
        }
        
        if (writeData(createTDataType()) != 0) {
            throw new TineLinkException(getLinkAddress(), getLinkStatus(), getErrorMessage());
        }
    }


    /**
     * Returns the array size.
     * This method returns the array size as 
     * specified or dynamically detected.
     */
    public int getArraySize() {
        return arraySize;
    }
    
    private TDataType createTDataType() {
        TDataType result = null;
        result = new TDataType(outputBuffer);
        result.dFormat = TFormat.CF_TEXT;
        return result;
    }
    
}
