/*
 * (c) by DESY 2016 - Group MCS 1, Hamburg, Germany
 * 
 * For more information about the license you should have a look at the
 * following web page:
 * 
 *         http://www.desy.de/legal/license/index_eng.html
 * 
 * If no concrete license can be found at this web page or the connection is
 * broken, then the following license is relevant:
 * 
 * Copyright (C) 2016 DESY.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301  USA
 */
package de.desy.mst.libs.util.helper;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Formatter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.regex.Pattern;

import org.apache.commons.lang3.SystemUtils;

/**
 * Helper framework for Java logger instances. Implements getter functions for 
 * fetching a logger instance depending on the specified class. A normal user 
 * will mostly call the {@link #getLogger(Class)} method to retrieve a logger 
 * instance.
 * 
 * <p>
 * 
 * If the log is also written to a file by defining the method argument of 
 * {@code isStorable} then all logging is persistently stored on the mounted 
 * drive. The drive depends on the OS:
 * <ul>
 * <li>On WINDOWS systems it always uses the root drive L and the log directory</li>
 * <li>On UNIX systems it always uses the folder /var/tmp/log</li>
 * </ul>
 * Underneath this root folder the main class name is taken to log the messages.
 * Therefore you should always call a first logger in the main class because
 * from its class name the file is build. See also below.
 * 
 * <p>  
 * 
 * Per default a flag for the MSTApp logger is set to {@code false} and can be specified by
 * calling one of both methods {@link #getLogger(java.lang.Class, java.util.logging.Level, de.desy.mst.libs.util.helper.LogProvider.ELoggingProperty, boolean) } and
 * {@link #getLogger(java.lang.Class, java.util.logging.Level, de.desy.mst.libs.util.helper.LogProvider.ELoggingProperty, boolean) }, both by setting
 * the third parameter to {@code true}.
 * 
 * <p>
 * 
 * <b>NOTE 1:</b> The usage of the property {@link ELoggingProperty#FILE_STORAGE}
 * effects from the first class who gets the logger during runtime. All
 * consecutive loggers will indirectly also write to the created file. Best call
 * for project logging is therefore in the class which include the main functionality.
 * 
 * <p>
 * 
 * <b>NOTE 2:</b> Furthermore the property {@link ELoggingProperty#MCS_LOGGING}
 * only has an effect on that class which gets the logger with this field set.
 * In other words, this flag is limited to the calling class. 
 * 
 * <b>NOTE 2:</b> Never use the method {@link #setLogFilename(String)}! The file name is filter from the
 * concrete package and main class name instance from its first call to {@link #getLogger(Class)}.
 * So you should always create one logger in the main class to clarify
 * the file name. If you do create a logger at a later stage in some
 * other class or in a sub-package the file name will change to the concrete
 * first calling class instance. This can result in unexpected file names. The
 * log file tries to find the corresponding log directory on windows drive L. If
 * it doesn't exist it will create one. In this directory the project name for
 * the class package name is filter to a directory name. 
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">Marcus Walla</a>
 * @version $Id: LogProvider.java 73863 2016-11-16 12:02:46Z walla $
 */
public final class LogProvider 
{
	private static final Comparator<String> COMPARE_STRING  = (final String str1, final String str2) -> (str1 != null && str2 != null) ? str1.compareToIgnoreCase(str2) : 0;

	private static String logFilename                       = null;
	private static Map<Class<?>, Logger> clazzes            = new HashMap<>();
	
	private static final Logger anonymous                   = Logger.getLogger("");
	private static final SortedSet<String> DESY_ACCELERATOR = new TreeSet<>(COMPARE_STRING);
	private static final SortedSet<String> DESY_FACILITY    = new TreeSet<>(COMPARE_STRING);
	private static final SortedSet<String> KNOWN_HOST       = new TreeSet<>(COMPARE_STRING);
	
    private static final String PRE_PATH                    ;
	public static final boolean DEFAULT_TIME_AND_DATE       = false;
	public static final String LOG_DIRECTORY                = "log";
	
	// --- CLASS INITIALIZATION ------------------------------------------------
	static 
    {
		boolean hasConsoleLogger = false;
		
		for(final Handler handler: LogProvider.anonymous.getHandlers())
			if (handler instanceof ConsoleHandler) {
				handler.setFormatter(new MCSFormatterConsole(false));
				hasConsoleLogger = true;
				break;
			}
		if (!hasConsoleLogger) {
			final Handler consoleHandler = new ConsoleHandler();
			consoleHandler.setFormatter(new MCSFormatterConsole(false));
			LogProvider.anonymous.addHandler(consoleHandler);
		}
		
		if (SystemUtils.IS_OS_WINDOWS)
			PRE_PATH = "L:/log";
		else if (SystemUtils.IS_OS_UNIX)
			PRE_PATH = "/var/tmp/log";
		else
			PRE_PATH = "/log";
		
		DESY_ACCELERATOR.addAll(Arrays.asList("AUXILIARY", "COMMON", "DESY2", "DORIS", "FLASH", "HERA", "LIBS", "LINAC2", "PETRA", "PEX", "PITZ", "REGAE"));
		DESY_FACILITY.addAll(Arrays.asList("DEVL", "DIAG", "INFO", "INSTR", "LIBS", "MAG", "OPR", "OPERATING", "PINTLK", "RF", "SER", "SERVICE", "TIM", "UTIL", "VAC", "ORBIT", "UND"));
		KNOWN_HOST.addAll(Arrays.asList("CLIENT", "SERVER"));
	}
	
	// --- ENUMERATION ---------------------------------------------------------
	public enum ELoggingProperty {
		MCS_LOGGING,
		
		/** 
		 * Reflects if the logging information is stored in a 
		 * separate file. The files are rotated. You can specify the log
		 * filename with the method {@link #setLogFilename(String)}. 
		 * As a suffix of the filename a number from 1 to 10 represents older 
		 * log filenames. 
		 * The size of a file is 1000000 bytes.
		 */
		FILE_STORAGE
	}
	
	// --- CLASS METHODS -------------------------------------------------------
//	private static Path getPreFilename() {
//		// TODO: should be tested on a Unix system!
//		if (SystemUtils.IS_OS_UNIX)
//			return createLogDir(System.getProperty("user.home") + LOG_DIRECTORY);
//		else if (SystemUtils.IS_OS_WINDOWS)
//			return createLogDir("L:/log" );
//		LogProvider.anonymous.log(Level.WARNING, "unexpected OS: " + System.getProperty("os.name") + ", version " + System.getProperty("os.version") + " (" + System.getProperty("os.arch") + ") -> return default log path" );
//		return Paths.get(System.getProperty("user.home"), LOG_DIRECTORY);
//	}
//	
//	private static Path createLogDir(final String fileDrive) {
//		final Path pathURI = Paths.get(URI.create("file:///" + fileDrive));
//		if (!Files.exists(pathURI)) {
//			try {
//				Files.createDirectory(pathURI, PosixFilePermissions.asFileAttribute(PosixFilePermissions.fromString("rwxr-xr-x")));
//			}
//			catch (final IOException exception) {
//				throw new InvalidPathException(pathURI.toString(), "unexpected path");
//			}
//		}
//		LogProvider.anonymous.log(Level.FINE, "successful created log directory '" + pathURI.toString() + "'");
//		return pathURI;
//	}
	
	// --- CONSTRUCTOR ---------------------------------------------------------
	private LogProvider() { /* NO OP */ }
	
	// --- NESTED CLASS -------------------------------------------------------
	@SuppressWarnings("unused")
	private static class WildCardFileFilter implements FileFilter {
		
		private final String pattern;
		
	    public WildCardFileFilter(final String pattern) {
	        this.pattern = pattern.replace("*", ".*").replace("?", ".");
	    }
	 
	    @Override public boolean accept(final File file) {
	    	return Pattern.compile(this.pattern).matcher(file.getName()).find();
	    }
	}
	
	// --- CLASS METHODS -------------------------------------------------------
	/**
	 * Creates the current Java logger instance of the specified class. For
	 * more information you should have a look at {@link #getLogger(Class, Level)}.
	 * The default log level is set to {@code Level.INFO}. The logging is stored
	 * in a file.
	 * @param clazz Logging class instance.
	 * @return returns an instance of the log classes.
	 */
	public static Logger getLogger(final Class<?> clazz) {
		return LogProvider.getLogger(clazz, Level.INFO, DEFAULT_TIME_AND_DATE);
	}
	
	/**
	 * Gets the current Java logger instance of the specified class and level. For
	 * more information you should have a look at {@link #getLogger(java.lang.Class, java.util.logging.Level, de.desy.mst.libs.util.helper.LogProvider.ELoggingProperty, boolean) }.
	 * The default log level is set to {@code Level.INFO}. The logging is stored
	 * in a file and only a timestamp is printed.
	 * @param clazz Logging class instance.
	 * @param level Log level.
	 * @return returns an instance of the log classes.
	 */
	public static Logger getLogger(final Class<?> clazz, final Level level) {
		return LogProvider.getLogger(clazz, level, DEFAULT_TIME_AND_DATE);
	}
	
	/**
	 * Gets the current Java logger instance of the specified class and level. For
	 * more information you should have a look at {@link #getLogger(java.lang.Class, java.util.logging.Level, de.desy.mst.libs.util.helper.LogProvider.ELoggingProperty, boolean) }.
	 * The default log level is set to {@code Level.INFO}. The logging is stored
	 * in a file and only a timestamp is printed.
	 * <p>
	 * 
	 * <b>NOTE 1:</b> The usage of the property {@link ELoggingProperty#FILE_STORAGE}
	 * effects from the first class who gets the logger during runtime. All
	 * consecutive loggers will indirectly also write to the created file. Best call
	 * for project logging is therefore in the class which include the main functionality.
	 * 
	 * <p>
	 * 
	 * <b>NOTE 2:</b> Furthermore the property {@link ELoggingProperty#MCS_LOGGING}
	 * only has an effect on that class which gets the logger with this field set.
	 * In other words, this flag is limited to the calling class. 
	 * 
	 * @param clazz Logging class instance.
	 * @param level Log level.
	 * @param logProperties Specifies the logging properties used for the returned instance.
	 * @return returns an instance of the log classes.
	 */
	public static Logger getLogger(final Class<?> clazz, final Level level, ELoggingProperty... logProperties) 
    {
		return LogProvider.getLogger(clazz, level, DEFAULT_TIME_AND_DATE, logProperties);
	}
	
	/**
	 * Gets the current Java logger instance of the specified class. The
	 * log messages are stored in a persistent file. The file name can be 
	 * set via {@link #setLogFilename(String)}. For more information you should 
	 * have a look at {@link #getLogger(java.lang.Class, java.util.logging.Level, de.desy.mst.libs.util.helper.LogProvider.ELoggingProperty, boolean) }.
	 * The default log level is set to {@code Level.INFO} and only a timestamp
	 * is printed.
	 * <p>
	 * 
	 * <b>NOTE 1:</b> The usage of the property {@link ELoggingProperty#FILE_STORAGE}
	 * effects from the first class who gets the logger during runtime. All
	 * consecutive loggers will indirectly also write to the created file. Best call
	 * for project logging is therefore in the class which include the main functionality.
	 * 
	 * <p>
	 * 
	 * <b>NOTE 2:</b> Furthermore the property {@link ELoggingProperty#MCS_LOGGING}
	 * only has an effect on that class which gets the logger with this field set.
	 * In other words, this flag is limited to the calling class. 
	 * 
	 * @param clazz Logging class instance.
	 * @param logProperties Specifies the logging properties used for the returned instance.
	 * @return returns an instance of the log classes.
	 */
	public static Logger getLogger(final Class<?> clazz, final ELoggingProperty... logProperties) {
		return LogProvider.getLogger(clazz, Level.INFO, DEFAULT_TIME_AND_DATE, logProperties);
	}
	
	/**
	 * Gets the current Java logger instance of the specified class and
	 * a level which is specified with its log level. 
	 * A logger instance can be created via the following skeleton:
	 * <pre>
public class A {
    private static final Logger log = LogProvider.getLogger(A.class);
    ...
    void abc() {
        log.info("only a comment...");
    }
}
	   </pre>
	 * You can influence the behavior of the logger with a property 
	 * file. This file must contain valid key-value-pairs for controlling
	 * the behavior of the logger. For more information about the file see at 
	 * <a href="http://de.wikibooks.org/wiki/Java_Standard:_Logging_API_von_Java">Wikibooks (DE)</a> web page. 
	 * One example property file is included in the root folder
	 * {@code examples} under {@code log/mpsdiag.properties}.
	 * This can easily be set via the command line argument of 
	 * {@code -Djava.util.logging.config.file=}<i>/path/to/app.properties</i>.
	 * <p>
	 * 
	 * <b>NOTE 1:</b> The usage of the property {@link ELoggingProperty#FILE_STORAGE}
	 * effects from the first class who gets the logger during runtime. All
	 * consecutive loggers will indirectly also write to the created file. Best call
	 * for project logging is therefore in the class which include the main functionality.
	 * 
	 * <p>
	 * 
	 * <b>NOTE 2:</b> Furthermore the property {@link ELoggingProperty#MCS_LOGGING}
	 * only has an effect on that class which gets the logger with this field set.
	 * In other words, this flag is limited to the calling class. 
	 * 
	 * <p>
	 * Moreover you can also specify the log filename with the method {@link #setLogFilename(String)}.
	 * @param clazz Logging class instance.
	 * @param level Log level.
	 * @param hasDateFormat Specifies if the logging message only includes the 
	 * 		time (false) or time with date (true).
	 * @param logProperties Specifies the logging properties used for the returned instance.
	 * @return returns an instance of the log classes.
	 */
	public static Logger getLogger(final Class<?> clazz, final Level level, final boolean hasDateFormat, final ELoggingProperty... logProperties) {
		if (clazzes.containsKey(clazz)) return clazzes.get(clazz);

		final Logger log = Logger.getLogger(clazz.getName());
		clazzes.put(clazz, log);
		
		if (level != null)
			log.setLevel(level);

		// Loggers are defined hierarchically and a child logger configures 
		// itself from its parent. The method Logger.setUseParentHandler(boolean):void 
		// allows to disable such default settings. Here it is switched off.
	    //log.setUseParentHandlers(false);
	    
        for(final ELoggingProperty property: logProperties)
        {
            removeAllHandlers(getRoot(log));
            if (checkMCSLogging(property)) {
                addHandler(getRoot(log), new MCSHandlerBridge());
            }

            if (!checkMCSLogging(property))
                addConsoleHandler(log, level, new MCSFormatterConsole(hasDateFormat));

            if (checkFileStorage(property))
                // Note: unfortunately fetching all handlers from root logger will 
                // indirectly add a new console handler to it.
                // => So the root logger will have two handlers: a console and file handler
                createRootFileHandler(log, hasDateFormat);
        }
        
	    return log;
	}
	
	private static void removeAllHandlers(final Logger log) {
		if (log == null || log.getHandlers().length == 0)
			return;
		
		for(final Handler handler: log.getHandlers())
			if (!(handler instanceof FileHandler))
				log.removeHandler(handler);
	}
	
	private static void addHandler(final Logger log, final Handler handler) {
		if (log == null)
			return;
		
		for(Handler tmpHandler: log.getHandlers())
			if (tmpHandler.getClass().isInstance(handler))
				return;
		
		log.addHandler(handler);
	}
	
	private static boolean checkMCSLogging(final ELoggingProperty property) {
		return property != null && property == ELoggingProperty.MCS_LOGGING;
	}
	
	private static boolean checkFileStorage(final ELoggingProperty property) {
		return property != null && property == ELoggingProperty.FILE_STORAGE;
	}
	
	private static void addConsoleHandler(final Logger log, final Level level, final Formatter formatter) {
	    boolean hasConsoleHandler = false;
	    for(Handler handler: log.getHandlers())
	    	if (handler instanceof ConsoleHandler) {
	    	    handler.setLevel(level);
	    	    handler.setFormatter(formatter);
	    	    hasConsoleHandler = true;
	    	}
	    if (!hasConsoleHandler) {
		    final Handler handler = new ConsoleHandler();
    	    handler.setLevel(level);
    	    handler.setFormatter(formatter);
		    log.addHandler(handler);
	    }
	}
 	
	private static void createRootFileHandler(final Logger log, final boolean hasDateFormat) {
		final Logger root = getRoot(log);

		// Note: unfortunately fetching all handlers from root logger will 
		// indirectly add a new console handler to it. See method
		// hasHandler(Logger, Class):void
		if (root == null || hasHandler(root, FileHandler.class))
			return;
		
    	// TODO: activate for Java 7
		LogProvider.anonymous.warning("storing of the log files is currently deprecated but will be activated in Java 7 - class " + log.getName());
		
    	try {
    		logFilename = createLogFilename(log);
			final FileHandler fileHdlr = new FileHandler(logFilename, 1000000, 10);
			fileHdlr.setFormatter(new MCSFormatterFile(hasDateFormat));
		    fileHdlr.setLevel(log.getLevel());
		    
		    root.addHandler(fileHdlr);
		}
		catch (final SecurityException exception) {
			LogProvider.anonymous.log(Level.SEVERE, "security exception", exception);
		}
		catch (final IOException exception) {
			LogProvider.anonymous.log(Level.WARNING, "unable to create the FileHandler, path=" + logFilename, exception);
		}
	}
	
	private static String createLogFilename(final Logger log) throws SecurityException, IOException {
		final StringBuilder path = new StringBuilder().append(LogProvider.PRE_PATH.toString()).append(File.separator);

		if (!new File(path.toString()).isDirectory()) 
			throw new IOException("unable to detect path of " + path.toString() + " -> no log file is written");
		else if (!new File(path.toString()).canWrite())
			throw new IOException("unable to write to path " + path.toString() + " -> no log file is written");
		
		final String projectName = fetchProjectName(log);
		if (!new File(path.toString() + projectName).isDirectory()) {
			final boolean success = new File(path.toString() + projectName).mkdirs();
			if (success)
				LogProvider.anonymous.warning("unable to detect path of " + path.toString() + projectName + " -> creating");
			else
				throw new IOException("unable to create path of " + path.toString() + projectName + " -> no file is written");
		}
		
		return path.append(projectName).append(File.separator).append(fetchLogFilename(log)).append("_%g.log").toString();
	}
	
	private static Logger getRoot(final Logger log) {
		Logger root = log;
		while(root.getParent() != null)
			root = log.getParent();
		return root;
	}

	/**
	 * Inspects if the logging instance has a handler of class {@code handler}.
	 * @param log
	 * @param handler
	 * @return
	 */
	private static boolean hasHandler(final Logger log, final Class<? extends Handler> handler) {
		boolean hasHandler = false;
		// Note: unfortunately fetching all handlers from root logger will 
		// indirectly add a new console handler to it.
		final Handler[] handlers = log.getHandlers();
		for(final Handler tmp: handlers)
			if (handler.isInstance(tmp))
				hasHandler = true;
			else if (tmp instanceof ConsoleHandler)
				log.removeHandler(tmp);
		return hasHandler;
	}

	/** Expects the project name right before the class name. If no project
	 * name exists it returns "unknown_package".  
	 * @param clazz
	 * @return
	 */
	static String fetchProjectName(final Logger log) {
		final String[] packages = log.getName().split("\\.");
		if (packages == null || packages.length <= 1)
			return "unknown_package";
		else if (packages.length >= 6 && DESY_ACCELERATOR.contains(packages[3]) && DESY_FACILITY.contains(packages[4]) && KNOWN_HOST.contains(packages[5]))
			return packages[6];
		else if (packages.length >= 5 && DESY_ACCELERATOR.contains(packages[3]) && DESY_FACILITY.contains(packages[4]))
			return packages[5];
		return packages[packages.length - 2];
	}
	
	static String fetchLogFilename(final Logger log) {
		final String[] packages = log.getName().split("\\.");
		if (packages == null || packages.length == 0)
			return "unknown_log";
		return packages[packages.length - 1];
	}
	
	/**
	 * Sets the log file name. Can also include the absolute path to file
	 * including the mount point on the operating system. Generates a pattern 
	 * for rotating the log files at the file name. This is according to the 
	 * Java documentation specified with {@code %g}. 
	 * 
	 * @param logFilename Log file name. If the file name is set to null or empty 
	 * 		it will return immediately. 
	 * @deprecated Never use this method. The file name is filter from the
	 * 		concrete package and main class name on the first call to {@link #getLogger(Class)}.
	 * 		So you should always create one logger in the main class to clarify
	 * 		the file name. This function will return immediately without
	 * 		modifying this class. 
	 */
	@Deprecated
	public final static void setLogFilename(final String logFilename) {
		LogProvider.anonymous.warning("method is deprecated and therefore nothing is set");
	}
	
	public final static class MCSHandlerBridge extends Handler 
    {
		private de.desy.mst.libs.framework.mstapp.logging.Logger getMSTLogger(final LogRecord record) {
			return de.desy.mst.libs.framework.mstapp.logging.Logger.SW;
		}
        
		@Override
		public void publish(final LogRecord record) {
			de.desy.mst.libs.framework.mstapp.logging.Logger log = getMSTLogger(record);
			
			// NOTE: there is no other possibility to request the log level then
			// an if-construct. The switch-case-clause will not work because in 
			// the case part the compiler expects a constant literal expression.
			// However, this is delivered only at runtime. Error message is:
			// "case expressions must be a constant expression"
			//
			// final int level = record.getLevel().intValue();
			// switch(level) {
			// case Level.INFO.getValue(): ...
			// }
			//
			// Also the following construct will not compile:
			//
			// final int INFO = Level.INFO.intValue();
			//...
			// final int level = record.getLevel().intValue();
			// switch(level) {
			// case INFO: ...
			// }
			//
			final StringBuilder builder = new StringBuilder()
				.append(getClassName(record.getLoggerName())).append('.')
				.append(record.getSourceMethodName()).append(' ')
				.append(record.getMessage());
			
			if (record.getLevel() == Level.INFO)
				if (record.getThrown() != null)
					log.info(builder.toString(), record.getThrown());
				else
					log.info(builder.toString());
			else if (record.getLevel() == Level.SEVERE)
				if (record.getThrown() != null)
					log.error(builder.toString(), record.getThrown());
				else
					log.error(builder.toString());
			else if (record.getLevel() == Level.WARNING)
				if (record.getThrown() != null)
					log.warn(builder.toString(), record.getThrown());
				else
					log.warn(builder.toString());
			else if (record.getLevel() == Level.FINE || record.getLevel() == Level.FINER || record.getLevel() == Level.FINEST)
				log.debug(builder.toString());
			else
				if (record.getThrown() != null)
					log.info(builder.toString(), record.getThrown());
				else
					log.info(builder.toString());
		}

		@Override
		public void close() throws SecurityException {
			// no-op
		}

		@Override
		public void flush() {
			// no-op
		}
		
		private String getClassName(final String fullClassName) {
			final int idxOfDot = fullClassName.lastIndexOf('.');
			if (idxOfDot == -1)
				return fullClassName;
			else
				return fullClassName.substring(idxOfDot + 1, fullClassName.length());
		}
	}
	
	private static abstract class AMCSFormatter extends Formatter {
		
		private final DateFormat format; 
		
		AMCSFormatter(final boolean hasDateFormat) {
			this.format = (hasDateFormat) ? DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM, Locale.getDefault()) :  DateFormat.getTimeInstance(DateFormat.MEDIUM, Locale.getDefault());
		}

    	protected String levelToString(final Level level) {
    		if (level.intValue() == Level.OFF.intValue()) 			return "[OFF]    "; // is OFF really needed for the output? According to the specification I would say no!
    		else if (level.intValue() == Level.FINEST.intValue()) 	return "[FINEST] ";
    		else if (level.intValue() == Level.FINER.intValue()) 	return "[FINER]  ";
    		else if (level.intValue() == Level.FINE.intValue()) 	return "[FINE]   ";
    		else if (level.intValue() == Level.CONFIG.intValue()) 	return "[CONFIG] ";
    		else if (level.intValue() == Level.INFO.intValue()) 	return "[INFO]   ";
    		else if (level.intValue() == Level.WARNING.intValue())	return "[WARN]   ";
    		else if (level.intValue() == Level.SEVERE.intValue())	return "[SEVERE] ";
    		else if (level.intValue() == Level.ALL.intValue())		return "[ALL]    ";
    		else 													return "[UNKNWON]";
    	}

    	protected StringBuilder getMessage(final LogRecord record) {
    	  	final StringBuilder builder = new StringBuilder();
			builder.append(levelToString(record.getLevel())).append(' ')
				.append(getDateFormat().format(new Date(record.getMillis()))).append(", ")
				.append(record.getLoggerName().substring(record.getLoggerName().lastIndexOf('.') + 1)).append('.')
				.append(record.getSourceMethodName()).append(": ")
				.append(record.getMessage());
			
			if (record.getThrown() == null)
    			return builder;
    		
			builder.append(record.getThrown().toString()).append('\n');
    		for(final StackTraceElement trace: record.getThrown().getStackTrace())
    	  		builder.append(trace.getClassName()).append('.').append(trace.getMethodName()).append(" [line ").append(trace.getLineNumber()).append("]").append("\n");
    		return builder;
    	}
    	
    	protected DateFormat getDateFormat() { return this.format; }
	}
	
	private static class MCSFormatterConsole extends AMCSFormatter {
		
		public MCSFormatterConsole(final boolean hasDateFormat) {
			super(hasDateFormat);
		}
		
    	@Override public String format(final LogRecord record) {
            final StringBuilder builder = getMessage(record);
            
            if      (record.getLevel() == Level.SEVERE)
                de.desy.mst.libs.framework.mstapp.logging.Logger.SW.error(builder.toString());
            else if (record.getLevel() == Level.WARNING)
                de.desy.mst.libs.framework.mstapp.logging.Logger.SW.warn(builder.toString());
            else if  (record.getLevel() == Level.FINE   || record.getLevel() == Level.FINER || record.getLevel() == Level.FINEST)
                de.desy.mst.libs.framework.mstapp.logging.Logger.SW.debug(builder.toString());
            else if (record.getLevel() == Level.CONFIG || record.getLevel() == Level.INFO  || record.getLevel() == Level.OFF)
                de.desy.mst.libs.framework.mstapp.logging.Logger.SW.debug(builder.toString());
            else
                de.desy.mst.libs.framework.mstapp.logging.Logger.SW.fatal(builder.toString());
               
    		return getMessage(record).append('\n').toString();
    	}
	}
	
	private static class MCSFormatterFile extends AMCSFormatter {
		
		public MCSFormatterFile(final boolean hasDateFormat) {
			super(hasDateFormat);
		}
		
    	@Override public String format(final LogRecord record) {
    		return getMessage(record).append("\r\n").toString();
    	}
	}
}
/* end of file */